      PROGRAM KBMAIN
* ------------------------------------------------------------------------
* The   following routines have   been  written to  solve for  equilibrium
* solutions of the two-agent  OLG economy  studied  in Kurz and  Beltratti
* "The  Equity Premium   is no  Puzzle"   in Kurz,  M.  (ed.), "Endogenous
* Economic  Fluctuations:     Studies   in   the   Theory      of Rational
* Belief". Studies in Economic Theory. No.  6, Springer-Verlag, Berlin and
* New   York, (1997).   The first   order   conditions together  with  the
* equilibrium conditions specify a system of non-linear equations which is
* here solved using  the software package TENSOLVE,  a suite of  FORTRAN77
* subroutines, written by Ali Bouaricha,  Argonne National Laboratory, and
* Robert B. Schnabel, University  of Colorado. For  a complete overview of
* the software  package TENSOLVE see the  postscript file supplied here by
* along with  the  FORTRAN77 code.   Either Gauss-Newton  method or a  new
* class  of  methods called  tensor methods can  be  choosen  to solve the
* system of  non-linear equations. In almost  all  the applications of the
* following routines we  used   the Gauss-Newton method.  The   subroutine
* DGECO and  DGEFA have been taken  from LINPACK, version  dated 08/14/78.
* Cleve Moler, University of New Mexico, Argonne National Lab. 
*
* 
* List of routines, subroutines and functions in the present file:
*
*  KBMAIN...the main routine which is the driver for the general 
*           equilibrium cases to solve.
*  BELIEFS..a subroutine which computes the Markov stationary measure 
*           Gamma in the economy and the Markov matrices representing 
*           agent's beliefs in their optimization problem.
*  SYSTEM...a subroutine containing the system of equations to solve.
*  UP1......a double precision function which computes the marginal 
*           utility of consumption of agent 1 when young.
*  UP2......a double precision function which computes the marginal 
*           utility of consumption of agent 1 when old.
*  UP1S.....a double precision function which computes the marginal 
*           utility of consumption of agent 2 when young.
*  UP2S.....a double precision function which computes the marginal 
*           utility of consumption of agent 2 when old.
*  COUNT....an integer function which counts the number of values for 
*           each input parameter file.
*  EXT......a function which generates an alphanumeric extension to
*           identify the output files of each case.
*  GAUSS....a subroutine which solves linear systems of equations by 
*           Gaussian elimination. It is used to compute the stationary 
*           distribution of the Markov matrix Gamma which governs the 
*           economy.
*  DGECO....a subroutine to estimate the condition of the Jacobian 
*           matrix of the system of equations.
*  DGEFA....a subroutine called by DGECO which factors a double
*           precision matrix by Gaussian elimination.
*
*  OUTPUT
*           Three output files are generated.
*           If ROOT='NAME01' then
*           NAME01 is a summary file which contains computational 
*           information and statistics of all cases submitted for 
*           solution.
*           NAME01nnn.RES is a detailed file of results for each case.
*           NAME01nnn.DAT is an ascii file containing the Newton solution
*           vector for each case which can be used as initial guess for
*           other cases.
*           'nnn' is the alphanumeric extension given by the Function EXT
*           to sequentially identify each case either in a sequence or in 
*           a multi-dimensional grid to solve.
* ----------------------------------------------------------------------

* ----------------------------------------------------------------------
*                                                                      
* memory allocation for Rational Belief structure and 		
* exogenous process                                                    
*                                                                      
*    N     = number of states                                          
*                                                                      
* ----------------------------------------------------------------------
      INTEGER          N
      PARAMETER        (N=8)
      DOUBLE PRECISION QQ1(N,N),QQ2(N,N),GM(N,N),PI(N),DIV(N),
     $                 Q1(N,N),Q2(N,N),F1(N,N),F2(N,N),G1(N,N),G2(N,N)
      LOGICAL          SINGUL,CHBL
* ----------------------------------------------------------------------
*
* parameters input by Matlab or other external routine               
*
* ----------------------------------------------------------------------
      DOUBLE PRECISION DH,DL,R,RS,G,GS,W,WS,A1,A2,PHI,L(N),M(N),
     $                 AVEC(4),BVEC(4)
* ----------------------------------------------------------------------
*                                                                      
* input/output files                                                   
*                                                                      
* ----------------------------------------------------------------------
      CHARACTER*6      ROOT*6,DNAME*13,START*13,FNAME*13,NAME*5,EXT*3,
     $                 GRID(15)
      INTEGER          COUNT,CCOUNT,SIZE(15),H(15)
* ----------------------------------------------------------------------
*                                                                      
* set the memory allocation for the non-linear problem to solve    
*                                                                      
*    NL     = number of unknowns                                       
*    ML     = number of equations                                      
*                                                                      
* ----------------------------------------------------------------------
      INTEGER          NL,ML
      PARAMETER        (NL=32, ML=32)

* -------Starting guess 

      DOUBLE PRECISION SQS,SQB,STH,SBN
* ----------------------------------------------------------------------
*                                                                      
* set the memory allocation for the TENSOLVE procedure and DGECO 
* subroutine
*    
*    MAXM>=ML+NL+2 
*    MAXN>=NL+2 
*    PMAX>=NINT(SQRT(NL)) 
*    LIN >=3       
*    LUNC>=2*NINT(SQRT(NL))+4
*    LNEM>=NL+2*NINT(DSQRT(NL))+11 
*    LNEN>=2*NINT(DSQRT(NL))+9
*
* ----------------------------------------------------------------------
      INTEGER          MAXM,MAXN,PMAX,LIN,LUNC,LNEM,LNEN
      PARAMETER        (MAXM = 66, MAXN = 34, PMAX = 6)
      PARAMETER        (LIN = 3, LUNC = 16, LNEM = 55, LNEN = 21)
      INTEGER          ITNLIM,JACFLG,METHOD,GLOBAL,IPR,MSG,TERMCD,
     $                 IWRKN(MAXN,LIN),LIMIT,METH,MESG,GLOB,IPVT(NL)
      DOUBLE PRECISION X0(NL),WRKUNC(PMAX,LUNC),WRKNEM(MAXM,LNEM),
     $                 WRKNEN(MAXN,LNEN),TYPX(NL),TYPF(ML),Z(NL),
     $                 XP(NL),FP(ML),GP(NL),X0OLD(NL),NRMFXP,FHAT(ML),
     $                 AJA(ML,NL),EPSM,RCOND,GRADTL,STEPTL,FTOL,
     $                 STEPMX,DLT,STPMX,STPTL,FCTL,GRTL,RADIUS
      EXTERNAL         TSDUMJ,SYSTEM
* ----------------------------------------------------------------------
*                                                                      
* portfolio, prices, consumption and statistics                       
*                                                                      
* ----------------------------------------------------------------------

      DOUBLE PRECISION QS(N),TH(N),THS(N),BN(N),QB(N),BNS(N),
     $                 X1(N),X1S(N),X2(N,N),X2S(N,N),QSBAR,QBBAR,
     $                 QSVAR,QBVAR,RET(N,N),RETBR,RETVR,RF(N),RFBAR,
     $                 BTRAD,STRAD,RFVAR
* ----------------------------------------------------------------------
*                                                                      
* other parameters                                                          
*                                                                      
* ----------------------------------------------------------------------
      LOGICAL          ON,CHECK,CHECK2
      INTEGER          I,J
      DOUBLE PRECISION ZERO,ONE,DSQRT,DABS,DPMEPS
      INTRINSIC        DSQRT,DABS,MAX
      PARAMETER        (ZERO=0.0D+00, ONE=1.0D+00)
* ----------------------------------------------------------------------
*                                                                      
* common area                                                          
*                                                                      
* ----------------------------------------------------------------------
      COMMON /PARAM/   R,RS,G,GS,W,WS 
      COMMON /VECTOR/  DIV 
      COMMON /MATRIX/  Q1,Q2
* ----------------------------------------------------------------------
* ------------------------------START-----------------------------------

* ----------------------------------------------------------------------
*                                                                      
* Inputs and parameters specification.                                 
*                                                                      
* ----------------------------------------------------------------------
      EPSM = DPMEPS()
      ON = .FALSE.
      CCOUNT = 0
      DO 1 I = 1, 15
         H(I) = 0
 1    CONTINUE

* --------Defining output labels

      GRID(1) = 'BETA1'
      GRID(2) = 'BETA2'
      GRID(3) = 'GAMMA1'
      GRID(4) = 'GAMMA2'
      GRID(5) = 'ALPHA1'
      GRID(6) = 'ALPHA2'
      GRID(7) = 'W1'
      GRID(8) = 'W2'
      GRID(9) = 'PHI'
      GRID(10) = 'H-DIV'
      GRID(11) = 'L-DIV'
      GRID(12) = 'LAMBDA'
      GRID(13) = 'MU'
      GRID(14) = 'A-VEC'
      GRID(15) = 'B-VEC'

* --------User defined parameters for TENSOLVE

      PRINT *,'MESSAGE CODE - MSG '
      READ *, MESG
      PRINT *,'ITERATION LIMIT '
      READ *, LIMIT
      PRINT *,'MAXIMUM STEP - STEPMX '
      READ *, STPMX
      PRINT *,'FUNCTION TOLERANCE '
      READ *, FCTL
      PRINT *,'GRADIENT TOLERANCE '
      READ *, GRTL
      PRINT *,'STEP TOLERANCE '
      READ *, STPTL
      PRINT *,'TRUST REGION RADIUS'
      READ *, RADIUS
 5    PRINT *,'1 -> TENSOR METHOD  0 -> NEWTON METHOD'
      READ *, METH
      IF ((METH .LT. 0) .OR. (METH .GT. 1)) GOTO 5
 7    PRINT *,'1 -> TRUST REGION   0 -> LINE SEARCH'
      READ *, GLOB
      IF ((GLOB .LT. 0) .OR. (GLOB .GT. 1)) GOTO 7
      PRINT *, 'ENTER THE 6 CHARACTER ROOT TO USE FOR DATA, SUMMARY AND 
     +OUTPUT FILENAMES [ALL CAPS]: '
      READ *, ROOT
      OPEN (UNIT = 27, FILE = ROOT,  STATUS = 'NEW')
      WRITE (27,'(/1X, A)')'THE PROGRAM  HAS BEEN PERFORMED SETTING ALL
     +THE PARAMETERS  EMPLOYED'
      WRITE (27,'(1X, A)') 'WITH THE QUADRATIC TENSOR METHOD TO THE DEFA 
     +ULT VALUES (ref. manual)'
      WRITE (27,'(1X, A)') 'EXCEPT FOR:'
      WRITE (27,'(3X, A, I5)')'MSG                 = ',MESG
      WRITE (27,'(3X, A, I5)')'ITERATION LIMIT     = ',LIMIT
      WRITE (27,'(3X, A, E18.10)') 'FUNCTION TOLERANCE  = ',FCTL
      WRITE (27,'(3X, A, E18.10)') 'GRADIENT TOLERANCE  = ',GRTL
      WRITE (27,'(3X, A, E18.10)') 'STEP     TOLERANCE  = ',STPTL
      WRITE (27,'(3X, A, E18.10)') 'MAXIMUM STEP SIZES  = ',STPMX
      WRITE (27,'(3X, A, E18.10)') 'TRUST REG RADIUS    = ',RADIUS
      IF (METH .EQ. 0) THEN
         WRITE (27,'(3X, A)')'METHOD USED         =  NEWTON'
      ELSE
         WRITE (27,'(3X, A)')'METHOD USED         =  TENSOR'         
      END IF
      IF (GLOB .EQ. 0) THEN
         WRITE (27,'(3X, A)')'GLOBAL STRATEGY     =  LINE SEARCH'
      ELSE
         WRITE (27,'(3X, A)')'GLOBAL STRATEGY     =  TRUST REGION'         
      END IF

* --------Input of starting values for the System of Euler Equations to solve

 10   CH = 0
      PRINT *, 'STARTING VALUE CHOICES'
      PRINT *, '1) NEW STARTING VALUES'
      PRINT *, '2) OLD DATA FILE'
      READ *, CH
      IF ((CH .LT. 1) .OR. (CH .GT. 2)) GOTO 10
      IF (CH .EQ. 1) THEN
         PRINT *, 'QS - UNIFORM INITIAL GUESS OF COMMON STOCK PRICE:'
         READ *, SQS
         PRINT *, 'QB - UNIFORM INITIAL GUESS OF BOND PRICE:'
         READ *, SQB
         PRINT *, 'THETA - UNIFORM INITIAL GUESS OF STOCK HOLDINGS:'
         READ *, STH
         PRINT *, 'B - UNIFORM INITIAL GUESS OF BOND HOLDINGS:'
         READ *, SBN
      ELSE IF (CH .EQ. 2) THEN
         PRINT *, 'ENTER THE NAME OF THE FILE OF STARTING VALUES:'
         READ *, START
      END IF

* --------Counting the number of values for each parameter to define size 
*         and dimension of the multidimensional grid of cases to solve

      NAME = 'PAR1'
      SIZE(1) = COUNT(NAME)
      NAME = 'PAR2'
      SIZE(2) = COUNT(NAME)
      NAME = 'PAR3'
      SIZE(3) = COUNT(NAME)
      NAME = 'PAR4'
      SIZE(4) = COUNT(NAME)
      NAME = 'PAR5'
      SIZE(5) = COUNT(NAME)
      NAME = 'PAR6'
      SIZE(6) = COUNT(NAME)
      NAME = 'PAR7'
      SIZE(7) = COUNT(NAME)
      NAME = 'PAR8'
      SIZE(8) = COUNT(NAME)
      NAME = 'PAR9'
      SIZE(9) = COUNT(NAME)
      NAME = 'PAR10'
      SIZE(10) = COUNT(NAME) 
      NAME = 'PAR11'
      SIZE(11) = COUNT(NAME) 
      NAME = 'PAR12'
      SIZE(12) = COUNT(NAME) / N
      NAME = 'PAR13'
      SIZE(13) = COUNT(NAME) / N
      NAME = 'PAR14'
      SIZE(14) = COUNT(NAME) / 4
      NAME = 'PAR15'
      SIZE(15) = COUNT(NAME) / 4

* --------Opening and reading the input parameters files

      OPEN (UNIT = 31, FILE = 'PAR1', STATUS = 'OLD', ERR = 1450)
      OPEN (UNIT = 32, FILE = 'PAR2', STATUS = 'OLD', ERR = 1450)
      OPEN (UNIT = 33, FILE = 'PAR3', STATUS = 'OLD', ERR = 1450)
      OPEN (UNIT = 34, FILE = 'PAR4', STATUS = 'OLD', ERR = 1450)
      OPEN (UNIT = 35, FILE = 'PAR5', STATUS = 'OLD', ERR = 1450)
      OPEN (UNIT = 36, FILE = 'PAR6', STATUS = 'OLD', ERR = 1450)
      OPEN (UNIT = 37, FILE = 'PAR7', STATUS = 'OLD', ERR = 1450)
      OPEN (UNIT = 38, FILE = 'PAR8', STATUS = 'OLD', ERR = 1450)
      OPEN (UNIT = 39, FILE = 'PAR9', STATUS = 'OLD', ERR = 1450)
      OPEN (UNIT = 40, FILE = 'PAR10', STATUS = 'OLD', ERR = 1450)
      OPEN (UNIT = 41, FILE = 'PAR11', STATUS = 'OLD', ERR = 1450)
      OPEN (UNIT = 42, FILE = 'PAR12', STATUS = 'OLD', ERR = 1450)
      OPEN (UNIT = 43, FILE = 'PAR13', STATUS = 'OLD', ERR = 1450)
      OPEN (UNIT = 44, FILE = 'PAR14', STATUS = 'OLD', ERR = 1450)
      OPEN (UNIT = 45, FILE = 'PAR15', STATUS = 'OLD', ERR = 1450)
 30   READ (31, *, END = 1430) R
      H(1) = H(1) +1
 40   READ (32, *, END = 1420) RS
      H(2) = H(2) + 1
 50   READ (33, *, END = 1410) G
      H(3) = H(3) + 1
 60   READ (34, *, END = 1400) GS
      H(4) = H(4) + 1
 70   READ (35, *, END = 1390) A1
      H(5) = H(5) + 1
 80   READ (36, *, END = 1380) A2
      H(6) = H(6) + 1
 110  READ (37, *, END = 1350) W
      H(7) = H(7) + 1
 120  READ (38, *, END = 1340) WS
      H(8) = H(8) + 1
 130  READ (39, *, END = 1330) PHI
      H(9) = H(9) + 1
 140  READ (40, *, END = 1320) DH
      H(10) = H(10) + 1
 150  READ (41, *, END = 1310) DL
      H(11) = H(11) + 1
 160  READ (42, *, END = 1300) L
      H(12) = H(12) + 1
 170  READ (43, *, END = 1290) M
      H(13) = H(13) + 1
 180  READ (44, *, END = 1280) AVEC
      H(14) = H(14) + 1
 190  READ (45, *, END = 1270) BVEC
      H(15) = H(15) + 1


* ------- The Rational Belief Structure

      CALL BELIEFS(A1,A2,PHI,L,M,AVEC,BVEC,QQ1,QQ2,F1,F2,G1,G2,GM,PI,
     +             SINGUL,CHBL)
      IF (SINGUL) THEN
         PRINT *, 'MATRIX GAMMA IS (NEARLY) SINGULAR'
         WRITE (27,'(//1X, A/)') 'MATRIX GAMMA IS (NEARLY) SINGULAR -- 
     $PROGRAM STOPPED'
         GOTO 1445
      END IF
      IF (CHBL) THEN
         PRINT *, 'THE RATIONAL BELIEF STRUCTURE IS NOT CORRECT'
         WRITE (27,'(//1X, A/)') 'THE RATIONAL BELIEF STRUCTURE IS NOT C
     $ORRECT -- PROGRAM STOPPED -- CHECK PARAMETERS AND RERUN ROUTINE'
         GOTO 1445
      END IF
	DO 210 I = 1, N
	   DO 200 J = 1, N
		  Q1(I,J)=QQ1(I,J)
		  Q2(I,J)=QQ2(I,J)
 200     CONTINUE
 210  CONTINUE

* -------Constructing the dividend state

      DO 287 I = 1, 4
         DIV(I) = DH
         DIV(I+4) = DL
 287  CONTINUE
      
*----------------------------------------------------------------------*
*                                                                      *
* Counting and opening the files for each case in the grid             *
*                                                                      *
*----------------------------------------------------------------------*

      CCOUNT = CCOUNT + 1 
      FNAME = ROOT//EXT(CCOUNT)//'.RES'
      DNAME = ROOT//EXT(CCOUNT)//'.DAT'
      OPEN (UNIT = 25, FILE = FNAME, STATUS = 'NEW')
      OPEN (UNIT = 26, FILE = DNAME, STATUS = 'NEW')

*----------------------------------------------------------------------*
*                                                                      *
* Solving the model using either Newton-Gauss method or a quadratic    *
* tensor method                                                        *
*                                                                      *
*----------------------------------------------------------------------*
      IF (CCOUNT .EQ. 1) THEN
         IF (CH .EQ. 1) THEN
            DO 990 I = 1, N
               X0(I) = SQS
               X0(I+N) = SQB
               X0(I+2*N) = STH
               X0(I+3*N) = SBN
 990        CONTINUE
         ELSE IF (CH .EQ. 2) THEN
            OPEN (UNIT = 29, FILE = START, STATUS = 'OLD')
            READ (29,*) X0
            CLOSE (29)
         END IF
      ELSE
         DO 995 I = 1, NL
            X0(I) = XP(I)
 995     CONTINUE
      END IF
      DO 997 I = 1, NL
         X0OLD(I) = X0(I)
 997  CONTINUE
      PRINT *, 'COMPUTING CASE NUMBER: ',CCOUNT
      CALL TSDFLT(ML,NL,ITNLIM,JACFLG,GRADTL,STEPTL,FTOL,METHOD,
     +            GLOBAL,STEPMX,DLT,TYPX,TYPF,IPR,MSG)
      ITNLIM = LIMIT
      MSG    = MESG
      GRADTL = GRTL
      STEPTL = STPTL
      FTOL   = FCTL
      METHOD = METH
      GLOBAL = GLOB
      STEPMX = STPMX
      DLT    = RADIUS
      CALL TSNECI(MAXM,MAXN,PMAX,X0,ML,NL,TYPX,TYPF,ITNLIM,JACFLG, 
     +            GRADTL,STEPTL,FTOL,METHOD,GLOBAL,STEPMX,DLT,IPR, 
     +            WRKUNC,LUNC,WRKNEM,LNEM,WRKNEN,LNEN,IWRKN,LIN, 
     +            TSDUMJ,SYSTEM,MSG,XP,FP,GP,TERMCD)
      DO 1000 I = 1, N
         QS(I) = XP(I)
         QB(I) = XP(I+N)
         TH(I) = XP(I+2*N)
         BN(I) = XP(I+3*N)
 1000 CONTINUE

* --------Equilibrium conditions for stocks and bonds

      DO 1010 I = 1, N
         THS(I) = ONE - TH(I)
         BNS(I) = - BN(I)
 1010 CONTINUE

* --------Consumption of the two agents when young and when old.

      DO 1020 I = 1, N
         X1(I)=W-QS(I)*TH(I)-QB(I)*BN(I)
         X1S(I)=WS-QS(I)*THS(I)-QB(I)*BNS(I)

* --------Check for negative consumption when young

         IF ((X1(I) .LT. ZERO) .OR. (X1S(I) .LT. ZERO)) THEN
            CHECK = .TRUE.
         END IF 
 1020 CONTINUE

      DO 1040 I = 1, N
         DO 1030 J = 1, N
            X2(I,J)=TH(I)*(QS(J)+ONE)*DIV(J)+BN(I)
            X2S(I,J)=THS(I)*(QS(J)+ONE)*DIV(J)+BNS(I)

* --------Check for negative consumption when old

            IF ((X2(I,J) .LT. ZERO) .OR. (X2S(I,J) .LT. ZERO)) THEN
               CHECK2 = .TRUE.
            END IF 
 1030    CONTINUE
 1040 CONTINUE   

* --------Risk-free and risky rates of return

      DO 1041 I = 1, N
         RF(I)=(ONE-QB(I))/QB(I)
         DO 1042 J = 1, N
            RET(I,J) = ((QS(J)+ONE)*DIV(J)-QS(I))/QS(I)
 1042    CONTINUE
 1041 CONTINUE

*----------------------------------------------------------------------*
*                                                                      *
* Computing the statistics                                             *
*                                                                      *
*----------------------------------------------------------------------*

      QSBAR = ZERO
      QBBAR = ZERO
      RETBR = ZERO
      RFBAR = ZERO
      BTRAD = ZERO
      STRAD = ZERO
      DO 1050 I = 1, N
         QSBAR = QSBAR + QS(I) * PI(I) 
         QBBAR = QBBAR + QB(I) * PI(I)
         RFBAR = RFBAR + RF(I) * PI(I)
         BTRAD = BTRAD + DABS(QB(I)*BN(I)) * PI(I)
         STRAD = STRAD + QS(I)*MAX(TH(I),THS(I)) * PI(I)
         DO 1049 J = 1, N
            RETBR = RETBR + RET(I,J) * PI(I) * GM(I,J)
 1049    CONTINUE
 1050 CONTINUE
      QSVAR = ZERO
      QBVAR = ZERO
      RETVR = ZERO
      RFVAR = ZERO
      DO 1060 I = 1, N
         QSVAR = QSVAR + ((QS(I) - QSBAR) ** 2) * PI(I)
         QBVAR = QBVAR + ((QB(I) - QBBAR) ** 2) * PI(I)
         RFVAR = RFVAR + ((RF(I) - RFBAR) ** 2) * PI(I)
         DO 1059 J = 1, N
            RETVR = RETVR + ((RET(I,J) - RETBR) ** 2) * PI(I) * GM(I,J)
 1059    CONTINUE
 1060 CONTINUE
      QSVAR=DSQRT(QSVAR)
      QBVAR=DSQRT(QBVAR)
      RETVR=DSQRT(RETVR)
      RFVAR=DSQRT(RFVAR)
      BTRAD=BTRAD/(W+WS+ONE)
      STRAD=STRAD/(W+WS+ONE)

*----------------------------------------------------------------------*
*                                                                      *
* Computing the Jacobian and the condition number                      *
*                                                                      *
*----------------------------------------------------------------------*

      CALL TSFDFJ(XP,FP,ML,ML,NL,EPSM,SYSTEM,FHAT,AJA)
*

      CALL DGECO(AJA,ML,NL,IPVT,RCOND,Z)

*----------------------------------------------------------------------*
*                                                                      *
* Printing the output files                                            *
*                                                                      *
*----------------------------------------------------------------------*

      WRITE (26, *) XP
 1070 FORMAT (//1X, A/)
 1080 FORMAT (/T5, A/)
 1081 FORMAT (T5, A/)
 1090 FORMAT (/1X, A6, ' = ', F7.3, 3X, A6, ' = ', F7.3, 3X, A6, ' = ',
     +       F7.3, 3X, A6, ' = ', F7.3)
 1100 FORMAT (/1X, A6, ' = ', 8F8.4)
 1110 FORMAT (/1X, A6, ' = ', 4F8.4)
 1120 FORMAT (1X, 8F14.8)
 1130 FORMAT (/1X, A70, ' = ', F14.8)
 1135 FORMAT (1X, A6, ' = ', I3)
 1137 FORMAT (/T5, A, ' = ',I2/)
 1138 FORMAT (/T5, A, E18.10/)
 1139 FORMAT (I3, 1X, E18.10, 3X, I3, 1X, E18.10, 3X, I3, 1X, E18.10, 3X
     +, I3, 1X, E18.10)
      WRITE (27, 1070) 'CASE FILENAME = '//FNAME
      WRITE (25, 1070) 'KURZ-BELTRATTI'
      WRITE (25, 1080) 'PARAMETER SPECIFICATION'
      WRITE (25, 1090) GRID(1), R, GRID(2), RS, GRID(3), G, GRID(4), GS
      WRITE (25, 1090) GRID(5), A1, GRID(6), A2, GRID(7), W,GRID(8), WS
      WRITE (25, 1090) GRID(9), PHI
      WRITE (25, 1100) 'DIV   ',DIV
      WRITE (25, 1100) GRID(12),L
      WRITE (25, 1100) GRID(13),M
      WRITE (25, 1110) GRID(14),AVEC
      WRITE (25, 1110) GRID(15),BVEC
      WRITE (25, 1070) 'COORDINATES OF THE EXPERIMENT IN THE GIVEN GRID 
     + OF PARAMETERS'
      WRITE (27, 1070) 'COORDINATES OF THE EXPERIMENT IN THE GIVEN GRID 
     + OF PARAMETERS'
      DO 1140 I = 1, 15
         IF (SIZE(I) .GT. 1) THEN
            ON = .TRUE.
            WRITE (25, 1135) GRID(I),H(I)
            WRITE (27, 1135) GRID(I),H(I)
         END IF
 1140 CONTINUE
      IF (.NOT. ON) THEN
         WRITE (25, 1080) 'SINGLE EXPERIMENT (NO GRID)'
         WRITE (27, 1080) 'SINGLE EXPERIMENT (NO GRID)'
      END IF
      WRITE (25, 1070) 'THE STOCHASTIC STRUCTURE'
      WRITE (25, 1080) 'MATRIX GAMMA'
      WRITE (25, 1120),((GM(I,J), J = 1, N), I = 1, N)
      WRITE (25, 1080) 'STATIONARY DISTRIBUTION '
      WRITE (25, 1120) PI
      WRITE (25, 1080) 'MATRIX F1'
      WRITE (25, 1120),((F1(I,J), J = 1, N), I = 1, N)
      WRITE (25, 1080) 'MATRIX F2'
      WRITE (25, 1120),((F2(I,J), J = 1, N), I = 1, N)
      WRITE (25, 1080) 'MATRIX G1'
      WRITE (25, 1120),((G1(I,J), J = 1, N), I = 1, N)
      WRITE (25, 1080) 'MATRIX G2'
      WRITE (25, 1120),((G2(I,J), J = 1, N), I = 1, N)
      IF (CCOUNT .EQ. 1) THEN
         IF (CH .EQ. 2) THEN
            WRITE (25, 1070) 'STARTING VALUES FOR SIMULATION = '//START
         ELSE
            WRITE (25, 1070) 'STARTING VALUES FOR SIMULATION'
         ENDIF
      ELSE
         WRITE (25, 1070) 'STARTING VALUES FOR SIMULATION = '//ROOT//EXT
     +(CCOUNT-1)//'.DAT'
      ENDIF
      WRITE (25, 1080) 'QS - PRICE OF COMMON STOCK:'
      WRITE (25, 1120),(X0OLD(I),I = 1, 8)
      WRITE (25, 1080) 'QB - PRICE OF BOND:'
      WRITE (25, 1120),(X0OLD(I),I = 9, 16)
      WRITE (25, 1080) 'THETA - STOCK HOLDINGS:'
      WRITE (25, 1120),(X0OLD(I),I = 17, 24)
      WRITE (25, 1080) 'B - BOND HOLDINGS:'
      WRITE (25, 1120),(X0OLD(I),I = 25, 32)
      WRITE (25, 1070) 'SIMULATION RESULTS'
      WRITE (25, 1080) 'CONSUMPTION OF THE YOUNG AGENT 1'
      WRITE (25, 1120) X1
      WRITE (25, 1080) 'CONSUMPTION OF THE OLD AGENT 1'
      WRITE (25, 1120),((X2(I,J), J = 1, N), I = 1, N)
      WRITE (25, 1080) 'CONSUMPTION OF THE YOUNG AGENT 2'
      WRITE (25, 1120) X1S
      WRITE (25, 1080) 'CONSUMPTION OF THE OLD AGENT 2'
      WRITE (25, 1120),((X2S(I,J), J = 1, N), I = 1, N)
      WRITE (25, 1080) 'STOCK PURCHASES OF AGENT 1'
      WRITE (25, 1120) TH
      WRITE (25, 1080) 'STOCK PURCHASES OF AGENT 2'
      WRITE (25, 1120) THS
      WRITE (25, 1080) 'PRICES OF COMMON STOCK'
      WRITE (25, 1120) QS
      WRITE (25, 1080) 'RISKY RATES OF RETURN'
      WRITE (25, 1120),((RET(I,J), J = 1, N), I = 1, N)
      WRITE (25, 1080) 'BOND HOLDINGS OF AGENT 1'
      WRITE (25, 1120) BN
      WRITE (25, 1080) 'BOND HOLDINGS OF AGENT 2'
      WRITE (25, 1120) BNS
      WRITE (25, 1080) 'PRICES OF BONDS'
      WRITE (25, 1120) QB
      WRITE (25, 1070) 'STATISTICS'
      WRITE (25, 1130) 'MEAN PRICE OF COMMON STOCK',QSBAR
      WRITE (25, 1130) 'STD. DEVIATION OF PRICES OF COMMON STOCK',QSVAR
      WRITE (25, 1130) 'MEAN PRICE OF BOND',QBBAR    
      WRITE (25, 1130) 'STD. DEVIATION OF PRICES OF BOND',QBVAR        
      WRITE (25, 1130) 'MEAN RISKY RATE OF RETURN',RETBR
      WRITE (25, 1130) 'STD. DEVIATION OF RISKY RATES OF RETURN',RETVR
      WRITE (25, 1130) 'MEAN RISK FREE RATE OF RETURN',RFBAR
      WRITE (25, 1130) 'STD. DEVIATION OF RISK FREE RATES OF RETURN'
     $,RFVAR
      WRITE (25, 1130) 'EQUITY PREMIUM',RETBR-RFBAR
      WRITE (25, 1130) 'MEAN BOND TRADING VOLUME',BTRAD           
      WRITE (25, 1130) 'MEAN STOCK TRADING VOLUME',STRAD 
      WRITE (27, 1070) 'STATISTICS'
      WRITE (27, 1130) 'MEAN PRICE OF COMMON STOCK',QSBAR
      WRITE (27, 1130) 'STD. DEVIATION OF PRICES OF COMMON STOCK',QSVAR
      WRITE (27, 1130) 'MEAN PRICE OF BOND',QBBAR    
      WRITE (27, 1130) 'STD. DEVIATION OF PRICES OF BOND',QBVAR        
      WRITE (27, 1130) 'MEAN RISKY RATE OF RETURN',RETBR
      WRITE (27, 1130) 'STD. DEVIATION OF RISKY RATES OF RETURN',RETVR
      WRITE (27, 1130) 'MEAN RISK FREE RATE OF RETURN',RFBAR
      WRITE (27, 1130) 'STD. DEVIATION OF RISK FREE RATES OF RETURN'
     $,RFVAR
      WRITE (27, 1130) 'EQUITY PREMIUM',RETBR-RFBAR
      WRITE (27, 1130) 'MEAN BOND TRADING VOLUME',BTRAD           
      WRITE (27, 1130) 'MEAN STOCK TRADING VOLUME',STRAD 
      WRITE (27, 1070) 'NOTES'
      IF (TERMCD .EQ. 0) THEN
         WRITE (27, 1080) 'NO TERMINATION CRITERION SATISFIED'
      ELSE IF (TERMCD .EQ. 1) THEN
         WRITE (27, 1080) 'SIMULATION TERMINATED SUCCESSFULLY - FUNCTION
     + VALUE CLOSE TO ZERO'
      ELSE IF (TERMCD .EQ. 2) THEN
         WRITE (27, 1080) 'GRADIENT CLOSE TO ZERO BUT FUNCTION TOLERANCE
     + NOT REACHED'
      ELSE IF (TERMCD .EQ. 3) THEN
         WRITE (27, 1080) 'SUCCESSIVE ITERATES WITHIN STEP TOLERANCE. TH
     +E CURRENT ITERATE MAY BE A SOLUTION,'
         WRITE (27, 1081) 'OR THE ALGORITHM IS MAKING VERY SLOW PROGRESS
     + AND IS NOT NEAR A SOLUTION'
      ELSE IF (TERMCD .EQ. 4) THEN
         WRITE (27, 1080) 'LAST GLOBAL STEP FAILED TO LOCATE A POINT LOW
     +ER THAN XP. IT IS LIKELY THAT'
         WRITE (27, 1081) 'EITHER XP IS AN APPROXIMATE SOLUTION OF THE P
     +ROBLEM OR STEPTL IS TOO LARGE.'
      ELSE IF (TERMCD .EQ. 5) THEN
         WRITE (27, 1080) 'ITERATION LIMIT EXCEEDED - ALGORITHM FAILED'
      ELSE IF (TERMCD .EQ. 6) THEN
         WRITE (27, 1080) 'MAXIMUM STEP SIZE EXCEEDED 5 CONSECUTIVE TIME
     +S. EITHER THE FUNCTION IS'
         WRITE (27, 1081) 'UNBOUNDED BELOW AND BECOMES ASYMPTOTIC TO A F
     +INITE VALUE FROM ABOVE IN'
         WRITE (27, 1081) 'SOME DIRECTION OR STEPMX IS TOO SMALL.'
      END IF
      IF (CHECK) THEN 
         WRITE (27, 1080) 'NEGATIVE CONSUMPTION WHEN YOUNG'
      END IF
      IF (CHECK2) THEN
         WRITE (27, 1080) 'NEGATIVE CONSUMPTION WHEN OLD'
      END IF
      WRITE (27, 1137) 'MESSAGE CODE',MSG
      NRMFXP = ZERO
      DO 1190 I = 1, ML
         NRMFXP = NRMFXP + FP(I)**2
 1190 CONTINUE
      WRITE (27, 1138) 'NORM OF VECTOR OF FUNCTION VALUES F(XP) =       
     +               ',(NRMFXP/2D0)
      WRITE (27, 1138) 'RCOND (Reciprocal of Condition Number of the Jac
     +obian in XP) = ',RCOND
      WRITE (25, 1070) 'VALUES OF FUNCTIONS IN XP'
      DO 1195 I = 1, ML-3, 4
         WRITE (25, 1139) I,FP(I),I+1,FP(I+1),I+2,FP(I+2),I+3,FP(I+3)
 1195 CONTINUE
      WRITE (27, 1070) '================================================
     +========================='
      CLOSE (26)
      CLOSE (25)
      GO TO 190
 1270 H(15) = 0
      REWIND (45)
      GOTO 180
 1280 H(14) = 0
      REWIND (44)
      GO TO 170
 1290 H(13) = 0
      REWIND (43)
      GO TO 160
 1300 H(12) = 0
      REWIND (42)
      GO TO 150
 1310 H(11) = 0
      REWIND (41)
      GO TO 140
 1320 H(10) = 0
      REWIND (40)
      GO TO 130
 1330 H(9) = 0
      REWIND (39)
      GO TO 120
 1340 H(8) = 0
      REWIND (38)
      GO TO 110
 1350 H(7) = 0
      REWIND (37)
      GO TO 80
 1380 H(6) = 0
      REWIND (36)
      GO TO 70
 1390 H(5) = 0
      REWIND (35)
      GO TO 60
 1400 H(4) = 0
      REWIND (34)
      GO TO 50
 1410 H(3) = 0
      REWIND (33)
      GO TO 40
 1420 H(2) = 0
      REWIND (32)
      GO TO 30
 1430 PRINT *, 'PROGRAM TERMINATED'
      DO 1440 I = 31, 45
         CLOSE(I)
 1440 CONTINUE
 1445 CLOSE (27)
      GO TO 1460
 1450 PRINT *, 'ERROR OPENING THE FILES OF PARAMETERS'
      STOP
 1460 END
*
*
*
      SUBROUTINE BELIEFS(A1,A2,PHI,L,M,AVEC,BVEC,QQ1,QQ2,F1,F2,G1,G2,
     $                   GM,PI,SINGUL,CHBL)

**BELIEFS***************************************************************
*                                                                      *
* Subroutine to compute the rational Belief structure with an 8 by 8   *
* markov transition matrix GM as stationary measure.                   *
* If the belief structure is not correct, CHBL is returned as true.    *
* If the system to compute the stationary distribution PI of GM        *
* is singular, SINGUL is returned as true. and the                     *
* solution PI  is undefined.   			                       *
* INPUTS : 							       *
*         A1,A2,L,M   = rational belief parameters		       *
*         PHI         = exogenous process parameter		       *
*         AVEC,BVEC   = vectors to identify matrices A and B	       *
* OUTPUTS:							       *
*         GM          = stationary measure			       *
*         QQ1,QQ2       = belief matrices of the agents to be used in  *
*                       the optimization			       *
*         PI          = stationary distribution of GM		       *
*         SINGUL,CHBL = logical values				       *
*								       *
************************************************************************

      INTEGER          I,J,LIMROW,LIMCOL,N
      PARAMETER        (N=8, LIMROW=8, LIMCOL=9)
      LOGICAL          SINGUL,CHBL
      DOUBLE PRECISION A1,A2,PHI,L(N),M(N),AVEC(4),BVEC(4)
      DOUBLE PRECISION A(4,4),B(4,4),F1(N,N),G1(N,N),F2(N,N),G2(N,N),
     $                 QQ1(N,N),QQ2(N,N),GM(N,N),SD(N,N+1),PI(N),SPI,
     $                 SGM(N),SF1(N),SF2(N),SG1(N),SG2(N),SQ1(N),SQ2(N)
      INTRINSIC        ABS
      EXTERNAL         GAUSS
	                 
************************************************************************
*                                                                      *
* double precision coefficients.                                       *
*                                                                      *
************************************************************************

      DOUBLE PRECISION ONE,ZERO,EPS
      PARAMETER       (ONE=1.0D+0, ZERO=0.0D+0, EPS=1.0D-15)

************************************************************************
*                                                                      *
* Building matrices A, B,                                              *
*                                                                      *
************************************************************************

      DO 100 I = 1, 4
         A(I,1) = AVEC(I)
         A(I,2) = A1 - AVEC(I)
         A(I,3) = A2 - AVEC(I)
         A(I,4) = ONE + AVEC(I) - A1 - A2
         B(I,1) = BVEC(I)
         B(I,2) = A1 - BVEC(I)
         B(I,3) = A2 - BVEC(I)
         B(I,4) = ONE + BVEC(I) - A1 - A2
 100  CONTINUE

************************************************************************
*                                                                      *
* Building the matrix GAMMA                                            *
*                                                                      *
************************************************************************

      DO 250 I = 1, 4
         DO 150 J = 1, 4
            GM(I,J) = PHI * A(I,J)
 150     CONTINUE
         DO 200 J = 5, 8
            GM(I,J) = (ONE - PHI) * A(I,J-4)
 200     CONTINUE
 250  CONTINUE
      DO 400 I = 5, 8
         DO 300 J = 1, 4
            GM(I,J) = (ONE - PHI) * B(I-4,J)
 300     CONTINUE
         DO 350 J = 5, 8
            GM(I,J) = PHI * B(I-4,J-4)
 350     CONTINUE
 400  CONTINUE

************************************************************************
*                                                                      *
* Finding the stationary distribution                                  *
*                                                                      *
************************************************************************

      DO 500 I = 1, 7
         DO 450 J = 1, 8
            SD(I,J) = GM(J,I)
 450     CONTINUE
         SD(I,I) = SD(I,I) - ONE
         SD(8,I) = ONE
         SD(I,9) = ZERO
 500  CONTINUE
      SD(8,8) = ONE
      SD(8,9) = ONE

* Use subroutine GAUSS to find the solution,
      
      CALL GAUSS(SD, LIMROW, LIMCOL, N, PI, SINGUL)
      
************************************************************************
*                                                                      *
* Building matrices F1 and F2                                          *
*                                                                      *
************************************************************************

      DO 650 I = 1, 4
         DO 550 J = 1, 4
            F1(I,J) = PHI * L(I) *  A(I,J)
 550     CONTINUE
         DO 600 J = 5, 8
            F1(I,J) = (ONE - PHI * L(I)) * A(I,J-4)
 600     CONTINUE
 650  CONTINUE
      DO 800 I = 5, 8
         DO 700 J = 1, 4
            F1(I,J) = (ONE - PHI) * L(I) * B(I-4,J)
 700     CONTINUE
         DO 750 J = 5, 8
            F1(I,J) = (ONE - (ONE - PHI) * L(I)) * B(I-4,J-4)
 750     CONTINUE
 800  CONTINUE
      DO 900 I = 1, 8
         DO 850 J = 1, 8
            F2(I,J) = (ONE / (ONE - A1)) * (GM(I,J) - A1 * F1(I,J))
 850     CONTINUE
 900  CONTINUE

************************************************************************
*                                                                      *
* Building matrices G1 and G2                                          *
*                                                                      *
************************************************************************

      DO 1050 I = 1, 4
         DO 950 J = 1, 4
            G1(I,J) = PHI * M(I) *  A(I,J)
 950     CONTINUE
         DO 1000 J = 5, 8
            G1(I,J) = (ONE - PHI * M(I)) * A(I,J-4)
 1000    CONTINUE
 1050 CONTINUE
      DO 1200 I = 5, 8
         DO 1100 J = 1, 4
            G1(I,J) = (ONE - PHI) * M(I) * B(I-4,J)
 1100    CONTINUE
         DO 1150 J = 5, 8
            G1(I,J) = (ONE - (ONE - PHI) * M(I)) * B(I-4,J-4)
 1150    CONTINUE
 1200 CONTINUE
      DO 1300 I = 1, 8
         DO 1250 J = 1, 8
            G2(I,J) = (ONE / (ONE - A2)) * (GM(I,J) - A2 * G1(I,J))
 1250    CONTINUE
 1300 CONTINUE

************************************************************************
*                                                                      *
* Building matrices Q1 and Q2                                          *
*                                                                      *
************************************************************************

      DO 1400 I = 1, 2
         DO 1350 J = 1, 8
            QQ1(I,J)   = F1(I,J)
            QQ1(I+2,J) = F2(I+2,J)
            QQ1(I+4,J) = F1(I+4,J)
            QQ1(I+6,J) = F2(I+6,J)
 1350    CONTINUE
 1400 CONTINUE
      DO 1450 J = 1, 8
         QQ2(1,J) = G1(1,J)
         QQ2(2,J) = G2(2,J)
         QQ2(3,J) = G1(3,J)
         QQ2(4,J) = G2(4,J)
         QQ2(5,J) = G1(5,J)
         QQ2(6,J) = G2(6,J)
         QQ2(7,J) = G1(7,J)
         QQ2(8,J) = G2(8,J)
 1450 CONTINUE
      CHBL = .FALSE.
      SPI = ZERO
      DO 1480 I = 1, 8
         SGM(I) = ZERO
         SF1(I) = ZERO
         SF2(I) = ZERO
         SG1(I) = ZERO
         SG2(I) = ZERO
         SQ1(I) = ZERO
         SQ2(I) = ZERO
 1480 CONTINUE
      DO 1550 I = 1, 8
         SPI = SPI + PI(I)
         DO 1500 J = 1, 8
            SGM(I) = SGM(I) + GM(I,J)
            SF1(I) = SF1(I) + F1(I,J)
            SF2(I) = SF2(I) + F2(I,J)
            SG1(I) = SG1(I) + G1(I,J)
            SG2(I) = SG2(I) + G2(I,J)
            SQ1(I) = SQ1(I) + QQ1(I,J)
            SQ2(I) = SQ2(I) + QQ2(I,J)
 1500    CONTINUE
 1550 CONTINUE
      IF (ABS(SPI-ONE) .GT. EPS) THEN
         CHBL = .TRUE.
      ENDIF
      DO 1600 I = 1, 8
         IF (ABS(SGM(I)-ONE) .GT. EPS) THEN
            CHBL = .TRUE.
         ENDIF
         IF (ABS(SF1(I)-ONE) .GT. EPS) THEN
            CHBL = .TRUE.
         ENDIF
         IF (ABS(SF2(I)-ONE) .GT. EPS) THEN
            CHBL = .TRUE.
         ENDIF
         IF (ABS(SG1(I)-ONE) .GT. EPS) THEN
            CHBL = .TRUE.
         ENDIF
         IF (ABS(SG2(I)-ONE) .GT. EPS) THEN
            CHBL = .TRUE.
         ENDIF
         IF (ABS(SQ1(I)-ONE) .GT. EPS) THEN
            CHBL = .TRUE.
         ENDIF
         IF (ABS(SQ2(I)-ONE) .GT. EPS) THEN
            CHBL = .TRUE.
         ENDIF
 1600 CONTINUE
      DO 1700 I = 1, 8
         IF (PI(I) .LT. ZERO) THEN
            CHBL = .TRUE.
         ENDIF
         DO 1650 J = 1, 8
            IF (GM(I,J) .LT. ZERO) THEN
               CHBL = .TRUE.
            ENDIF
            IF (F1(I,J) .LT. ZERO) THEN
               CHBL = .TRUE.
            ENDIF
            IF (F2(I,J) .LT. ZERO) THEN
               CHBL = .TRUE.
            ENDIF
            IF (G1(I,J) .LT. ZERO) THEN
               CHBL = .TRUE.
            ENDIF
            IF (G2(I,J) .LT. ZERO) THEN
               CHBL = .TRUE.
            ENDIF
            IF (QQ1(I,J) .LT. ZERO) THEN
               CHBL = .TRUE.
            ENDIF
            IF (QQ2(I,J) .LT. ZERO) THEN
               CHBL = .TRUE.
            ENDIF
 1650    CONTINUE
 1700 CONTINUE
      RETURN
      END
*
*
*
      SUBROUTINE SYSTEM(X,F,ML,NL)
      INTEGER          I,J,N
      PARAMETER        (N=8)
      DOUBLE PRECISION X(NL),F(ML),QS(N),TH(N),BN(N),QB(N),
     +                 UP1,UP1S,UP2,UP2S,
     +                 SUM1(N),SUM2(N),SUM1S(N),SUM2S(N),
     +                 R,RS,G,GS,W,WS,DIV(N),Q1(N,N),Q2(N,N)
      DOUBLE PRECISION ONE,ZERO
      PARAMETER        (ONE=1.0D0+00, ZERO=0.0D+00)
      COMMON /PARAM/ R,RS,G,GS,W,WS /VECTOR/ DIV /MATRIX/ Q1,Q2

* -------Initialization of variables

      DO 5 I = 1,N
         QS(I)  = X(I)
         QB(I)  = X(I+N)
         TH(I)  = X(I+2*N)
         BN(I)  = X(I+3*N)
 5    CONTINUE
      DO 10 I = 1, N
         SUM1(I)  = ZERO
         SUM2(I)  = ZERO
         SUM1S(I) = ZERO
         SUM2S(I) = ZERO
 10   CONTINUE
      DO 15 I = 1, ML
         F(I) = ZERO
 15   CONTINUE


* -------Computing the expectations in the Euler equations

      DO 30 I = 1, N
         DO 20 J = 1, N
            SUM1(I)  = SUM1(I)+UP2(QS(J),TH(I),BN(I),DIV(J))
     +                 *(QS(J)+ONE)*DIV(J)*Q1(I,J)
            SUM2(I)  = SUM2(I)+UP2(QS(J),TH(I),BN(I),DIV(J))
     +                 *Q1(I,J)
            SUM1S(I) = SUM1S(I)+UP2S(QS(J),TH(I),BN(I),DIV(J))
     +                 *(QS(J)+ONE)*DIV(J)*Q2(I,J)
            SUM2S(I) = SUM2S(I)+UP2S(QS(J),TH(I),BN(I),DIV(J))
     +                 *Q2(I,J)

 20      CONTINUE
 30   CONTINUE


* -------The reduced system of Euler equations

      DO 40 I = 1, N
         F(I)    =QS(I)*UP1(QS(I),TH(I),BN(I),QB(I))-R*SUM1(I)
         F(I+N)  =QB(I)*UP1(QS(I),TH(I),BN(I),QB(I))-R*SUM2(I)
         F(I+2*N)=QS(I)*UP1S(QS(I),TH(I),BN(I),QB(I))-RS*SUM1S(I)
         F(I+3*N)=QB(I)*UP1S(QS(I),TH(I),BN(I),QB(I))-RS*SUM2S(I)
 40   CONTINUE
      END
*
*
*
      DOUBLE PRECISION FUNCTION UP1(QS,TH,BN,QB)
      DOUBLE PRECISION X1,QS,TH,BN,QB,EPS,R,RS,G,GS,W,WS
      COMMON /PARAM/ R,RS,G,GS,W,WS
      EPS=5.0D-3

* -------Consumption of agent 1 when young

      X1=W-QS*TH-QB*BN

* -------Marginal Utility of agent 1 when young

      IF (X1 .LT. EPS) THEN
         UP1=(-G*(EPS**(-G-1D0)))*X1+(EPS**(-G)+G*EPS**(-G))
      ELSE 
         UP1=X1**(-G)
      ENDIF
      END
*
*
*
      DOUBLE PRECISION FUNCTION UP1S(QS,TH,BN,QB)
      DOUBLE PRECISION X1S,QS,TH,BN,QB,EPS,R,RS,G,GS,W,WS
      COMMON /PARAM/ R,RS,G,GS,W,WS
      EPS=5.0D-3

* -------Consumption of agent 2 when young

      X1S=WS-QS*(1D0-TH)+QB*BN

* -------Marginal Utility of agent 2 when young

      IF (X1S .LT. EPS) THEN
         UP1S=(-GS*(EPS**(-GS-1D0)))*X1S+(EPS**(-GS)+GS*EPS**(-GS))
      ELSE 
         UP1S=X1S**(-GS)
      ENDIF
      END
*
*
*
      DOUBLE PRECISION FUNCTION UP2(QS,TH,BN,DIV)
      DOUBLE PRECISION X2,QS,TH,BN,R,RS,G,GS,W,WS,DIV
      COMMON /PARAM/ R,RS,G,GS,W,WS
      EPS=5.0D-3

* -------Consumption of agent 1 when old

      X2=TH*(QS+1D0)*DIV+BN

* -------Marginal Utility of agent 1 when old

      IF (X2 .LT. EPS) THEN
         UP2=(-G*(EPS **(-G-1D0)))*X2+(EPS **(-G)+G*EPS **(-G))
      ELSE 
         UP2=X2**(-G)
      ENDIF
      END
*
*
*
      DOUBLE PRECISION FUNCTION UP2S(QS,TH,BN,DIV)
      DOUBLE PRECISION X2S,QS,TH,BN,R,RS,G,GS,W,WS,DIV
      COMMON /PARAM/ R,RS,G,GS,W,WS
      EPS=5.0D-3

* -------Consumption of agent 2 when old

      X2S=(1D0-TH)*(QS+1D0)*DIV-BN

* -------Marginal Utility of agent 2 when old

      IF (X2S .LT. EPS) THEN
         UP2S=(-GS*(EPS**(-GS-1D0)))*X2S+(EPS**(-GS)+GS*EPS**(-GS))
      ELSE 
         UP2S=X2S**(-GS)
      ENDIF
      END
*
*
*
      FUNCTION COUNT(NAME)
      INTEGER  EOF, COUNT
      CHARACTER*5  NAME
      COUNT = 0
      OPEN(UNIT=1, FILE=NAME, STATUS='OLD')
10    READ (1,*, IOSTAT=EOF)
      IF (EOF .GE. 0) THEN
         COUNT=COUNT+1
         GO TO 10
      END IF
      CLOSE (1)
      END
*
*
*
      FUNCTION EXT(CCOUNT)
      INTEGER H, T, O, CCOUNT, TCOUNT, OCOUNT
      CHARACTER*1 STRH, STRT, STRO
      CHARACTER*3 EXT
      H = CCOUNT / 100
      TCOUNT = CCOUNT - 100 * H
      T = TCOUNT / 10
      OCOUNT = TCOUNT - 10 * T
      O = OCOUNT
      STRH = CHAR(H + 48)
      STRT = CHAR(T + 48)
      STRO = CHAR(O + 48)
      EXT = STRH//STRT//STRO
      END
*
*
*
      SUBROUTINE GAUSS(SD, LIMROW, LIMCOL, N, PI, SINGUL)

**GAUSS*****************************************************************
*                                                                      *
* Subroutine to find solution of a linear system of N equations in N   *
* unknowns using Gaussian elimination, provided a unique solution      *
* exists.  The coefficients and constants of the linear system are     * 
* stored in the matrix SD,  which has LIMROW rows and LIMCOL columns.  *
* If the system is singular, SINGUL is returned as true, and the       *
* solution PI  is undefined.   Local identifiers used are:             *
*     I,J,K  : indices                                                 *
*     MULT   : multiplier used to eliminate an unknown                 *
*     ABSPIV : absolute value of pivot element                         *
*     PIVROW : row containing pivot element                            *
*     EPSIL  : a small positive real value ("almost zero")             *
*     TEMP   : used to interchange rows of matrix                      *
*                                                                      *
* Accepts: Two-dimensional array SD, integers LIMROW, LIMCOL, and N    *
* Returns: One-dimensional array PI and logical value SINGUL           *
************************************************************************
      INTEGER LIMROW, LIMCOL
      DOUBLE PRECISION SD(LIMROW, LIMCOL), PI(LIMROW), TEMP, MULT, 
     +                 EPSIL, ABSPIV,DABS
      PARAMETER (EPSIL = 1D-17)
      INTEGER N, PIVROW,I,J,K
      LOGICAL SINGUL
      INTRINSIC DABS
       
      SINGUL = .FALSE.
      DO 50 I = 1, N

*         Locate pivot element

         ABSPIV = DABS(SD(I,I))
         PIVROW = I
         DO 10 K = I + 1, N
            IF (DABS(SD(K,I)) .GT. ABSPIV) THEN
               ABSPIV = DABS(SD(K,I))
               PIVROW = K
            END IF
 10      CONTINUE
         
*         Check if matrix is (nearly) singular
		
         IF (ABSPIV .LT. EPSIL) THEN
            SINGUL = .TRUE.
            RETURN
         END IF

*         It isn't, so interchange rows PIVROW and I if necessary

         IF (PIVROW .NE. I) THEN
            DO 20 J = 1, N + 1
               TEMP = SD(I,J)
               SD(I,J) = SD(PIVROW,J)
               SD(PIVROW,J) = TEMP
 20         CONTINUE
         END IF

*         Eliminate Ith unknown from equations I + 1, ..., N

         DO 40 J = I + 1, N
            MULT = -SD(J,I) / SD(I,I)
            DO 30 K = I, N + 1
               SD(J,K) = SD(J,K) +  MULT * SD(I,K)
 30         CONTINUE
 40      CONTINUE

 50   CONTINUE

* Find the solutions by back substitution

      PI(N) = SD(N, N + 1) / SD(N,N)
      DO 70 J = N - 1, 1, -1
         PI(J) = SD(J, N + 1)
         DO 60 K = J + 1, N
            PI(J) = PI(J) - SD(J,K) * PI(K)
 60      CONTINUE
         PI(J) = PI(J) / SD(J,J)
 70   CONTINUE
      END
*
*
*
      SUBROUTINE DGECO(A,LDA,N,IPVT,RCOND,Z)
      INTEGER LDA,N,IPVT(1)
      DOUBLE PRECISION A(LDA,1),Z(1)
      DOUBLE PRECISION RCOND
*
*     DGECO FACTORS A DOUBLE PRECISION MATRIX BY GAUSSIAN ELIMINATION
*     AND ESTIMATES THE CONDITION OF THE MATRIX.
*
*     IF  RCOND  IS NOT NEEDED, DGEFA IS SLIGHTLY FASTER.
*     TO SOLVE  A*X = B , FOLLOW DGECO BY DGESL.
*     TO COMPUTE  INVERSE(A)*C , FOLLOW DGECO BY DGESL.
*     TO COMPUTE  DETERMINANT(A) , FOLLOW DGECO BY DGEDI.
*     TO COMPUTE  INVERSE(A) , FOLLOW DGECO BY DGEDI.
*
*     ON ENTRY
*
*        A       DOUBLE PRECISION(LDA, N)
*                THE MATRIX TO BE FACTORED.
*
*        LDA     INTEGER
*                THE LEADING DIMENSION OF THE ARRAY  A .
*
*        N       INTEGER
*                THE ORDER OF THE MATRIX  A .
*
*     ON RETURN
*
*        A       AN UPPER TRIANGULAR MATRIX AND THE MULTIPLIERS
*                WHICH WERE USED TO OBTAIN IT.
*                THE FACTORIZATION CAN BE WRITTEN  A = L*U  WHERE
*                L  IS A PRODUCT OF PERMUTATION AND UNIT LOWER
*                TRIANGULAR MATRICES AND  U  IS UPPER TRIANGULAR.
*
*        IPVT    INTEGER(N)
*                AN INTEGER VECTOR OF PIVOT INDICES.
*
*        RCOND   DOUBLE PRECISION
*                AN ESTIMATE OF THE RECIPROCAL CONDITION OF  A .
*                FOR THE SYSTEM  A*X = B , RELATIVE PERTURBATIONS
*                IN  A  AND  B  OF SIZE  EPSILON  MAY CAUSE
*                RELATIVE PERTURBATIONS IN  X  OF SIZE  EPSILON/RCOND .
*                IF  RCOND  IS SO SMALL THAT THE LOGICAL EXPRESSION
*                           1.0 + RCOND .EQ. 1.0
*                IS TRUE, THEN  A  MAY BE SINGULAR TO WORKING
*                PRECISION.  IN PARTICULAR,  RCOND  IS ZERO  IF
*                EXACT SINGULARITY IS DETECTED OR THE ESTIMATE
*                UNDERFLOWS.
*
*        Z       DOUBLE PRECISION(N)
*                A WORK VECTOR WHOSE CONTENTS ARE USUALLY UNIMPORTANT.
*                IF  A  IS CLOSE TO A SINGULAR MATRIX, THEN  Z  IS
*                AN APPROXIMATE NULL VECTOR IN THE SENSE THAT
*                NORM(A*Z) = RCOND*NORM(A)*NORM(Z) .
*
*     LINPACK. THIS VERSION DATED 08/14/78 .
*     CLEVE MOLER, UNIVERSITY OF NEW MEXICO, ARGONNE NATIONAL LAB.
*
*     SUBROUTINES AND FUNCTIONS
*
*     LINPACK DGEFA
*     BLAS DAXPY,DDOT,DSCAL,DASUM
*     FORTRAN DABS,DMAX1,DSIGN
*
*     INTERNAL VARIABLES
*
      DOUBLE PRECISION DDOT,EK,T,WK,WKM
      DOUBLE PRECISION ANORM,S,DASUM,SM,YNORM
      INTEGER INFO,J,K,KB,KP1,L
*
*
*     COMPUTE 1-NORM OF A
*
      ANORM = 0.0D0
      DO 10 J = 1, N
         ANORM = DMAX1(ANORM,DASUM(N,A(1,J),1))
   10 CONTINUE
*
*     FACTOR
*
      CALL DGEFA(A,LDA,N,IPVT,INFO)
*
*     RCOND = 1/(NORM(A)*(ESTIMATE OF NORM(INVERSE(A)))) .
*     ESTIMATE = NORM(Z)/NORM(Y) WHERE  A*Z = Y  AND  TRANS(A)*Y = E .
*     TRANS(A)  IS THE TRANSPOSE OF A .  THE COMPONENTS OF  E  ARE
*     CHOSEN TO CAUSE MAXIMUM LOCAL GROWTH IN THE ELEMENTS OF W  WHERE
*     TRANS(U)*W = E .  THE VECTORS ARE FREQUENTLY RESCALED TO AVOID
*     OVERFLOW.
*
*     SOLVE TRANS(U)*W = E
*
      EK = 1.0D0
      DO 20 J = 1, N
         Z(J) = 0.0D0
   20 CONTINUE
      DO 100 K = 1, N
         IF (Z(K) .NE. 0.0D0) EK = DSIGN(EK,-Z(K))
         IF (DABS(EK-Z(K)) .LE. DABS(A(K,K))) GO TO 30
            S = DABS(A(K,K))/DABS(EK-Z(K))
            CALL DSCAL(N,S,Z,1)
            EK = S*EK
   30    CONTINUE
         WK = EK - Z(K)
         WKM = -EK - Z(K)
         S = DABS(WK)
         SM = DABS(WKM)
         IF (A(K,K) .EQ. 0.0D0) GO TO 40
            WK = WK/A(K,K)
            WKM = WKM/A(K,K)
         GO TO 50
   40    CONTINUE
            WK = 1.0D0
            WKM = 1.0D0
   50    CONTINUE
         KP1 = K + 1
         IF (KP1 .GT. N) GO TO 90
            DO 60 J = KP1, N
               SM = SM + DABS(Z(J)+WKM*A(K,J))
               Z(J) = Z(J) + WK*A(K,J)
               S = S + DABS(Z(J))
   60       CONTINUE
            IF (S .GE. SM) GO TO 80
               T = WKM - WK
               WK = WKM
               DO 70 J = KP1, N
                  Z(J) = Z(J) + T*A(K,J)
   70          CONTINUE
   80       CONTINUE
   90    CONTINUE
         Z(K) = WK
  100 CONTINUE
      S = 1.0D0/DASUM(N,Z,1)
      CALL DSCAL(N,S,Z,1)
*
*     SOLVE TRANS(L)*Y = W
*
      DO 120 KB = 1, N
         K = N + 1 - KB
         IF (K .LT. N) Z(K) = Z(K) + DDOT(N-K,A(K+1,K),1,Z(K+1),1)
         IF (DABS(Z(K)) .LE. 1.0D0) GO TO 110
            S = 1.0D0/DABS(Z(K))
            CALL DSCAL(N,S,Z,1)
  110    CONTINUE
         L = IPVT(K)
         T = Z(L)
         Z(L) = Z(K)
         Z(K) = T
  120 CONTINUE
      S = 1.0D0/DASUM(N,Z,1)
      CALL DSCAL(N,S,Z,1)
*
      YNORM = 1.0D0
*
*     SOLVE L*V = Y
*
      DO 140 K = 1, N
         L = IPVT(K)
         T = Z(L)
         Z(L) = Z(K)
         Z(K) = T
         IF (K .LT. N) CALL DAXPY(N-K,T,A(K+1,K),1,Z(K+1),1)
         IF (DABS(Z(K)) .LE. 1.0D0) GO TO 130
            S = 1.0D0/DABS(Z(K))
            CALL DSCAL(N,S,Z,1)
            YNORM = S*YNORM
  130    CONTINUE
  140 CONTINUE
      S = 1.0D0/DASUM(N,Z,1)
      CALL DSCAL(N,S,Z,1)
      YNORM = S*YNORM
*
*     SOLVE  U*Z = V
*
      DO 160 KB = 1, N
         K = N + 1 - KB
         IF (DABS(Z(K)) .LE. DABS(A(K,K))) GO TO 150
            S = DABS(A(K,K))/DABS(Z(K))
            CALL DSCAL(N,S,Z,1)
            YNORM = S*YNORM
  150    CONTINUE
         IF (A(K,K) .NE. 0.0D0) Z(K) = Z(K)/A(K,K)
         IF (A(K,K) .EQ. 0.0D0) Z(K) = 1.0D0
         T = -Z(K)
         CALL DAXPY(K-1,T,A(1,K),1,Z(1),1)
  160 CONTINUE
*     MAKE ZNORM = 1.0
      S = 1.0D0/DASUM(N,Z,1)
      CALL DSCAL(N,S,Z,1)
      YNORM = S*YNORM
*
      IF (ANORM .NE. 0.0D0) RCOND = YNORM/ANORM
      IF (ANORM .EQ. 0.0D0) RCOND = 0.0D0
      RETURN
      END
*
*
*
      SUBROUTINE DGEFA(A,LDA,N,IPVT,INFO)
      INTEGER LDA,N,IPVT(1),INFO
      DOUBLE PRECISION A(LDA,1)
*
*     DGEFA FACTORS A DOUBLE PRECISION MATRIX BY GAUSSIAN ELIMINATION.
*
*     DGEFA IS USUALLY CALLED BY DGECO, BUT IT CAN BE CALLED
*     DIRECTLY WITH A SAVING IN TIME IF  RCOND  IS NOT NEEDED.
*     (TIME FOR DGECO) = (1 + 9/N)*(TIME FOR DGEFA) .
*
*     ON ENTRY
*
*        A       DOUBLE PRECISION(LDA, N)
*                THE MATRIX TO BE FACTORED.
*
*        LDA     INTEGER
*                THE LEADING DIMENSION OF THE ARRAY  A .
*
*        N       INTEGER
*                THE ORDER OF THE MATRIX  A .
*
*     ON RETURN
*
*        A       AN UPPER TRIANGULAR MATRIX AND THE MULTIPLIERS
*                WHICH WERE USED TO OBTAIN IT.
*                THE FACTORIZATION CAN BE WRITTEN  A = L*U  WHERE
*                L  IS A PRODUCT OF PERMUTATION AND UNIT LOWER
*                TRIANGULAR MATRICES AND  U  IS UPPER TRIANGULAR.
*
*        IPVT    INTEGER(N)
*                AN INTEGER VECTOR OF PIVOT INDICES.
*
*        INFO    INTEGER
*                = 0  NORMAL VALUE.
*                = K  IF  U(K,K) .EQ. 0.0 .  THIS IS NOT AN ERROR
*                     CONDITION FOR THIS SUBROUTINE, BUT IT DOES
*                     INDICATE THAT DGESL OR DGEDI WILL DIVIDE BY ZERO
*                     IF CALLED.  USE  RCOND  IN DGECO FOR A RELIABLE
*                     INDICATION OF SINGULARITY.
*
*     LINPACK. THIS VERSION DATED 08/14/78 .
*     CLEVE MOLER, UNIVERSITY OF NEW MEXICO, ARGONNE NATIONAL LAB.
*
*     SUBROUTINES AND FUNCTIONS
*
*     BLAS DAXPY,DSCAL,IDAMAX
*
*     INTERNAL VARIABLES
*
      DOUBLE PRECISION T
      INTEGER IDAMAX,J,K,KP1,L,NM1
*
*
*     GAUSSIAN ELIMINATION WITH PARTIAL PIVOTING
*
      INFO = 0
      NM1 = N - 1
      IF (NM1 .LT. 1) GO TO 70
      DO 60 K = 1, NM1
         KP1 = K + 1
*
*        FIND L = PIVOT INDEX
*
         L = IDAMAX(N-K+1,A(K,K),1) + K - 1
         IPVT(K) = L
*
*        ZERO PIVOT IMPLIES THIS COLUMN ALREADY TRIANGULARIZED
*
         IF (A(L,K) .EQ. 0.0D0) GO TO 40
*
*           INTERCHANGE IF NECESSARY
*
            IF (L .EQ. K) GO TO 10
               T = A(L,K)
               A(L,K) = A(K,K)
               A(K,K) = T
   10       CONTINUE
*
*           COMPUTE MULTIPLIERS
*
            T = -1.0D0/A(K,K)
            CALL DSCAL(N-K,T,A(K+1,K),1)
*
*           ROW ELIMINATION WITH COLUMN INDEXING
*
            DO 30 J = KP1, N
               T = A(L,J)
               IF (L .EQ. K) GO TO 20
                  A(L,J) = A(K,J)
                  A(K,J) = T
   20          CONTINUE
               CALL DAXPY(N-K,T,A(K+1,K),1,A(K+1,J),1)
   30       CONTINUE
         GO TO 50
   40    CONTINUE
            INFO = K
   50    CONTINUE
   60 CONTINUE
   70 CONTINUE
      IPVT(N) = N
      IF (A(N,N) .EQ. 0.0D0) INFO = N
      RETURN
      END
