
      SUBROUTINE BAKSLV(NR,N,A,X,B)                                     BAKSLV
C                                                                       BAKSLV
C PURPOSE                                                               BAKSLV
C -------                                                               BAKSLV
C SOLVE  AX=B  WHERE A IS UPPER TRIANGULAR MATRIX.                      BAKSLV
C NOTE THAT A IS INPUT AS A LOWER TRIANGULAR MATRIX AND                 BAKSLV
C THAT THIS ROUTINE TAKES ITS TRANSPOSE IMPLICITLY.                     BAKSLV
C                                                                       BAKSLV
C PARAMETERS                                                            BAKSLV
C ----------                                                            BAKSLV
C NR           --> ROW DIMENSION OF MATRIX                              BAKSLV
C N            --> DIMENSION OF PROBLEM                                 BAKSLV
C A(N,N)       --> LOWER TRIANGULAR MATRIX (PRESERVED)                  BAKSLV
C X(N)        <--  SOLUTION VECTOR                                      BAKSLV
C B(N)         --> RIGHT-HAND SIDE VECTOR                               BAKSLV
C                                                                       BAKSLV
C NOTE                                                                  BAKSLV
C ----                                                                  BAKSLV
C IF B IS NO LONGER REQUIRED BY CALLING ROUTINE,                        BAKSLV
C THEN VECTORS B AND X MAY SHARE THE SAME STORAGE.                      BAKSLV
C                                                                       BAKSLV
      INTEGER NR,N,I,IP1,J                                              BAKSLV
      DOUBLE PRECISION SUM                                              BAKSLV 
      DOUBLE PRECISION A(NR,1),X(N),B(N)                                BAKSLV
C                                                                       BAKSLV
C SOLVE (L-TRANSPOSE)X=B. (BACK SOLVE)                                  BAKSLV
C                                                                       BAKSLV
      I=N                                                               BAKSLV
      X(I)=B(I)/A(I,I)                                                  BAKSLV
      IF(N.EQ.1) RETURN                                                 BAKSLV
   30 IP1=I                                                             BAKSLV
      I=I-1                                                             BAKSLV
      SUM=0.                                                            BAKSLV
      DO 40 J=IP1,N                                                     BAKSLV
        SUM=SUM+A(J,I)*X(J)                                             BAKSLV
   40 CONTINUE                                                          BAKSLV
      X(I)=(B(I)-SUM)/A(I,I)                                            BAKSLV
      IF(I.GT.1) GO TO 30                                               BAKSLV
      RETURN                                                            BAKSLV
      END                                                               BAKSLV
      SUBROUTINE CHOLDC(NR,N,A,DIAGMX,TOL,ADDMAX)                       CHOLDC
C                                                                       CHOLDC
C PURPOSE                                                               CHOLDC
C -------                                                               CHOLDC
C FIND THE PERTURBED L(L-TRANSPOSE) [WRITTEN LL+] DECOMPOSITION         CHOLDC
C OF A+D, WHERE D IS A NON-NEGATIVE DIAGONAL MATRIX ADDED TO A IF       CHOLDC
C NECESSARY TO ALLOW THE CHOLESKY DECOMPOSITION TO CONTINUE.            CHOLDC
C                                                                       CHOLDC
C PARAMETERS                                                            CHOLDC
C ----------                                                            CHOLDC
C NR           --> ROW DIMENSION OF MATRIX                              CHOLDC
C N            --> DIMENSION OF PROBLEM                                 CHOLDC
C A(N,N)      <--> ON ENTRY: MATRIX FOR WHICH TO FIND PERTURBED         CHOLDC
C                       CHOLESKY DECOMPOSITION                          CHOLDC
C                  ON EXIT:  CONTAINS L OF LL+ DECOMPOSITION            CHOLDC
C                  IN LOWER TRIANGULAR PART AND DIAGONAL OF "A"         CHOLDC
C DIAGMX       --> MAXIMUM DIAGONAL ELEMENT OF "A"                      CHOLDC
C TOL          --> TOLERANCE                                            CHOLDC
C ADDMAX      <--  MAXIMUM AMOUNT IMPLICITLY ADDED TO DIAGONAL OF "A"   CHOLDC
C                  IN FORMING THE CHOLESKY DECOMPOSITION OF A+D         CHOLDC
C INTERNAL VARIABLES                                                    CHOLDC
C ------------------                                                    CHOLDC
C AMINL    SMALLEST ELEMENT ALLOWED ON DIAGONAL OF L                    CHOLDC
C AMNLSQ   =AMINL**2                                                    CHOLDC
C OFFMAX   MAXIMUM OFF-DIAGONAL ELEMENT IN COLUMN OF A                  CHOLDC
C                                                                       CHOLDC
C                                                                       CHOLDC
C DESCRIPTION                                                           CHOLDC
C -----------                                                           CHOLDC
C THE NORMAL CHOLESKY DECOMPOSITION IS PERFORMED.  HOWEVER, IF AT ANY   CHOLDC
C POINT THE ALGORITHM WOULD ATTEMPT TO SET L(I,I)=SQRT(TEMP)            CHOLDC
C WITH TEMP < TOL*DIAGMX, THEN L(I,I) IS SET TO SQRT(TOL*DIAGMX)        CHOLDC
C INSTEAD.  THIS IS EQUIVALENT TO ADDING TOL*DIAGMX-TEMP TO A(I,I)      CHOLDC
C                                                                       CHOLDC
C                                                                       CHOLDC
      INTEGER NR,N,J,JM1,K,JP1,I                                        CHOLDC
      DOUBLE PRECISION DIAGMX,TOL,ADDMAX,AMINL,SUM,TEMP,AMNLSQ,OFFMAX   CHOLDC
      DOUBLE PRECISION A(NR,1)                                          CHOLDC
C                                                                       CHOLDC
      ADDMAX=0.                                                         CHOLDC
      AMINL=SQRT(DIAGMX*TOL)                                            CHOLDC
      AMNLSQ=AMINL*AMINL                                                CHOLDC
C                                                                       CHOLDC
C FORM COLUMN J OF L                                                    CHOLDC
C                                                                       CHOLDC
      DO 100 J=1,N                                                      CHOLDC
C FIND DIAGONAL ELEMENTS OF L                                           CHOLDC
        SUM=0.                                                          CHOLDC
        IF(J.EQ.1) GO TO 20                                             CHOLDC
        JM1=J-1                                                         CHOLDC
        DO 10 K=1,JM1                                                   CHOLDC
          SUM=SUM + A(J,K)*A(J,K)                                       CHOLDC
   10   CONTINUE                                                        CHOLDC
   20   TEMP=A(J,J)-SUM                                                 CHOLDC
        IF(TEMP.LT.AMNLSQ) GO TO 30                                     CHOLDC
C       IF(TEMP.GE.AMINL**2)                                            CHOLDC
C       THEN                                                            CHOLDC
          A(J,J)=SQRT(TEMP)                                             CHOLDC
          GO TO 40                                                      CHOLDC
C       ELSE                                                            CHOLDC
C                                                                       CHOLDC
C FIND MAXIMUM OFF-DIAGONAL ELEMENT IN COLUMN                           CHOLDC
   30     OFFMAX=0.                                                     CHOLDC
          IF(J.EQ.N) GO TO 37                                           CHOLDC
          JP1=J+1                                                       CHOLDC
          DO 35 I=JP1,N                                                 CHOLDC
            IF(ABS(A(I,J)).GT.OFFMAX) OFFMAX=ABS(A(I,J))                CHOLDC
   35     CONTINUE                                                      CHOLDC
   37     IF(OFFMAX.LE.AMNLSQ) OFFMAX=AMNLSQ                            CHOLDC
C                                                                       CHOLDC
C ADD TO DIAGONAL ELEMENT  TO ALLOW CHOLESKY DECOMPOSITION TO CONTINUE  CHOLDC
          A(J,J)=SQRT(OFFMAX)                                           CHOLDC
          ADDMAX=MAX(ADDMAX,OFFMAX-TEMP)                                CHOLDC
C       ENDIF                                                           CHOLDC
C                                                                       CHOLDC
C FIND I,J ELEMENT OF LOWER TRIANGULAR MATRIX                           CHOLDC
   40   IF(J.EQ.N) GO TO 100                                            CHOLDC
        JP1=J+1                                                         CHOLDC
        DO 70 I=JP1,N                                                   CHOLDC
          SUM=0.0                                                       CHOLDC
          IF(J.EQ.1) GO TO 60                                           CHOLDC
          JM1=J-1                                                       CHOLDC
          DO 50 K=1,JM1                                                 CHOLDC
            SUM=SUM+A(I,K)*A(J,K)                                       CHOLDC
   50     CONTINUE                                                      CHOLDC
   60     A(I,J)=(A(I,J)-SUM)/A(J,J)                                    CHOLDC
   70   CONTINUE                                                        CHOLDC
  100 CONTINUE                                                          CHOLDC
      RETURN                                                            CHOLDC
      END                                                               CHOLDC
      SUBROUTINE CHLHSN(NR,N,A,EPSM,SX,UDIAG)                           CHLHSN
C                                                                       CHLHSN
C PURPOSE                                                               CHLHSN
C -------                                                               CHLHSN
C FIND THE L(L-TRANSPOSE) [WRITTEN LL+] DECOMPOSITION OF THE PERTURBED  CHLHSN
C MODEL HESSIAN MATRIX A+MU*I(WHERE MU\0 AND I IS THE IDENTITY MATRIX)  CHLHSN
C WHICH IS SAFELY POSITIVE DEFINITE.  IF A IS SAFELY POSITIVE DEFINITE  CHLHSN
C UPON ENTRY, THEN MU=0.                                                CHLHSN
C                                                                       CHLHSN
C PARAMETERS                                                            CHLHSN
C ----------                                                            CHLHSN
C NR           --> ROW DIMENSION OF MATRIX                              CHLHSN
C N            --> DIMENSION OF PROBLEM                                 CHLHSN
C A(N,N)      <--> ON ENTRY; "A" IS MODEL HESSIAN (ONLY LOWER           CHLHSN
C                  TRIANGULAR PART AND DIAGONAL STORED)                 CHLHSN
C                  ON EXIT:  A CONTAINS L OF LL+ DECOMPOSITION OF       CHLHSN
C                  PERTURBED MODEL HESSIAN IN LOWER TRIANGULAR          CHLHSN
C                  PART AND DIAGONAL AND CONTAINS HESSIAN IN UPPER      CHLHSN
C                  TRIANGULAR PART AND UDIAG                            CHLHSN
C EPSM         --> MACHINE EPSILON                                      CHLHSN
C SX(N)        --> DIAGONAL SCALING MATRIX FOR X                        CHLHSN
C UDIAG(N)    <--  ON EXIT: CONTAINS DIAGONAL OF HESSIAN                CHLHSN
C                                                                       CHLHSN
C INTERNAL VARIABLES                                                    CHLHSN
C ------------------                                                    CHLHSN
C TOL              TOLERANCE                                            CHLHSN
C DIAGMN           MINIMUM ELEMENT ON DIAGONAL OF A                     CHLHSN
C DIAGMX           MAXIMUM ELEMENT ON DIAGONAL OF A                     CHLHSN
C OFFMAX           MAXIMUM OFF-DIAGONAL ELEMENT OF A                    CHLHSN
C OFFROW           SUM OF OFF-DIAGONAL ELEMENTS IN A ROW OF A           CHLHSN
C EVMIN            MINIMUM EIGENVALUE OF A                              CHLHSN
C EVMAX            MAXIMUM EIGENVALUE OF A                              CHLHSN
C                                                                       CHLHSN
C DESCRIPTION                                                           CHLHSN
C -----------                                                           CHLHSN
C 1. IF "A" HAS ANY NEGATIVE DIAGONAL ELEMENTS, THEN CHOOSE MU>0        CHLHSN
C SUCH THAT THE DIAGONAL OF A:=A+MU*I IS ALL POSITIVE                   CHLHSN
C WITH THE RATIO OF ITS SMALLEST TO LARGEST ELEMENT ON THE              CHLHSN
C ORDER OF SQRT(EPSM).                                                  CHLHSN
C                                                                       CHLHSN
C 2. "A" UNDERGOES A PERTURBED CHOLESKY DECOMPOSITION WHICH             CHLHSN
C RESULTS IN AN LL+ DECOMPOSITION OF A+D, WHERE D IS A                  CHLHSN
C NON-NEGATIVE DIAGONAL MATRIX WHICH IS IMPLICITLY ADDED TO             CHLHSN
C "A" DURING THE DECOMPOSITION IF "A" IS NOT POSITIVE DEFINITE.         CHLHSN
C "A" IS RETAINED AND NOT CHANGED DURING THIS PROCESS BY                CHLHSN
C COPYING L INTO THE UPPER TRIANGULAR PART OF "A" AND THE               CHLHSN
C DIAGONAL INTO UDIAG.  THEN THE CHOLESKY DECOMPOSITION ROUTINE         CHLHSN
C IS CALLED.  ON RETURN, ADDMAX CONTAINS MAXIMUM ELEMENT OF D.          CHLHSN
C                                                                       CHLHSN
C 3. IF ADDMAX=0, "A" WAS POSITIVE DEFINITE GOING INTO STEP 2           CHLHSN
C AND RETURN IS MADE TO CALLING PROGRAM.  OTHERWISE,                    CHLHSN
C THE MINIMUM NUMBER SDD WHICH MUST BE ADDED TO THE                     CHLHSN
C DIAGONAL OF A TO MAKE IT SAFELY STRICTLY DIAGONALLY DOMINANT          CHLHSN
C IS CALCULATED.  SINCE A+ADDMAX*I AND A+SDD*I ARE SAFELY               CHLHSN
C POSITIVE DEFINITE, CHOOSE MU=MIN(ADDMAX,SDD) AND DECOMPOSE            CHLHSN
C A+MU*I TO OBTAIN L.                                                   CHLHSN
C                                                                       CHLHSN
      INTEGER NR,N,I,J,IM1,JP1,IP1,JM1                                  CHLHSN
      DOUBLE PRECISION EPSM,TOL,DIAGMX,DIAGMN,POSMAX,AMU,OFFMAX         CHLHSN
      DOUBLE PRECISION ADDMAX,EVMIN,EVMAX,OFFROW,SDD,ZERO               CHLHSN
      DOUBLE PRECISION A(NR,1),SX(N),UDIAG(N)                           CHLHSN
      DATA ZERO/0.0D0/                                                  CHLHSN
C                                                                       CHLHSN
C SCALE HESSIAN                                                         CHLHSN
C PRE- AND POST- MULTIPLY "A" BY INV(SX)                                CHLHSN
C                                                                       CHLHSN
      DO 20 J=1,N                                                       CHLHSN
        DO 10 I=J,N                                                     CHLHSN
          A(I,J)=A(I,J)/(SX(I)*SX(J))                                   CHLHSN
   10   CONTINUE                                                        CHLHSN
   20 CONTINUE                                                          CHLHSN
C                                                                       CHLHSN
C STEP1                                                                 CHLHSN
C -----                                                                 CHLHSN
C NOTE:  IF A DIFFERENT TOLERANCE IS DESIRED THROUGHOUT THIS            CHLHSN
C ALGORITHM, CHANGE TOLERANCE HERE:                                     CHLHSN
      TOL=SQRT(EPSM)                                                    CHLHSN
C                                                                       CHLHSN
      DIAGMX=A(1,1)                                                     CHLHSN
      DIAGMN=A(1,1)                                                     CHLHSN
      IF(N.EQ.1) GO TO 35                                               CHLHSN
      DO 30 I=2,N                                                       CHLHSN
        IF(A(I,I).LT.DIAGMN) DIAGMN=A(I,I)                              CHLHSN
        IF(A(I,I).GT.DIAGMX) DIAGMX=A(I,I)                              CHLHSN
   30 CONTINUE                                                          CHLHSN
   35 POSMAX=MAX(DIAGMX,0.0D0)                                          CHLHSN
C                                                                       CHLHSN
C DIAGMN .LE. 0                                                         CHLHSN
C                                                                       CHLHSN
      IF(DIAGMN.GT.POSMAX*TOL) GO TO 100                                CHLHSN
C     IF(DIAGMN.LE.POSMAX*TOL)                                          CHLHSN
C     THEN                                                              CHLHSN
        AMU=TOL*(POSMAX-DIAGMN)-DIAGMN                                  CHLHSN
        IF(AMU.NE.0.) GO TO 60                                          CHLHSN
C       IF(AMU.EQ.0.)                                                   CHLHSN
C       THEN                                                            CHLHSN
C                                                                       CHLHSN
C FIND LARGEST OFF-DIAGONAL ELEMENT OF A                                CHLHSN
          OFFMAX=0.                                                     CHLHSN
          IF(N.EQ.1) GO TO 50                                           CHLHSN
          DO 45 I=2,N                                                   CHLHSN
            IM1=I-1                                                     CHLHSN
            DO 40 J=1,IM1                                               CHLHSN
              IF(ABS(A(I,J)).GT.OFFMAX) OFFMAX=ABS(A(I,J))              CHLHSN
   40       CONTINUE                                                    CHLHSN
   45     CONTINUE                                                      CHLHSN
   50     AMU=OFFMAX                                                    CHLHSN
          IF(AMU.NE.0.) GO TO 55                                        CHLHSN
C         IF(AMU.EQ.0.)                                                 CHLHSN
C         THEN                                                          CHLHSN
            AMU=1.0                                                     CHLHSN
            GO TO 60                                                    CHLHSN
C         ELSE                                                          CHLHSN
   55       AMU=AMU*(1.0+TOL)                                           CHLHSN
C         ENDIF                                                         CHLHSN
C       ENDIF                                                           CHLHSN
C                                                                       CHLHSN
C A=A + MU*I                                                            CHLHSN
C                                                                       CHLHSN
   60   DO 65 I=1,N                                                     CHLHSN
          A(I,I)=A(I,I)+AMU                                             CHLHSN
   65   CONTINUE                                                        CHLHSN
        DIAGMX=DIAGMX+AMU                                               CHLHSN
C     ENDIF                                                             CHLHSN
C                                                                       CHLHSN
C STEP2                                                                 CHLHSN
C -----                                                                 CHLHSN
C COPY LOWER TRIANGULAR PART OF "A" TO UPPER TRIANGULAR PART            CHLHSN
C AND DIAGONAL OF "A" TO UDIAG                                          CHLHSN
C                                                                       CHLHSN
  100 CONTINUE                                                          CHLHSN
      DO 110 J=1,N                                                      CHLHSN
        UDIAG(J)=A(J,J)                                                 CHLHSN
        IF(J.EQ.N) GO TO 110                                            CHLHSN
        JP1=J+1                                                         CHLHSN
        DO 105 I=JP1,N                                                  CHLHSN
          A(J,I)=A(I,J)                                                 CHLHSN
  105   CONTINUE                                                        CHLHSN
  110 CONTINUE                                                          CHLHSN
C                                                                       CHLHSN
      CALL CHOLDC(NR,N,A,DIAGMX,TOL,ADDMAX)                             CHLHSN
C                                                                       CHLHSN
C                                                                       CHLHSN
C STEP3                                                                 CHLHSN
C -----                                                                 CHLHSN
C IF ADDMAX=0, "A" WAS POSITIVE DEFINITE GOING INTO STEP 2,             CHLHSN
C THE LL+ DECOMPOSITION HAS BEEN DONE, AND WE RETURN.                   CHLHSN
C OTHERWISE, ADDMAX>0.  PERTURB "A" SO THAT IT IS SAFELY                CHLHSN
C DIAGONALLY DOMINANT AND FIND LL+ DECOMPOSITION                        CHLHSN
C                                                                       CHLHSN
      IF(ADDMAX.LE.0.) GO TO 170                                        CHLHSN
C     IF(ADDMAX.GT.0.)                                                  CHLHSN
C     THEN                                                              CHLHSN
C                                                                       CHLHSN
C RESTORE ORIGINAL "A" (LOWER TRIANGULAR PART AND DIAGONAL)             CHLHSN
C                                                                       CHLHSN
        DO 120 J=1,N                                                    CHLHSN
          A(J,J)=UDIAG(J)                                               CHLHSN
          IF(J.EQ.N) GO TO 120                                          CHLHSN
          JP1=J+1                                                       CHLHSN
          DO 115 I=JP1,N                                                CHLHSN
            A(I,J)=A(J,I)                                               CHLHSN
  115     CONTINUE                                                      CHLHSN
  120   CONTINUE                                                        CHLHSN
C                                                                       CHLHSN
C FIND SDD SUCH THAT A+SDD*I IS SAFELY POSITIVE DEFINITE                CHLHSN
C NOTE:  EVMIN<0 SINCE A IS NOT POSITIVE DEFINITE;                      CHLHSN
C                                                                       CHLHSN
        EVMIN=0.                                                        CHLHSN
        EVMAX=A(1,1)                                                    CHLHSN
        DO 150 I=1,N                                                    CHLHSN
          OFFROW=0.                                                     CHLHSN
          IF(I.EQ.1) GO TO 135                                          CHLHSN
          IM1=I-1                                                       CHLHSN
          DO 130 J=1,IM1                                                CHLHSN
            OFFROW=OFFROW+ABS(A(I,J))                                   CHLHSN
  130     CONTINUE                                                      CHLHSN
  135     IF(I.EQ.N) GO TO 145                                          CHLHSN
          IP1=I+1                                                       CHLHSN
          DO 140 J=IP1,N                                                CHLHSN
            OFFROW=OFFROW+ABS(A(J,I))                                   CHLHSN
  140     CONTINUE                                                      CHLHSN
  145     EVMIN=MIN(EVMIN,A(I,I)-OFFROW)                                CHLHSN
          EVMAX=MAX(EVMAX,A(I,I)+OFFROW)                                CHLHSN
  150   CONTINUE                                                        CHLHSN
        SDD=TOL*(EVMAX-EVMIN)-EVMIN                                     CHLHSN
C                                                                       CHLHSN
C PERTURB "A" AND DECOMPOSE AGAIN                                       CHLHSN
C                                                                       CHLHSN
        AMU=MIN(SDD,ADDMAX)                                             CHLHSN
        DO 160 I=1,N                                                    CHLHSN
          A(I,I)=A(I,I)+AMU                                             CHLHSN
          UDIAG(I)=A(I,I)                                               CHLHSN
  160   CONTINUE                                                        CHLHSN
C                                                                       CHLHSN
C "A" NOW GUARANTEED SAFELY POSITIVE DEFINITE                           CHLHSN
C                                                                       CHLHSN
        CALL CHOLDC(NR,N,A,ZERO,TOL,ADDMAX)                             CHLHSN
C     ENDIF                                                             CHLHSN
C                                                                       CHLHSN
C UNSCALE HESSIAN AND CHOLESKY DECOMPOSITION MATRIX                     CHLHSN
C                                                                       CHLHSN
  170 DO 190 J=1,N                                                      CHLHSN
        DO 175 I=J,N                                                    CHLHSN
          A(I,J)=SX(I)*A(I,J)                                           CHLHSN
  175   CONTINUE                                                        CHLHSN
        IF(J.EQ.1) GO TO 185                                            CHLHSN
        JM1=J-1                                                         CHLHSN
        DO 180 I=1,JM1                                                  CHLHSN
          A(I,J)=SX(I)*SX(J)*A(I,J)                                     CHLHSN
  180   CONTINUE                                                        CHLHSN
  185   UDIAG(J)=UDIAG(J)*SX(J)*SX(J)                                   CHLHSN
  190 CONTINUE                                                          CHLHSN
      RETURN                                                            CHLHSN
      END                                                               CHLHSN
      SUBROUTINE DFAUT(N,TYPSIZ,FSCALE,METHOD,IEXP,MSG,NDIGIT,          DFAULT
     +     ITNLIM,IAGFLG,IAHFLG,IPR,DLT,GRADTL,STEPMX,STEPTL)           DFAULT
C                                                                       DFAULT
C PURPOSE                                                               DFAULT
C -------                                                               DFAULT
C SET DEFAULT VALUES FOR EACH INPUT VARIABLE TO                         DFAULT
C MINIMIZATION ALGORITHM.                                               DFAULT
C                                                                       DFAULT
C PARAMETERS                                                            DFAULT
C ----------                                                            DFAULT
C N            --> DIMENSION OF PROBLEM                                 DFAULT
C TYPSIZ(N)   <--  TYPICAL SIZE FOR EACH COMPONENT OF X                 DFAULT
C FSCALE      <--  ESTIMATE OF SCALE OF MINIMIZATION FUNCTION           DFAULT
C METHOD      <--  ALGORITHM TO USE TO SOLVE MINIMIZATION PROBLEM       DFAULT
C IEXP        <--  =0 IF MINIMIZATION FUNCTION NOT EXPENSIVE TO EVALUATEDFAULT
C MSG         <--  MESSAGE TO INHIBIT CERTAIN AUTOMATIC CHECKS + OUTPUT DFAULT
C NDIGIT      <--  NUMBER OF GOOD DIGITS IN MINIMIZATION FUNCTION       DFAULT
C ITNLIM      <--  MAXIMUM NUMBER OF ALLOWABLE ITERATIONS               DFAULT
C IAGFLG      <--  =0 IF ANALYTIC GRADIENT NOT SUPPLIED                 DFAULT
C IAHFLG      <--  =0 IF ANALYTIC HESSIAN NOT SUPPLIED                  DFAULT
C IPR         <--  DEVICE TO WHICH TO SEND OUTPUT                       DFAULT
C DLT         <--  TRUST REGION RADIUS                                  DFAULT
C GRADTL      <--  TOLERANCE AT WHICH GRADIENT CONSIDERED CLOSE ENOUGH  DFAULT
C                  TO ZERO TO TERMINATE ALGORITHM                       DFAULT
C STEPMX      <--  VALUE OF ZERO TO TRIP DEFAULT MAXIMUM IN OPTCHK      DFAULT
C STEPTL      <--  TOLERANCE AT WHICH SUCCESSIVE ITERATES CONSIDERED    DFAULT
C                  CLOSE ENOUGH TO TERMINATE ALGORITHM                  DFAULT
C                                                                       DFAULT
      INTEGER N,METHOD,IEXP,MSG,NDIGIT,ITNLIM,IAGFLG,IAHFLG,IPR,I       DFAULT
      DOUBLE PRECISION FSCALE,DLT,GRADTL,STEPMX,STEPTL,EPSM,DPMEPS      DFAULT
      DOUBLE PRECISION TYPSIZ(N),ZERO,ONE,THREE                         DFAULT
      DATA ZERO,ONE,THREE/0.0D0,1.0D0,3.0D0/                            DFAULT
C                                                                       DFAULT
C SET TYPICAL SIZE OF X AND MINIMIZATION FUNCTION                       DFAULT
      DO 10 I=1,N                                                       DFAULT
        TYPSIZ(I)=ONE                                                   DFAULT
   10 CONTINUE                                                          DFAULT
      FSCALE=ONE                                                        DFAULT
C                                                                       DFAULT
C SET TOLERANCES                                                        DFAULT
      DLT=-ONE                                                          DFAULT
      EPSM=DPMEPS()                                                     DFAULT
      GRADTL=EPSM**(ONE/THREE)                                          DFAULT
      STEPMX=ZERO                                                       DFAULT
      STEPTL=SQRT(EPSM)                                                 DFAULT
C                                                                       DFAULT
C SET FLAGS                                                             DFAULT
      METHOD=1                                                          DFAULT
      IEXP=1                                                            DFAULT
      MSG=0                                                             DFAULT
      NDIGIT=-1                                                         DFAULT
      ITNLIM=150                                                        DFAULT
      IAGFLG=0                                                          DFAULT
      IAHFLG=0                                                          DFAULT
      IPR=6                                                             DFAULT
C                                                                       DFAULT
      RETURN                                                            DFAULT
      END                                                               DFAULT
      SUBROUTINE DOGDRV(NR,N,X,F,G,A,P,XPLS,FPLS,FCN,SX,STEPMX,         DOGDRV
     + STEPTL,DLT,IRETCD,MXTAKE,SC,WRK1,WRK2,WRK3,IPR,                  DOGDRV
     + AJA,ANLS,SHAT,VECT1,VECT2,VECT3,VECT4,VECT5,VECT6,               DOGDRV
     + NRM,NRN,MM,NN,IQ)                                                DOGDRV
C                                                                       DOGDRV
C PURPOSE                                                               DOGDRV
C -------                                                               DOGDRV
C FIND A NEXT NEWTON ITERATE (XPLS) BY THE DOUBLE DOGLEG METHOD         DOGDRV
C                                                                       DOGDRV
C PARAMETERS                                                            DOGDRV
C ----------                                                            DOGDRV
C NR           --> ROW DIMENSION OF MATRIX                              DOGDRV
C N            --> DIMENSION OF PROBLEM                                 DOGDRV
C X(N)         --> OLD ITERATE X[K-1]                                   DOGDRV
C F            --> FUNCTION VALUE AT OLD ITERATE, F(X)                  DOGDRV
C G(N)         --> GRADIENT  AT OLD ITERATE, G(X), OR APPROXIMATE       DOGDRV
C A(N,N)       --> CHOLESKY DECOMPOSITION OF HESSIAN                    DOGDRV
C                  IN LOWER TRIANGULAR PART AND DIAGONAL                DOGDRV
C P(N)         --> NEWTON STEP                                          DOGDRV
C XPLS(N)     <--  NEW ITERATE X[K]                                     DOGDRV
C FPLS        <--  FUNCTION VALUE AT NEW ITERATE, F(XPLS)               DOGDRV
C FCN          --> NAME OF SUBROUTINE TO EVALUATE FUNCTION              DOGDRV
C SX(N)        --> DIAGONAL SCALING MATRIX FOR X                        DOGDRV
C STEPMX       --> MAXIMUM ALLOWABLE STEP SIZE                          DOGDRV
C STEPTL       --> RELATIVE STEP SIZE AT WHICH SUCCESSIVE ITERATES      DOGDRV
C                  CONSIDERED CLOSE ENOUGH TO TERMINATE ALGORITHM       DOGDRV
C DLT         <--> TRUST REGION RADIUS                                  DOGDRV
C                  [RETAIN VALUE BETWEEN SUCCESSIVE CALLS]              DOGDRV
C IRETCD      <--  RETURN CODE                                          DOGDRV
C                    =0 SATISFACTORY XPLS FOUND                         DOGDRV
C                    =1 FAILED TO FIND SATISFACTORY XPLS SUFFICIENTLY   DOGDRV
C                       DISTINCT FROM X                                 DOGDRV
C MXTAKE      <--  BOOLEAN FLAG INDICATING STEP OF MAXIMUM LENGTH USED  DOGDRV
C SC(N)        --> WORKSPACE [CURRENT STEP]                             DOGDRV
C WRK1(N)      --> WORKSPACE (AND PLACE HOLDING ARGUMENT TO TREGUP)     DOGDRV
C WRK2(N)      --> WORKSPACE                                            DOGDRV
C WRK3(N)      --> WORKSPACE                                            DOGDRV
C IPR          --> DEVICE TO WHICH TO SEND OUTPUT                       DOGDRV
C                                                                       DOGDRV
      INTEGER N,IPR,NRM,NRN,MM,NN,IQ,I,NR,IRETCD                        DOGDRV
      DOUBLE PRECISION F,FPLS,STEPMX,STEPTL,DLT,TMP,RNWTLN,CLN          DOGDRV
      DOUBLE PRECISION ETA,FPLSP                                        DOGDRV
      DOUBLE PRECISION X(N),XPLS(N),G(N),P(N)                           DOGDRV
      DOUBLE PRECISION SX(N)                                            DOGDRV
      DOUBLE PRECISION SC(N),WRK1(N),WRK2(N),WRK3(N)                    DOGDRV
      DOUBLE PRECISION A(NR,1)                                          DOGDRV
      DOUBLE PRECISION AJA(NRM,1),ANLS(NRM,1),SHAT(NRN,1)               DOGDRV
      DOUBLE PRECISION VECT1(MM),VECT2(MM),VECT3(MM)                    DOGDRV
      DOUBLE PRECISION VECT4(MM),VECT5(MM),VECT6(MM)                    DOGDRV
      LOGICAL FSTDOG,NWTAKE,MXTAKE                                      DOGDRV
      EXTERNAL FCN                                                      DOGDRV
C                                                                       DOGDRV
      IRETCD=4                                                          DOGDRV
      FSTDOG=.TRUE.                                                     DOGDRV
      TMP=0.                                                            DOGDRV
      DO 5 I=1,N                                                        DOGDRV
        TMP=TMP+(SX(I)*P(I))**2                                         DOGDRV
    5 CONTINUE                                                          DOGDRV
      RNWTLN=SQRT(TMP)                                                  DOGDRV
C$    WRITE(IPR,954) RNWTLN                                             DOGDRV
C                                                                       DOGDRV
  100 CONTINUE                                                          DOGDRV
C                                                                       DOGDRV
C FIND NEW STEP BY DOUBLE DOGLEG ALGORITHM                              DOGDRV
      CALL DOGSTP(NR,N,G,A,P,SX,RNWTLN,DLT,NWTAKE,FSTDOG,               DOGDRV
     +     WRK1,WRK2,CLN,ETA,SC,IPR,STEPMX)                             DOGDRV
C                                                                       DOGDRV
C CHECK NEW POINT AND UPDATE TRUST REGION                               DOGDRV
      CALL TREGUP(NR,N,X,F,G,A,FCN,SC,SX,NWTAKE,STEPMX,STEPTL,DLT,      DOGDRV
     + IRETCD,WRK3,FPLSP,XPLS,FPLS,MXTAKE,IPR,2,WRK1,                   DOGDRV
     + AJA,ANLS,SHAT,VECT1,VECT2,VECT3,VECT4,VECT5,VECT6,               DOGDRV
     + NRM,NRN,MM,NN,IQ)                                                DOGDRV
      IF(IRETCD.LE.1) RETURN                                            DOGDRV
      GO TO 100                                                         DOGDRV
  950 FORMAT(42H DOGDRV    INITIAL TRUST REGION NOT GIVEN.,             DOGDRV
     +       22H  COMPUTE CAUCHY STEP.)                                 DOGDRV
  951 FORMAT(18H DOGDRV    ALPHA =,E20.13/                              DOGDRV
     +       18H DOGDRV    BETA  =,E20.13/                              DOGDRV
     +       18H DOGDRV    DLT   =,E20.13/                              DOGDRV
     +       18H DOGDRV    NWTAKE=,L1    )                              DOGDRV
  952 FORMAT(28H DOGDRV    CURRENT STEP (SC))                           DOGDRV
  954 FORMAT(18H0DOGDRV    RNWTLN=,E20.13)                              DOGDRV
  955 FORMAT(14H DOGDRV       ,5(E20.13,3X))                            DOGDRV
      END                                                               DOGDRV
      SUBROUTINE DOGSTP(NR,N,G,A,P,SX,RNWTLN,DLT,NWTAKE,FSTDOG,         DOGSTP
     +     SSD,V,CLN,ETA,SC,IPR,STEPMX)                                 DOGSTP
C                                                                       DOGSTP
C PURPOSE                                                               DOGSTP
C -------                                                               DOGSTP
C FIND NEW STEP BY DOUBLE DOGLEG ALGORITHM                              DOGSTP
C                                                                       DOGSTP
C PARAMETERS                                                            DOGSTP
C ----------                                                            DOGSTP
C NR           --> ROW DIMENSION OF MATRIX                              DOGSTP
C N            --> DIMENSION OF PROBLEM                                 DOGSTP
C G(N)         --> GRADIENT AT CURRENT ITERATE, G(X)                    DOGSTP
C A(N,N)       --> CHOLESKY DECOMPOSITION OF HESSIAN IN                 DOGSTP
C                  LOWER PART AND DIAGONAL                              DOGSTP
C P(N)         --> NEWTON STEP                                          DOGSTP
C SX(N)        --> DIAGONAL SCALING MATRIX FOR X                        DOGSTP
C RNWTLN       --> NEWTON STEP LENGTH                                   DOGSTP
C DLT         <--> TRUST REGION RADIUS                                  DOGSTP
C NWTAKE      <--> BOOLEAN, =.TRUE. IF NEWTON STEP TAKEN                DOGSTP
C FSTDOG      <--> BOOLEAN, =.TRUE. IF ON FIRST LEG OF DOGLEG           DOGSTP
C SSD(N)      <--> WORKSPACE [CAUCHY STEP TO THE MINIMUM OF THE         DOGSTP
C                  QUADRATIC MODEL IN THE SCALED STEEPEST DESCENT       DOGSTP
C                  DIRECTION] [RETAIN VALUE BETWEEN SUCCESSIVE CALLS]   DOGSTP
C V(N)        <--> WORKSPACE  [RETAIN VALUE BETWEEN SUCCESSIVE CALLS]   DOGSTP
C CLN         <--> CAUCHY LENGTH                                        DOGSTP
C                  [RETAIN VALUE BETWEEN SUCCESSIVE CALLS]              DOGSTP
C ETA              [RETAIN VALUE BETWEEN SUCCESSIVE CALLS]              DOGSTP
C SC(N)       <--  CURRENT STEP                                         DOGSTP
C IPR          --> DEVICE TO WHICH TO SEND OUTPUT                       DOGSTP
C STEPMX       --> MAXIMUM ALLOWABLE STEP SIZE                          DOGSTP
C                                                                       DOGSTP
C INTERNAL VARIABLES                                                    DOGSTP
C ------------------                                                    DOGSTP
C CLN              LENGTH OF CAUCHY STEP                                DOGSTP
C                                                                       DOGSTP
      INTEGER NR,N,IPR,I,J                                              DOGSTP
      DOUBLE PRECISION STEPMX,DLT,RNWTLN,CLN,ETA,ALPHA,BETA,TMP         DOGSTP
      DOUBLE PRECISION DOT1,DOT2,ALAM                                   DOGSTP
      DOUBLE PRECISION G(N),P(N)                                        DOGSTP
      DOUBLE PRECISION SX(N)                                            DOGSTP
      DOUBLE PRECISION SC(N),SSD(N),V(N)                                DOGSTP
      DOUBLE PRECISION A(NR,1)                                          DOGSTP
      DOUBLE PRECISION DDOT                                             DOGSTP
      LOGICAL NWTAKE,FSTDOG                                             DOGSTP
C                                                                       DOGSTP
C CAN WE TAKE NEWTON STEP                                               DOGSTP
C                                                                       DOGSTP
      IF(RNWTLN.GT.DLT) GO TO 100                                       DOGSTP
C     IF(RNWTLN.LE.DLT)                                                 DOGSTP
C     THEN                                                              DOGSTP
        NWTAKE=.TRUE.                                                   DOGSTP
        DO 10 I=1,N                                                     DOGSTP
          SC(I)=P(I)                                                    DOGSTP
   10   CONTINUE                                                        DOGSTP
        DLT=RNWTLN                                                      DOGSTP
C$      WRITE(IPR,951)                                                  DOGSTP
        GO TO 700                                                       DOGSTP
C     ELSE                                                              DOGSTP
C                                                                       DOGSTP
C NEWTON STEP TOO LONG                                                  DOGSTP
C CAUCHY STEP IS ON DOUBLE DOGLEG CURVE                                 DOGSTP
C                                                                       DOGSTP
  100   NWTAKE=.FALSE.                                                  DOGSTP
        IF(.NOT.FSTDOG) GO TO 200                                       DOGSTP
C       IF(FSTDOG)                                                      DOGSTP
C       THEN                                                            DOGSTP
C                                                                       DOGSTP
C         CALCULATE DOUBLE DOGLEG CURVE (SSD)                           DOGSTP
          FSTDOG=.FALSE.                                                DOGSTP
          ALPHA=0.                                                      DOGSTP
          DO 105 I = 1, N                                               DOGSTP
            SSD(I)  = G(I)/SX(I)                                        DOGSTP 
  105     CONTINUE                                                      DOGSTP 
          DO 110 I=1,N                                                  DOGSTP
            ALPHA=ALPHA + SSD(I)*SSD(I)                                 DOGSTP
  110     CONTINUE                                                      DOGSTP
          BETA=0.                                                       DOGSTP
          DO 130 I=1,N                                                  DOGSTP
            TMP=0.                                                      DOGSTP
            DO 120 J=I,N                                                DOGSTP
              TMP=TMP + (A(J,I)/SX(J))*SSD(J)                           DOGSTP
  120       CONTINUE                                                    DOGSTP
            BETA=BETA+TMP*TMP                                           DOGSTP
  130     CONTINUE                                                      DOGSTP
          DO 140 I=1,N                                                  DOGSTP
            SSD(I)=-(ALPHA/BETA)*SSD(I)                                 DOGSTP
  140     CONTINUE                                                      DOGSTP
          CLN=ALPHA*SQRT(ALPHA)/BETA                                    DOGSTP
          ETA=.2 + (.8*ALPHA*ALPHA)/(-BETA*DDOT(N,G,1,P,1))             DOGSTP
          DO 150 I=1,N                                                  DOGSTP
            V(I)=ETA*SX(I)*P(I) - SSD(I)                                DOGSTP
  150     CONTINUE                                                      DOGSTP
          IF (DLT .EQ. (-1.0)) DLT = MIN(CLN, STEPMX)                   DOGSTP
C$        WRITE(IPR,954) ALPHA,BETA,CLN,ETA                             DOGSTP
C$        WRITE(IPR,955)                                                DOGSTP
C$        WRITE(IPR,960) (SSD(I),I=1,N)                                 DOGSTP
C$        WRITE(IPR,956)                                                DOGSTP
C$        WRITE(IPR,960) (V(I),I=1,N)                                   DOGSTP
C       ENDIF                                                           DOGSTP
  200   IF(ETA*RNWTLN.GT.DLT) GO TO 220                                 DOGSTP
C       IF(ETA*RNWTLN .LE. DLT)                                         DOGSTP
C       THEN                                                            DOGSTP
C                                                                       DOGSTP
C         TAKE PARTIAL STEP IN NEWTON DIRECTION                         DOGSTP
C                                                                       DOGSTP
          DO 210 I=1,N                                                  DOGSTP
            SC(I)=(DLT/RNWTLN)*P(I)                                     DOGSTP
  210     CONTINUE                                                      DOGSTP
C$        WRITE(IPR,957)                                                DOGSTP
          GO TO 700                                                     DOGSTP
C       ELSE                                                            DOGSTP
  220     IF(CLN.LT.DLT) GO TO 240                                      DOGSTP
C         IF(CLN.GE.DLT)                                                DOGSTP
C         THEN                                                          DOGSTP
C           TAKE STEP IN STEEPEST DESCENT DIRECTION                     DOGSTP
C                                                                       DOGSTP
            DO 230 I=1,N                                                DOGSTP
              SC(I)=(DLT/CLN)*SSD(I)/SX(I)                              DOGSTP
  230       CONTINUE                                                    DOGSTP
C$          WRITE(IPR,958)                                              DOGSTP
            GO TO 700                                                   DOGSTP
C         ELSE                                                          DOGSTP
C                                                                       DOGSTP
C           CALCULATE CONVEX COMBINATION OF SSD AND ETA*P               DOGSTP
C           WHICH HAS SCALED LENGTH DLT                                 DOGSTP
C                                                                       DOGSTP
  240       DOT1=DDOT(N,V,1,SSD,1)                                      DOGSTP
            DOT2=DDOT(N,V,1,V,1)                                        DOGSTP
            ALAM=(-DOT1+SQRT((DOT1*DOT1)-DOT2*(CLN*CLN-DLT*DLT)))/DOT2  DOGSTP
            DO 250 I=1,N                                                DOGSTP
              SC(I)=(SSD(I) + ALAM*V(I))/SX(I)                          DOGSTP
  250       CONTINUE                                                    DOGSTP
C$          WRITE(IPR,959)                                              DOGSTP
C         ENDIF                                                         DOGSTP
C       ENDIF                                                           DOGSTP
C     ENDIF                                                             DOGSTP
  700 CONTINUE                                                          DOGSTP
C$    WRITE(IPR,952) FSTDOG,NWTAKE,RNWTLN,DLT                           DOGSTP
C$    WRITE(IPR,953)                                                    DOGSTP
C$    WRITE(IPR,960) (SC(I),I=1,N)                                      DOGSTP
      RETURN                                                            DOGSTP
C                                                                       DOGSTP
  951 FORMAT(27H0DOGSTP    TAKE NEWTON STEP)                            DOGSTP
  952 FORMAT(18H DOGSTP    FSTDOG=,L1/                                  DOGSTP
     +       18H DOGSTP    NWTAKE=,L1/                                  DOGSTP
     +       18H DOGSTP    RNWTLN=,E20.13/                              DOGSTP
     +       18H DOGSTP    DLT   =,E20.13)                              DOGSTP
  953 FORMAT(28H DOGSTP    CURRENT STEP (SC))                           DOGSTP
  954 FORMAT(18H DOGSTP    ALPHA =,E20.13/                              DOGSTP
     +       18H DOGSTP    BETA  =,E20.13/                              DOGSTP
     +       18H DOGSTP    CLN   =,E20.13/                              DOGSTP
     +       18H DOGSTP    ETA   =,E20.13)                              DOGSTP
  955 FORMAT(28H DOGSTP    CAUCHY STEP (SSD))                           DOGSTP
  956 FORMAT(12H DOGSTP    V)                                           DOGSTP
  957 FORMAT(48H0DOGSTP    TAKE PARTIAL STEP IN NEWTON DIRECTION)       DOGSTP
  958 FORMAT(50H0DOGSTP    TAKE STEP IN STEEPEST DESCENT DIRECTION)     DOGSTP
  959 FORMAT(39H0DOGSTP    TAKE CONVEX COMBINATION STEP)                DOGSTP
  960 FORMAT(14H DOGSTP       ,5(E20.13,3X))                            DOGSTP
      END                                                               DOGSTP
      SUBROUTINE D1FCN(N,X,G,AJA,ANLS,SHAT,VECT1,VECT2,VECT3,           D1FCN
     +                 VECT4,VECT5,VECT6,NRM,NRN,MM,NN,IQ)              D1FCN
C                                                                       D1FCN
C PURPOSE                                                               D1FCN
C -------                                                               D1FCN
C DUMMY ROUTINE TO PREVENT UNSATISFIED EXTERNAL DIAGNOSTIC              D1FCN
C WHEN SPECIFIC ANALYTIC GRADIENT FUNCTION NOT SUPPLIED.                D1FCN
C                                                                       D1FCN
      INTEGER N,NRM,NRN,MM,NN,IQ                                        D1FCN
      DOUBLE PRECISION X(N),G(N)                                        D1FCN
      DOUBLE PRECISION AJA(NRM,1),ANLS(NRM,1),SHAT(NRN,1)               D1FCN
      DOUBLE PRECISION VECT1(MM),VECT2(MM),VECT3(MM)                    D1FCN
      DOUBLE PRECISION VECT4(MM),VECT5(MM),VECT6(MM)                    D1FCN
      STOP                                                              D1FCN
      END                                                               D1FCN
      SUBROUTINE D2FCN(NR,N,X,H)                                        D2FCN
C                                                                       D2FCN
C PURPOSE                                                               D2FCN
C -------                                                               D2FCN
C DUMMY ROUTINE TO PREVENT UNSATISFIED EXTERNAL DIAGNOSTIC              D2FCN
C WHEN SPECIFIC ANALYTIC HESSIAN FUNCTION NOT SUPPLIED.                 D2FCN
C                                                                       D2FCN
      INTEGER NR,N                                                      D2FCN
      DOUBLE PRECISION X(N),H(NR,1)                                     D2FCN
      STOP                                                              D2FCN
      END                                                               D2FCN
       SUBROUTINE FORSLV (NR,N,A,X,B)                                   FORSLV
C                                                                       FORSLV
C PURPOSE                                                               FORSLV
C --------                                                              FORSLV
C SOLVE  AX=B  WHERE A  IS LOWER TRIANGULAR  MATRIX                     FORSLV
C                                                                       FORSLV
C PARAMETERS                                                            FORSLV
C ---------                                                             FORSLV
C                                                                       FORSLV
C NR            -----> ROW DIMENSION OF MATRIX                          FORSLV
C N             -----> DIMENSION OF PROBLEM                             FORSLV
C A(N,N)        -----> LOWER TRIANGULAR MATRIX (PRESERVED)              FORSLV
C X(N)          <----  SOLUTION VECTOR                                  FORSLV 
C B(N)           ----> RIGHT-HAND SIDE VECTOR                           FORSLV
C                                                                       FORSLV
C NOTE                                                                  FORSLV
C-----                                                                  FORSLV
C THEN VECTORS B AND X MAY SHARE THE SAME STORAGE                       FORSLV
C                                                                       FORSLV
      INTEGER NR,N,I,J,IM1                                              FORSLV
      DOUBLE PRECISION SUM                                              FORSLV
      DOUBLE PRECISION A(NR,1),X(N),B(N)                                FORSLV
C                                                                       FORSLV
C SOLVE LX=B.  (FORWARD  SOLVE)                                         FORSLV
C                                                                       FORSLV
      X(1)=B(1)/A(1,1)                                                  FORSLV
      DO 20 I=2,N                                                       FORSLV
        SUM=0.0                                                         FORSLV
        IM1=I-1                                                         FORSLV
        DO 10 J=1,IM1                                                   FORSLV
          SUM=SUM+A(I,J)*X(J)                                           FORSLV
   10   CONTINUE                                                        FORSLV
        X(I)=(B(I)-SUM)/A(I,I)                                          FORSLV
   20 CONTINUE                                                          FORSLV
      RETURN                                                            FORSLV
      END                                                               FORSLV
      SUBROUTINE FSTOCD (N, X, FCN, SX, RNOISE, G)                      FSTOCD
C PURPOSE                                                               FSTOCD
C -------                                                               FSTOCD
C FIND CENTRAL DIFFERENCE APPROXIMATION G TO THE FIRST DERIVATIVE       FSTOCD
C (GRADIENT) OF THE FUNCTION DEFINED BY FCN AT THE POINT X.             FSTOCD
C                                                                       FSTOCD
C PARAMETERS                                                            FSTOCD
C ----------                                                            FSTOCD
C N            --> DIMENSION OF PROBLEM                                 FSTOCD
C X            --> POINT AT WHICH GRADIENT IS TO BE APPROXIMATED.       FSTOCD
C FCN          --> NAME OF SUBROUTINE TO EVALUATE FUNCTION.             FSTOCD
C SX           --> DIAGONAL SCALING MATRIX FOR X.                       FSTOCD
C RNOISE       --> RELATIVE NOISE IN FCN [F(X)].                        FSTOCD
C G           <--  CENTRAL DIFFERENCE APPROXIMATION TO GRADIENT.        FSTOCD
C                                                                       FSTOCD
C                                                                       FSTOCD
      INTEGER N,I                                                       FSTOCD
      DOUBLE PRECISION RNOISE,THIRD,XTEMPI,FPLUS,FMINUS,STEPI           FSTOCD
      DOUBLE PRECISION X(N)                                             FSTOCD
      DOUBLE PRECISION SX(N)                                            FSTOCD
      DOUBLE PRECISION G(N)                                             FSTOCD
      EXTERNAL FCN                                                      FSTOCD
C                                                                       FSTOCD
C FIND I TH  STEPSIZE, EVALUATE TWO NEIGHBORS IN DIRECTION OF I TH      FSTOCD
C UNIT VECTOR, AND EVALUATE I TH  COMPONENT OF GRADIENT.                FSTOCD
C                                                                       FSTOCD
      THIRD = 1.0/3.0                                                   FSTOCD
      DO 10 I = 1, N                                                    FSTOCD
         STEPI = RNOISE**THIRD * MAX(ABS(X(I)), 1.0/SX(I))              FSTOCD
         XTEMPI = X(I)                                                  FSTOCD
         X(I) = XTEMPI + STEPI                                          FSTOCD
         CALL FCN (N, X, FPLUS)                                         FSTOCD
         X(I) = XTEMPI - STEPI                                          FSTOCD
         CALL FCN (N, X, FMINUS)                                        FSTOCD
         X(I) = XTEMPI                                                  FSTOCD
         G(I) = (FPLUS - FMINUS)/(2.0*STEPI)                            FSTOCD
   10 CONTINUE                                                          FSTOCD
      RETURN                                                            FSTOCD
      END                                                               FSTOCD
      SUBROUTINE FSTOFD(NR,M,N,XPLS,FCN,FPLS,A,SX,RNOISE,FHAT,ICASE,    FSTOFD
     + AJA,ANLS,SHAT,VECT1,VECT2,VECT3,VECT4,VECT5,VECT6,NRM,NRN,       FSTOCD
     + MM,NN,IQ)                                                        FSTOFD
C PURPOSE                                                               FSTOFD
C -------                                                               FSTOFD
C FIND FIRST ORDER FORWARD FINITE DIFFERENCE APPROXIMATION "A" TO THE   FSTOFD
C FIRST DERIVATIVE OF THE FUNCTION DEFINED BY THE SUBPROGRAM "FNAME"    FSTOFD
C EVALUATED AT THE NEW ITERATE "XPLS".                                  FSTOFD
C                                                                       FSTOFD
C                                                                       FSTOFD
C FOR OPTIMIZATION USE THIS ROUTINE TO ESTIMATE:                        FSTOFD
C 1) THE FIRST DERIVATIVE (GRADIENT) OF THE OPTIMIZATION FUNCTION "FCN  FSTOFD
C    ANALYTIC USER ROUTINE HAS BEEN SUPPLIED;                           FSTOFD
C 2) THE SECOND DERIVATIVE (HESSIAN) OF THE OPTIMIZATION FUNCTION       FSTOFD
C    IF NO ANALYTIC USER ROUTINE HAS BEEN SUPPLIED FOR THE HESSIAN BUT  FSTOFD
C    ONE HAS BEEN SUPPLIED FOR THE GRADIENT ("FCN") AND IF THE          FSTOFD
C    OPTIMIZATION FUNCTION IS INEXPENSIVE TO EVALUATE                   FSTOFD
C                                                                       FSTOFD
C NOTE                                                                  FSTOFD
C ----                                                                  FSTOFD
C _M=1 (OPTIMIZATION) ALGORITHM ESTIMATES THE GRADIENT OF THE FUNCTION  FSTOFD
C      (FCN).   FCN(X) # F: R(N)-->R(1)                                 FSTOFD
C _M=N (SYSTEMS) ALGORITHM ESTIMATES THE JACOBIAN OF THE FUNCTION       FSTOFD
C      FCN(X) # F: R(N)-->R(N).                                         FSTOFD
C _M=N (OPTIMIZATION) ALGORITHM ESTIMATES THE HESSIAN OF THE OPTIMIZATIOFSTOFD
C      FUNCTION, WHERE THE HESSIAN IS THE FIRST DERIVATIVE OF "FCN"     FSTOFD
C                                                                       FSTOFD
C PARAMETERS                                                            FSTOFD
C ----------                                                            FSTOFD
C NR           --> ROW DIMENSION OF MATRIX                              FSTOFD
C M            --> NUMBER OF ROWS IN A                                  FSTOFD
C N            --> NUMBER OF COLUMNS IN A; DIMENSION OF PROBLEM         FSTOFD
C XPLS(N)      --> NEW ITERATE:  X[K]                                   FSTOFD
C FCN          --> NAME OF SUBROUTINE TO EVALUATE FUNCTION              FSTOFD
C FPLS(M)      --> _M=1 (OPTIMIZATION) FUNCTION VALUE AT NEW ITERATE:   FSTOFD
C                       FCN(XPLS)                                       FSTOFD
C                  _M=N (OPTIMIZATION) VALUE OF FIRST DERIVATIVE        FSTOFD
C                       (GRADIENT) GIVEN BY USER FUNCTION FCN           FSTOFD
C                  _M=N (SYSTEMS)  FUNCTION VALUE OF ASSOCIATED         FSTOFD
C                       MINIMIZATION FUNCTION                           FSTOFD
C A(NR,N)     <--  FINITE DIFFERENCE APPROXIMATION (SEE NOTE).  ONLY    FSTOFD
C                  LOWER TRIANGULAR MATRIX AND DIAGONAL ARE RETURNED    FSTOFD
C SX(N)        --> DIAGONAL SCALING MATRIX FOR X                        FSTOFD
C RNOISE       --> RELATIVE NOISE IN FCN [F(X)]                         FSTOFD
C FHAT(M)      --> WORKSPACE                                            FSTOFD
C ICASE        --> =1 OPTIMIZATION (GRADIENT)                           FSTOFD
C                  =2 SYSTEMS                                           FSTOFD
C                  =3 OPTIMIZATION (HESSIAN)                            FSTOFD
C                                                                       FSTOFD
C INTERNAL VARIABLES                                                    FSTOFD
C ------------------                                                    FSTOFD
C STEPSZ - STEPSIZE IN THE J-TH VARIABLE DIRECTION                      FSTOFD
C                                                                       FSTOFD
      INTEGER NR,M,N,ICASE,NRM,NRN,MM,NN,IQ,I,J,NM1,JP1                 FSTOFD
      DOUBLE PRECISION RNOISE,STEPSZ,XTMPJ,SQRTR,RSTEPSZ,HALF,ONE       FSTOFD
      DOUBLE PRECISION XPLS(N),FPLS(M)                                  FSTOFD
      DOUBLE PRECISION FHAT(M)                                          FSTOFD
      DOUBLE PRECISION SX(N)                                            FSTOFD
      DOUBLE PRECISION A(NR,1)                                          FSTOFD
      DOUBLE PRECISION AJA(NRM,1),ANLS(NRM,1),SHAT(NRN,1)               FSTOFD
      DOUBLE PRECISION VECT1(MM),VECT2(MM),VECT3(MM)                    FSTOFD
      DOUBLE PRECISION VECT4(MM),VECT5(MM),VECT6(MM)                    FSTOFD
      DATA HALF,ONE/0.50D0,1.0D0/                                       FSTOFD
      EXTERNAL FCN                                                      FSTOFD
C                                                                       FSTOFD
C FIND J-TH COLUMN OF A                                                 FSTOFD
C EACH COLUMN IS DERIVATIVE OF F(FCN) WITH RESPECT TO XPLS(J)           FSTOFD
C                                                                       FSTOFD
      SQRTR = SQRT(RNOISE)                                              FSTOFD
      DO 30 J=1,N                                                       FSTOFD
        STEPSZ=SQRTR*MAX(ABS(XPLS(J)),ONE/SX(J))                        FSTOFD
        XTMPJ=XPLS(J)                                                   FSTOFD
        XPLS(J)=XTMPJ+STEPSZ                                            FSTOFD
        CALL FCN(N,XPLS,FHAT,AJA,ANLS,SHAT,VECT1,VECT2,VECT3,           FSTOFD
     +  VECT4,VECT5,VECT6,NRM,NRN,MM,NN,IQ)                             FSTOFD
        XPLS(J)=XTMPJ                                                   FSTOFD
        RSTEPSZ = ONE/STEPSZ                                            FSTOFD
        DO 20 I=1,M                                                     FSTOFD
          A(I,J)=(FHAT(I)-FPLS(I))*RSTEPSZ                              FSTOFD
   20   CONTINUE                                                        FSTOFD
   30 CONTINUE                                                          FSTOFD
      IF(ICASE.NE.3) RETURN                                             FSTOFD
C                                                                       FSTOFD
C IF COMPUTING HESSIAN, A MUST BE SYMMETRIC                             FSTOFD
C                                                                       FSTOFD
      IF(N.EQ.1) RETURN                                                 FSTOFD
      NM1=N-1                                                           FSTOFD
      DO 50 J=1,NM1                                                     FSTOFD
        JP1=J+1                                                         FSTOFD
        DO 40 I=JP1,M                                                   FSTOFD
          A(I,J)=(A(I,J)+A(J,I))*HALF                                   FSTOFD
   40   CONTINUE                                                        FSTOFD
   50 CONTINUE                                                          FSTOFD
      RETURN                                                            FSTOFD
      END                                                               FSTOFD
      SUBROUTINE GRDCHK(N,X,FCN,F,G,TYPSIZ,SX,FSCALE,RNF,               GRDCHK
     + ANALTL,WRK1,MSG,IPR,AJA,ANLS,SHAT,VECT1,VECT2,VECT3,             GRDCHK
     + VECT4,VECT5,VECT6,NRM,NRN,MM,NN,IQ)                              GRDCHK
C                                                                       GRDCHK
C PURPOSE                                                               GRDCHK
C -------                                                               GRDCHK
C CHECK ANALYTIC GRADIENT AGAINST ESTIMATED GRADIENT                    GRDCHK
C                                                                       GRDCHK
C PARAMETERS                                                            GRDCHK
C ----------                                                            GRDCHK
C N            --> DIMENSION OF PROBLEM                                 GRDCHK
C X(N)         --> ESTIMATE TO A ROOT OF FCN                            GRDCHK
C FCN          --> NAME OF SUBROUTINE TO EVALUATE OPTIMIZATION FUNCTION GRDCHK
C                  MUST BE DECLARED EXTERNAL IN CALLING ROUTINE         GRDCHK
C                       FCN:  R(N) --> R(1)                             GRDCHK
C F            --> FUNCTION VALUE:  FCN(X)                              GRDCHK
C G(N)         --> GRADIENT:  G(X)                                      GRDCHK
C TYPSIZ(N)    --> TYPICAL SIZE FOR EACH COMPONENT OF X                 GRDCHK
C SX(N)        --> DIAGONAL SCALING MATRIX:  SX(I)=1./TYPSIZ(I)         GRDCHK
C FSCALE       --> ESTIMATE OF SCALE OF OBJECTIVE FUNCTION FCN          GRDCHK
C RNF          --> RELATIVE NOISE IN OPTIMIZATION FUNCTION FCN          GRDCHK
C ANALTL       --> TOLERANCE FOR COMPARISON OF ESTIMATED AND            GRDCHK
C                  ANALYTICAL GRADIENTS                                 GRDCHK
C WRK1(N)      --> WORKSPACE                                            GRDCHK
C MSG         <--  MESSAGE OR ERROR CODE                                GRDCHK
C                    ON OUTPUT: =-21, PROBABLE CODING ERROR OF GRADIENT GRDCHK
C IPR          --> DEVICE TO WHICH TO SEND OUTPUT                       GRDCHK
C                                                                       GRDCHK
      INTEGER N,MSG,IPR,NRM,NRN,MM,NN,KER,IQ,I                          GRDCHK
      DOUBLE PRECISION F,FSCALE,RNF,ANALTL,GS                           GRDCHK
      DOUBLE PRECISION X(N),G(N)                                        GRDCHK
      DOUBLE PRECISION SX(N),TYPSIZ(N)                                  GRDCHK
      DOUBLE PRECISION WRK1(N),WRK(1)                                   GRDCHK
      DOUBLE PRECISION AJA(NRM,1),ANLS(NRM,1),SHAT(NRN,1)               GRDCHK
      DOUBLE PRECISION VECT1(MM),VECT2(MM),VECT3(MM)                    GRDCHK
      DOUBLE PRECISION VECT4(MM),VECT5(MM),VECT6(MM)                    GRDCHK
      EXTERNAL FCN                                                      GRDCHK
C                                                                       GRDCHK
C COMPUTE FIRST ORDER FINITE DIFFERENCE GRADIENT AND COMPARE TO         GRDCHK
C ANALYTIC GRADIENT.                                                    GRDCHK
C                                                                       GRDCHK
      CALL FSTOFD(1,1,N,X,FCN,F,WRK1,SX,RNF,WRK,1,                      GRDCHK
     + AJA,ANLS,SHAT,VECT1,VECT2,VECT3,VECT4,VECT5,VECT6,NRM,NRN,       GRDCHK
     + MM,NN,IQ)                                                        GRDCHK
      KER=0                                                             GRDCHK
      DO 5 I=1,N                                                        GRDCHK
        GS=MAX(ABS(F),FSCALE)/MAX(ABS(X(I)),TYPSIZ(I))                  GRDCHK
        IF(ABS(G(I)-WRK1(I)).GT.MAX(ABS(G(I)),GS)*ANALTL) KER=1         GRDCHK
    5 CONTINUE                                                          GRDCHK
      IF(KER.EQ.0) GO TO 20                                             GRDCHK
C$        WRITE(IPR,901)                                                GRDCHK
C$        WRITE(IPR,902) (I,G(I),WRK1(I),I=1,N)                         GRDCHK
          MSG=-21                                                       GRDCHK
   20 CONTINUE                                                          GRDCHK
      RETURN                                                            GRDCHK
  901 FORMAT(47H0GRDCHK    PROBABLE ERROR IN CODING OF ANALYTIC,        GRDCHK
     +       19H GRADIENT FUNCTION./                                    GRDCHK
     +       16H GRDCHK     COMP,12X,8HANALYTIC,12X,8HESTIMATE)         GRDCHK
  902 FORMAT(11H GRDCHK    ,I5,3X,E20.13,3X,E20.13)                     GRDCHK
      END                                                               GRDCHK
      SUBROUTINE HESCHK(NR,N,X,FCN,D1FCN,D2FCN,F,G,A,TYPSIZ,SX,RNF,     HESCHK
     +     ANALTL,IAGFLG,UDIAG,WRK1,WRK2,MSG,IPR)                       HESCHK
C                                                                       HESCHK
C PURPOSE                                                               HESCHK
C -------                                                               HESCHK
C CHECK ANALYTIC HESSIAN AGAINST ESTIMATED HESSIAN                      HESCHK
C  (THIS MAY BE DONE ONLY IF THE USER SUPPLIED ANALYTIC HESSIAN         HESCHK
C   D2FCN FILLS ONLY THE LOWER TRIANGULAR PART AND DIAGONAL OF A)       HESCHK
C                                                                       HESCHK
C PARAMETERS                                                            HESCHK
C ----------                                                            HESCHK
C NR           --> ROW DIMENSION OF MATRIX                              HESCHK
C N            --> DIMENSION OF PROBLEM                                 HESCHK
C X(N)         --> ESTIMATE TO A ROOT OF FCN                            HESCHK
C FCN          --> NAME OF SUBROUTINE TO EVALUATE OPTIMIZATION FUNCTION HESCHK
C                  MUST BE DECLARED EXTERNAL IN CALLING ROUTINE         HESCHK
C                       FCN:  R(N) --> R(1)                             HESCHK
C D1FCN        --> NAME OF SUBROUTINE TO EVALUATE GRADIENT OF FCN.      HESCHK
C                  MUST BE DECLARED EXTERNAL IN CALLING ROUTINE         HESCHK
C D2FCN        --> NAME OF SUBROUTINE TO EVALUATE HESSIAN OF FCN.       HESCHK
C                  MUST BE DECLARED EXTERNAL IN CALLING ROUTINE         HESCHK
C F            --> FUNCTION VALUE:  FCN(X)                              HESCHK
C G(N)        <--  GRADIENT:  G(X)                                      HESCHK
C A(N,N)      <--  ON EXIT:  HESSIAN IN LOWER TRIANGULAR PART AND DIAG  HESCHK
C TYPSIZ(N)    --> TYPICAL SIZE FOR EACH COMPONENT OF X                 HESCHK
C SX(N)        --> DIAGONAL SCALING MATRIX:  SX(I)=1./TYPSIZ(I)         HESCHK
C RNF          --> RELATIVE NOISE IN OPTIMIZATION FUNCTION FCN          HESCHK
C ANALTL       --> TOLERANCE FOR COMPARISON OF ESTIMATED AND            HESCHK
C                  ANALYTICAL GRADIENTS                                 HESCHK
C IAGFLG       --> =1 IF ANALYTIC GRADIENT SUPPLIED                     HESCHK
C UDIAG(N)     --> WORKSPACE                                            HESCHK
C WRK1(N)      --> WORKSPACE                                            HESCHK
C WRK2(N)      --> WORKSPACE                                            HESCHK
C MSG         <--> MESSAGE OR ERROR CODE                                HESCHK
C                    ON INPUT : IF =1XX DO NOT COMPARE ANAL + EST HESS  HESCHK
C                    ON OUTPUT: =-22, PROBABLE CODING ERROR OF HESSIAN  HESCHK
C IPR          --> DEVICE TO WHICH TO SEND OUTPUT                       HESCHK
C                                                                       HESCHK
      INTEGER NR,N,IAGFLG,MSG,IPR,KER,J,I,JP1,IM1                       HESCHK
      DOUBLE PRECISION F,RNF,ANALTL,HS                                  HESCHK
      DOUBLE PRECISION X(N),G(N),A(NR,1)                                HESCHK
      DOUBLE PRECISION TYPSIZ(N),SX(N)                                  HESCHK
      DOUBLE PRECISION UDIAG(N),WRK1(N),WRK2(N)                         HESCHK
      EXTERNAL FCN,D1FCN,D2FCN                                          HESCHK
C                                                                       HESCHK
C COMPUTE FINITE DIFFERENCE APPROXIMATION A TO THE HESSIAN.             HESCHK
C                                                                       HESCHK
C     IF(IAGFLG.EQ.1) CALL FSTOFD(NR,N,N,X,D1FCN,G,A,SX,RNF,WRK1,3)     HESCHK
      IF(IAGFLG.NE.1) CALL SNDOFD(NR,N,X,FCN,F,A,SX,RNF,WRK1,WRK2)      HESCHK
C                                                                       HESCHK
      KER=0                                                             HESCHK
C                                                                       HESCHK
C COPY LOWER TRIANGULAR PART OF "A" TO UPPER TRIANGULAR PART            HESCHK
C AND DIAGONAL OF "A" TO UDIAG                                          HESCHK
C                                                                       HESCHK
      DO 30 J=1,N                                                       HESCHK
        UDIAG(J)=A(J,J)                                                 HESCHK
        DO 25 I=J+1,N                                                   HESCHK
          A(J,I)=A(I,J)                                                 HESCHK
   25   CONTINUE                                                        HESCHK
   30 CONTINUE                                                          HESCHK
C                                                                       HESCHK
C COMPUTE ANALYTIC HESSIAN AND COMPARE TO FINITE DIFFERENCE             HESCHK
C APPROXIMATION.                                                        HESCHK
C                                                                       HESCHK
      CALL D2FCN(NR,N,X,A)                                              HESCHK
      DO 40 J=1,N                                                       HESCHK
        HS=MAX(ABS(G(J)),1.0D0)/MAX(ABS(X(J)),TYPSIZ(J))                HESCHK
        IF(ABS(A(J,J)-UDIAG(J)).GT.MAX(ABS(UDIAG(J)),HS)*ANALTL)        HESCHK
     +       KER=1                                                      HESCHK
        IF(J.EQ.N) GO TO 40                                             HESCHK
        JP1=J+1                                                         HESCHK
        DO 35 I=JP1,N                                                   HESCHK
          IF(ABS(A(I,J)-A(J,I)).GT.MAX(ABS(A(I,J)),HS)*ANALTL) KER=1    HESCHK
   35   CONTINUE                                                        HESCHK
   40 CONTINUE                                                          HESCHK
C                                                                       HESCHK
      IF(KER.EQ.0) GO TO 90                                             HESCHK
        WRITE(IPR,901)                                                  HESCHK
        DO 50 I=1,N                                                     HESCHK
          IF(I.EQ.1) GO TO 45                                           HESCHK
          IM1=I-1                                                       HESCHK
          DO 43 J=1,IM1                                                 HESCHK
            WRITE(IPR,902) I,J,A(I,J),A(J,I)                            HESCHK
   43     CONTINUE                                                      HESCHK
   45     WRITE(IPR,902) I,I,A(I,I),UDIAG(I)                            HESCHK
   50   CONTINUE                                                        HESCHK
        MSG=-22                                                         HESCHK
C     ENDIF                                                             HESCHK
   90 CONTINUE                                                          HESCHK
      RETURN                                                            HESCHK
  901 FORMAT(47H HESCHK    PROBABLE ERROR IN CODING OF ANALYTIC,        HESCHK
     +       18H HESSIAN FUNCTION./                                     HESCHK
     +       21H HESCHK      ROW  COL,14X,8HANALYTIC,14X,10H(ESTIMATE)) HESCHK
  902 FORMAT(11H HESCHK    ,2I5,2X,E20.13,2X,1H(,E20.13,1H))            HESCHK
      END                                                               HESCHK
      SUBROUTINE HOOKDR(NR,N,X,F,G,A,UDIAG,P,XPLS,FPLS,FCN,SX,STEPMX,   HOOKDR
     +     STEPTL,DLT,IRETCD,MXTAKE,AMU,DLTP,PHI,PHIP0,                 HOOKDR
     +     SC,XPLSP,WRK0,EPSM,ITNCNT,IPR)                               HOOKDR
C                                                                       HOOKDR
C PURPOSE                                                               HOOKDR
C -------                                                               HOOKDR
C FIND A NEXT NEWTON ITERATE (XPLS) BY THE MORE-HEBDON METHOD           HOOKDR
C                                                                       HOOKDR
C PARAMETERS                                                            HOOKDR
C ----------                                                            HOOKDR
C NR           --> ROW DIMENSION OF MATRIX                              HOOKDR
C N            --> DIMENSION OF PROBLEM                                 HOOKDR
C X(N)         --> OLD ITERATE X[K-1]                                   HOOKDR
C F            --> FUNCTION VALUE AT OLD ITERATE, F(X)                  HOOKDR
C G(N)         --> GRADIENT AT OLD ITERATE, G(X), OR APPROXIMATE        HOOKDR
C A(N,N)       --> CHOLESKY DECOMPOSITION OF HESSIAN IN LOWER           HOOKDR
C                  TRIANGULAR PART AND DIAGONAL.                        HOOKDR
C                  HESSIAN IN UPPER TRIANGULAR PART AND UDIAG.          HOOKDR
C UDIAG(N)     --> DIAGONAL OF HESSIAN IN A(.,.)                        HOOKDR
C P(N)         --> NEWTON STEP                                          HOOKDR
C XPLS(N)     <--  NEW ITERATE X[K]                                     HOOKDR
C FPLS        <--  FUNCTION VALUE AT NEW ITERATE, F(XPLS)               HOOKDR
C FCN          --> NAME OF SUBROUTINE TO EVALUATE FUNCTION              HOOKDR
C SX(N)        --> DIAGONAL SCALING MATRIX FOR X                        HOOKDR
C STEPMX       --> MAXIMUM ALLOWABLE STEP SIZE                          HOOKDR
C STEPTL       --> RELATIVE STEP SIZE AT WHICH SUCCESSIVE ITERATES      HOOKDR
C                  CONSIDERED CLOSE ENOUGH TO TERMINATE ALGORITHM       HOOKDR
C DLT         <--> TRUST REGION RADIUS                                  HOOKDR
C IRETCD      <--  RETURN CODE                                          HOOKDR
C                    =0 SATISFACTORY XPLS FOUND                         HOOKDR
C                    =1 FAILED TO FIND SATISFACTORY XPLS SUFFICIENTLY   HOOKDR
C                       DISTINCT FROM X                                 HOOKDR
C MXTAKE      <--  BOOLEAN FLAG INDICATING STEP OF MAXIMUM LENGTH USED  HOOKDR
C AMU         <--> [RETAIN VALUE BETWEEN SUCCESSIVE CALLS]              HOOKDR
C DLTP        <--> [RETAIN VALUE BETWEEN SUCCESSIVE CALLS]              HOOKDR
C PHI         <--> [RETAIN VALUE BETWEEN SUCCESSIVE CALLS]              HOOKDR
C PHIP0       <--> [RETAIN VALUE BETWEEN SUCCESSIVE CALLS]              HOOKDR
C SC(N)        --> WORKSPACE                                            HOOKDR
C XPLSP(N)     --> WORKSPACE                                            HOOKDR
C WRK0(N)      --> WORKSPACE                                            HOOKDR
C EPSM         --> MACHINE EPSILON                                      HOOKDR
C ITNCNT       --> ITERATION COUNT                                      HOOKDR
C IPR          --> DEVICE TO WHICH TO SEND OUTPUT                       HOOKDR
C                                                                       HOOKDR
      INTEGER NR,N,IRETCD,ITNCNT,IPR,I,J                                HOOKDR
      DOUBLE PRECISION FPLS,STEPMX,STEPTL,DLT,AMU,DLTP,PHI              HOOKDR
      DOUBLE PRECISION PHIP0,EPSM,TMP,RNWTLN,ALPHA,BETA,F,ZERO,ONE      HOOKDR
      DOUBLE PRECISION X(N),G(N),P(N),XPLS(N),SX(N)                     HOOKDR
      DOUBLE PRECISION A(NR,1),UDIAG(N)                                 HOOKDR
      DOUBLE PRECISION SC(N),XPLSP(N),WRK0(N)                           HOOKDR
      LOGICAL MXTAKE,NWTAKE                                             HOOKDR
      LOGICAL FSTIME                                                    HOOKDR
      DATA ZERO,ONE/0.0D0,1.0D0/                                        HOOKDR
      EXTERNAL FCN                                                      HOOKDR
C                                                                       HOOKDR
      IRETCD=4                                                          HOOKDR
      FSTIME=.TRUE.                                                     HOOKDR
      TMP=ZERO                                                          HOOKDR
      DO 5 I=1,N                                                        HOOKDR
        TMP=TMP+(SX(I)*P(I))**2                                         HOOKDR
    5 CONTINUE                                                          HOOKDR
      RNWTLN=SQRT(TMP)                                                  HOOKDR
C$    WRITE(IPR,954) RNWTLN                                             HOOKDR
C                                                                       HOOKDR
      IF(ITNCNT.EQ.1) THEN                                              HOOKDR
        AMU=ZERO                                                        HOOKDR
C                                                                       HOOKDR
C       IF FIRST ITERATION AND TRUST REGION NOT PROVIDED BY USER,       HOOKDR
C       COMPUTE INITIAL TRUST REGION.                                   HOOKDR
C                                                                       HOOKDR
        IF(DLT.EQ. -ONE) THEN                                           HOOKDR
          ALPHA=ZERO                                                    HOOKDR
          DO 10 I=1,N                                                   HOOKDR
            ALPHA=ALPHA+(G(I)/SX(I))**2                                 HOOKDR
   10     CONTINUE                                                      HOOKDR
          BETA=ZERO                                                     HOOKDR
          DO 30 I=1,N                                                   HOOKDR
            TMP=ZERO                                                    HOOKDR
            DO 20 J=I,N                                                 HOOKDR
              TMP=TMP + (A(J,I)*G(J))/(SX(J)*SX(J))                     HOOKDR
   20       CONTINUE                                                    HOOKDR
            BETA=BETA+TMP*TMP                                           HOOKDR
   30     CONTINUE                                                      HOOKDR
          DLT=ALPHA*SQRT(ALPHA)/BETA                                    HOOKDR
          DLT = MIN(DLT, STEPMX)                                        HOOKDR
C$        WRITE(IPR,950)                                                HOOKDR
C$        WRITE(IPR,951) ALPHA,BETA,DLT                                 HOOKDR
        ENDIF                                                           HOOKDR
      ENDIF                                                             HOOKDR
C                                                                       HOOKDR
  100 CONTINUE                                                          HOOKDR
C                                                                       HOOKDR
C FIND NEW STEP BY MORE-HEBDON ALGORITHM                                HOOKDR
      CALL HOOKST(NR,N,G,A,UDIAG,P,SX,RNWTLN,DLT,AMU,                   HOOKDR
     +     DLTP,PHI,PHIP0,FSTIME,SC,NWTAKE,WRK0,EPSM,IPR)               HOOKDR
      DLTP=DLT                                                          HOOKDR
C                                                                       HOOKDR
C CHECK NEW POINT AND UPDATE TRUST REGION                               HOOKDR
C     CALL TREGUP(NR,N,X,F,G,A,FCN,SC,SX,NWTAKE,STEPMX,STEPTL,          HOOKDR
C    +         DLT,IRETCD,XPLSP,FPLSP,XPLS,FPLS,MXTAKE,IPR,3,UDIAG)     HOOKDR
      IF(IRETCD.LE.1) RETURN                                            HOOKDR
      GO TO 100                                                         HOOKDR
C                                                                       HOOKDR
  950 FORMAT(43H HOOKDR    INITIAL TRUST REGION NOT GIVEN. ,            HOOKDR
     +       21H COMPUTE CAUCHY STEP.)                                  HOOKDR
  951 FORMAT(18H HOOKDR    ALPHA =,E20.13/                              HOOKDR
     +       18H HOOKDR    BETA  =,E20.13/                              HOOKDR
     +       18H HOOKDR    DLT   =,E20.13)                              HOOKDR
  952 FORMAT(28H HOOKDR    CURRENT STEP (SC))                           HOOKDR
  954 FORMAT(18H0HOOKDR    RNWTLN=,E20.13)                              HOOKDR
  955 FORMAT(14H HOOKDR       ,5(E20.13,3X))                            HOOKDR
      END                                                               HOOKDR
      SUBROUTINE HOOKST(NR,N,G,A,UDIAG,P,SX,RNWTLN,DLT,AMU,             HOOKST
     +     DLTP,PHI,PHIP0,FSTIME,SC,NWTAKE,WRK0,EPSM,IPR)               HOOKST
C                                                                       HOOKST
C PURPOSE                                                               HOOKST
C -------                                                               HOOKST
C FIND NEW STEP BY MORE-HEBDON ALGORITHM                                HOOKST
C                                                                       HOOKST
C PARAMETERS                                                            HOOKST
C ----------                                                            HOOKST
C NR           --> ROW DIMENSION OF MATRIX                              HOOKST
C N            --> DIMENSION OF PROBLEM                                 HOOKST
C G(N)         --> GRADIENT AT CURRENT ITERATE, G(X)                    HOOKST
C A(N,N)       --> CHOLESKY DECOMPOSITION OF HESSIAN IN                 HOOKST
C                  LOWER TRIANGULAR PART AND DIAGONAL.                  HOOKST
C                  HESSIAN OR APPROX IN UPPER TRIANGULAR PART           HOOKST
C UDIAG(N)     --> DIAGONAL OF HESSIAN IN A(.,.)                        HOOKST
C P(N)         --> NEWTON STEP                                          HOOKST
C SX(N)        --> DIAGONAL SCALING MATRIX FOR N                        HOOKST
C RNWTLN       --> NEWTON STEP LENGTH                                   HOOKST
C DLT         <--> TRUST REGION RADIUS                                  HOOKST
C AMU         <--> [RETAIN VALUE BETWEEN SUCCESSIVE CALLS]              HOOKST
C DLTP         --> TRUST REGION RADIUS AT LAST EXIT FROM THIS ROUTINE   HOOKST
C PHI         <--> [RETAIN VALUE BETWEEN SUCCESSIVE CALLS]              HOOKST
C PHIP0       <--> [RETAIN VALUE BETWEEN SUCCESSIVE CALLS]              HOOKST
C FSTIME      <--> BOOLEAN. =.TRUE. IF FIRST ENTRY TO THIS ROUTINE      HOOKST
C                  DURING K-TH ITERATION                                HOOKST
C SC(N)       <--  CURRENT STEP                                         HOOKST
C NWTAKE      <--  BOOLEAN, =.TRUE. IF NEWTON STEP TAKEN                HOOKST
C WRK0         --> WORKSPACE                                            HOOKST
C EPSM         --> MACHINE EPSILON                                      HOOKST
C IPR          --> DEVICE TO WHICH TO SEND OUTPUT                       HOOKST
C                                                                       HOOKST
      INTEGER NR,N,IPR,I,JP1,J                                          HOOKST
      DOUBLE PRECISION RNWTLN,DLT,AMU,DLTP,PHI,PHIP0,HI,ALO,PHIP        HOOKST
      DOUBLE PRECISION AMUUP,ADDMAX,STEPLN,AMULO,EPSM,ZERO              HOOKST
      DOUBLE PRECISION G(N),P(N),SX(N),SC(N),WRK0(N)                    HOOKST
      DOUBLE PRECISION A(NR,1),UDIAG(N)                                 HOOKST
      DOUBLE PRECISION DNRM2                                            HOOKST
      LOGICAL NWTAKE,DONE                                               HOOKST
      LOGICAL FSTIME                                                    HOOKST
      DATA ZERO/0.0D0/                                                  HOOKST
C                                                                       HOOKST
C HI AND ALO ARE CONSTANTS USED IN THIS ROUTINE.                        HOOKST
C CHANGE HERE IF OTHER VALUES ARE TO BE SUBSTITUTED.                    HOOKST
      HI=1.5                                                            HOOKST
      ALO=.75                                                           HOOKST
C -----                                                                 HOOKST
      IF(RNWTLN.LE.HI*DLT) THEN                                         HOOKST
C                                                                       HOOKST
C       TAKE NEWTON STEP                                                HOOKST
C                                                                       HOOKST
        NWTAKE=.TRUE.                                                   HOOKST
        DO 10 I=1,N                                                     HOOKST
          SC(I)=P(I)                                                    HOOKST
   10   CONTINUE                                                        HOOKST
        DLT=MIN(DLT,RNWTLN)                                             HOOKST
        AMU=0.                                                          HOOKST
C$      WRITE(IPR,951)                                                  HOOKST
        RETURN                                                          HOOKST
      ENDIF                                                             HOOKST
C                                                                       HOOKST
C     NEWTON STEP NOT TAKEN                                             HOOKST
C                                                                       HOOKST
C$    WRITE(IPR,952)                                                    HOOKST
      NWTAKE=.FALSE.                                                    HOOKST
C                                                                       HOOKST
C SET PHIP TO 1.0 FOR COMPILATION. THIS SUBROUTINE IS NOT CURRENTLY     HOOKST
C USED BY TENSOLVE.                                                     HOOKST
C                                                                       HOOKST
      PHIP = 1.0                                                        HOOKST
      IF(AMU.GT.ZERO) THEN                                              HOOKST
        AMU=AMU - (PHI+DLTP) *((DLTP-DLT)+PHI)/(DLT*PHIP)               HOOKST
C$      WRITE(IPR,956) AMU                                              HOOKST
      ENDIF                                                             HOOKST
      PHI=RNWTLN-DLT                                                    HOOKST
      IF(FSTIME) THEN                                                   HOOKST
        DO 25 I=1,N                                                     HOOKST
          WRK0(I)=SX(I)*SX(I)*P(I)                                      HOOKST
   25   CONTINUE                                                        HOOKST
C                                                                       HOOKST
C         SOLVE L*Y = (SX**2)*P                                         HOOKST
C                                                                       HOOKST
        CALL FORSLV(NR,N,A,WRK0,WRK0)                                   HOOKST
        PHIP0=-DNRM2(N,WRK0,1)**2/RNWTLN                                HOOKST
        FSTIME=.FALSE.                                                  HOOKST
      ENDIF                                                             HOOKST
      PHIP=PHIP0                                                        HOOKST
      AMULO=-PHI/PHIP                                                   HOOKST
      AMUUP=0.0                                                         HOOKST
      DO 30 I=1,N                                                       HOOKST
        AMUUP=AMUUP+(G(I)*G(I))/(SX(I)*SX(I))                           HOOKST
   30 CONTINUE                                                          HOOKST
      AMUUP=SQRT(AMUUP)/DLT                                             HOOKST
      DONE=.FALSE.                                                      HOOKST
C$    WRITE(IPR,956) AMU                                                HOOKST
C$    WRITE(IPR,959) PHI                                                HOOKST
C$    WRITE(IPR,960) PHIP                                               HOOKST
C$    WRITE(IPR,957) AMULO                                              HOOKST
C$    WRITE(IPR,958) AMUUP                                              HOOKST
C                                                                       HOOKST
C       TEST VALUE OF AMU; GENERATE NEXT AMU IF NECESSARY               HOOKST
C                                                                       HOOKST
  100 CONTINUE                                                          HOOKST
      IF(DONE) RETURN                                                   HOOKST
C$    WRITE(IPR,962)                                                    HOOKST
      IF(AMU.GE.AMULO .AND. AMU.LE.AMUUP) GO TO 110                     HOOKST
C     IF(AMU.LT.AMULO .OR.  AMU.GT.AMUUP)                               HOOKST
C     THEN                                                              HOOKST
        AMU=MAX(SQRT(AMULO*AMUUP),AMUUP*1.0E-3)                         HOOKST
C$      WRITE(IPR,956) AMU                                              HOOKST
C     ENDIF                                                             HOOKST
  110 CONTINUE                                                          HOOKST
C                                                                       HOOKST
C     COPY (H,UDIAG) TO L                                               HOOKST
C     WHERE H <-- H+AMU*(SX**2) [DO NOT ACTUALLY CHANGE (H,UDIAG)]      HOOKST
      DO 130 J=1,N                                                      HOOKST
        A(J,J)=UDIAG(J) + AMU*SX(J)*SX(J)                               HOOKST
        IF(J.EQ.N) GO TO 130                                            HOOKST
        JP1=J+1                                                         HOOKST
        DO 120 I=JP1,N                                                  HOOKST
          A(I,J)=A(J,I)                                                 HOOKST
  120   CONTINUE                                                        HOOKST
  130 CONTINUE                                                          HOOKST
C                                                                       HOOKST
C     FACTOR H=L(L+)                                                    HOOKST
C                                                                       HOOKST
      CALL CHOLDC(NR,N,A,ZERO,SQRT(EPSM),ADDMAX)                        HOOKST
C                                                                       HOOKST
C     SOLVE H*P = L(L+)*SC = -G                                         HOOKST
C                                                                       HOOKST
      DO 140 I=1,N                                                      HOOKST
        WRK0(I)=-G(I)                                                   HOOKST
  140 CONTINUE                                                          HOOKST
      CALL LLTSLV(NR,N,A,SC,WRK0)                                       HOOKST
C$    WRITE(IPR,955)                                                    HOOKST
C$    WRITE(IPR,963) (SC(I),I=1,N)                                      HOOKST
C                                                                       HOOKST
C     RESET H.  NOTE SINCE UDIAG HAS NOT BEEN DESTROYED WE NEED DO      HOOKST
C     NOTHING HERE.  H IS IN THE UPPER PART AND IN UDIAG, STILL INTACT  HOOKST
C                                                                       HOOKST
      STEPLN=0.                                                         HOOKST
      DO 150 I=1,N                                                      HOOKST
        STEPLN=STEPLN + SX(I)*SX(I)*SC(I)*SC(I)                         HOOKST
  150 CONTINUE                                                          HOOKST
      STEPLN=SQRT(STEPLN)                                               HOOKST
      PHI=STEPLN-DLT                                                    HOOKST
      DO 160 I=1,N                                                      HOOKST
        WRK0(I)=SX(I)*SX(I)*SC(I)                                       HOOKST
  160 CONTINUE                                                          HOOKST
      CALL FORSLV(NR,N,A,WRK0,WRK0)                                     HOOKST
      PHIP=-DNRM2(N,WRK0,1)**2/STEPLN                                   HOOKST
C$    WRITE(IPR,961) DLT,STEPLN                                         HOOKST
C$    WRITE(IPR,959) PHI                                                HOOKST
C$    WRITE(IPR,960) PHIP                                               HOOKST
      IF((ALO*DLT.GT.STEPLN .OR. STEPLN.GT.HI*DLT) .AND.                HOOKST
     +     (AMUUP-AMULO.GT.0.)) GO TO 170                               HOOKST
C     IF((ALO*DLT.LE.STEPLN .AND. STEPLN.LE.HI*DLT) .OR.                HOOKST
C          (AMUUP-AMULO.LE.0.))                                         HOOKST
C     THEN                                                              HOOKST
C                                                                       HOOKST
C       SC IS ACCEPTABLE HOOKSTEP                                       HOOKST
C                                                                       HOOKST
C$      WRITE(IPR,954)                                                  HOOKST
        DONE=.TRUE.                                                     HOOKST
        GO TO 100                                                       HOOKST
C     ELSE                                                              HOOKST
C                                                                       HOOKST
C       SC NOT ACCEPTABLE HOOKSTEP.  SELECT NEW AMU                     HOOKST
C                                                                       HOOKST
  170   CONTINUE                                                        HOOKST
C$      WRITE(IPR,953)                                                  HOOKST
        AMULO=MAX(AMULO,AMU-(PHI/PHIP))                                 HOOKST
        IF(PHI.LT.0.) AMUUP=MIN(AMUUP,AMU)                              HOOKST
        AMU=AMU-(STEPLN*PHI)/(DLT*PHIP)                                 HOOKST
C$      WRITE(IPR,956) AMU                                              HOOKST
C$      WRITE(IPR,957) AMULO                                            HOOKST
C$      WRITE(IPR,958) AMUUP                                            HOOKST
        GO TO 100                                                       HOOKST
C      ENDIF                                                            HOOKST
C     ENDIF                                                             HOOKST
C                                                                       HOOKST
  951 FORMAT(27H0HOOKST    TAKE NEWTON STEP)                            HOOKST
  952 FORMAT(32H0HOOKST    NEWTON STEP NOT TAKEN)                       HOOKST
  953 FORMAT(31H HOOKST    SC IS NOT ACCEPTABLE)                        HOOKST
  954 FORMAT(27H HOOKST    SC IS ACCEPTABLE)                            HOOKST
  955 FORMAT(28H HOOKST    CURRENT STEP (SC))                           HOOKST
  956 FORMAT(18H HOOKST    AMU   =,E20.13)                              HOOKST
  957 FORMAT(18H HOOKST    AMULO =,E20.13)                              HOOKST
  958 FORMAT(18H HOOKST    AMUUP =,E20.13)                              HOOKST
  959 FORMAT(18H HOOKST    PHI   =,E20.13)                              HOOKST
  960 FORMAT(18H HOOKST    PHIP  =,E20.13)                              HOOKST
  961 FORMAT(18H HOOKST    DLT   =,E20.13/                              HOOKST
     +       18H HOOKST    STEPLN=,E20.13)                              HOOKST
  962 FORMAT(23H0HOOKST    FIND NEW AMU)                                HOOKST
  963 FORMAT(14H HOOKST       ,5(E20.13,3X))                            HOOKST
      END                                                               HOOKST
      SUBROUTINE HSNINT(NR,N,A,SX,METHOD)                               HSNINT
C                                                                       HSNINT
C                                                                       HSNINT
C PURPOSE                                                               HSNINT
C -------                                                               HSNINT
C PROVIDE INITIAL HESSIAN WHEN USING SECANT UPDATES                     HSNINT
C                                                                       HSNINT
C PARAMETERS                                                            HSNINT
C ----------                                                            HSNINT
C NR           --> ROW DIMENSION OF MATRIX                              HSNINT
C N            --> DIMENSION OF PROBLEM                                 HSNINT
C A(N,N)      <--  INITIAL HESSIAN (LOWER TRIANGULAR MATRIX)            HSNINT
C SX(N)        --> DIAGONAL SCALING MATRIX FOR X                        HSNINT
C METHOD       --> ALGORITHM TO USE TO SOLVE MINIMIZATION PROBLEM       HSNINT
C                    =1,2 FACTORED SECANT METHOD USED                   HSNINT
C                    =3   UNFACTORED SECANT METHOD USED                 HSNINT
C                                                                       HSNINT
      INTEGER NR,N,METHOD,J,I,JP1                                       HSNINT
      DOUBLE PRECISION A(NR,1),SX(N)                                    HSNINT
C                                                                       HSNINT
      DO 100 J=1,N                                                      HSNINT
        IF(METHOD.EQ.3) A(J,J)=SX(J)*SX(J)                              HSNINT
        IF(METHOD.NE.3) A(J,J)=SX(J)                                    HSNINT
        IF(J.EQ.N) GO TO 100                                            HSNINT
        JP1=J+1                                                         HSNINT
        DO 90 I=JP1,N                                                   HSNINT
          A(I,J)=0.                                                     HSNINT
   90   CONTINUE                                                        HSNINT
  100 CONTINUE                                                          HSNINT
      RETURN                                                            HSNINT
      END                                                               HSNINT
      SUBROUTINE LLTSLV(NR,N,A,X,B)                                     LLTSLV
C                                                                       LLTSLV
C PURPOSE                                                               LLTSLV
C -------                                                               LLTSLV
C SOLVE AX=B WHERE A HAS THE FORM L(L-TRANSPOSE)                        LLTSLV
C BUT ONLY THE LOWER TRIANGULAR PART, L, IS STORED.                     LLTSLV
C                                                                       LLTSLV
C PARAMETERS                                                            LLTSLV
C ----------                                                            LLTSLV
C NR           --> ROW DIMENSION OF MATRIX                              LLTSLV
C N            --> DIMENSION OF PROBLEM                                 LLTSLV
C A(N,N)       --> MATRIX OF FORM L(L-TRANSPOSE).                       LLTSLV
C                  ON RETURN A IS UNCHANGED.                            LLTSLV
C X(N)        <--  SOLUTION VECTOR                                      LLTSLV
C B(N)         --> RIGHT-HAND SIDE VECTOR                               LLTSLV
C                                                                       LLTSLV
C NOTE                                                                  LLTSLV
C ----                                                                  LLTSLV
C IF B IS NOT REQUIRED BY CALLING PROGRAM, THEN                         LLTSLV
C B AND X MAY SHARE THE SAME STORAGE.                                   LLTSLV
C                                                                       LLTSLV
      INTEGER NR,N                                                      LLTSLV
      DOUBLE PRECISION A(NR,1),X(N),B(N)                                LLTSLV
C                                                                       LLTSLV
C FORWARD SOLVE, RESULT IN X                                            LLTSLV
C                                                                       LLTSLV
      CALL FORSLV(NR,N,A,X,B)                                           LLTSLV
C                                                                       LLTSLV
C BACK SOLVE, RESULT IN X                                               LLTSLV
C                                                                       LLTSLV
      CALL BAKSLV(NR,N,A,X,X)                                           LLTSLV
      RETURN                                                            LLTSLV
      END                                                               LLTSLV
      SUBROUTINE OPTCHK(N,X,TYPSIZ,SX,FSCALE,GRADTL,ITNLIM,NDIGIT,EPSM, OPTCHK
     +     DLT,METHOD,IEXP,IAGFLG,IAHFLG,STEPMX,MSG,IPR)                OPTCHK
C                                                                       OPTCHK
C PURPOSE                                                               OPTCHK
C -------                                                               OPTCHK
C CHECK INPUT FOR REASONABLENESS                                        OPTCHK
C                                                                       OPTCHK
C PARAMETERS                                                            OPTCHK
C ----------                                                            OPTCHK
C N            --> DIMENSION OF PROBLEM                                 OPTCHK
C X(N)         --> ON ENTRY, ESTIMATE TO ROOT OF FCN                    OPTCHK
C TYPSIZ(N)   <--> TYPICAL SIZE OF EACH COMPONENT OF X                  OPTCHK
C SX(N)       <--  DIAGONAL SCALING MATRIX FOR X                        OPTCHK
C FSCALE      <--> ESTIMATE OF SCALE OF OBJECTIVE FUNCTION FCN          OPTCHK
C GRADTL       --> TOLERANCE AT WHICH GRADIENT CONSIDERED CLOSE         OPTCHK
C                  ENOUGH TO ZERO TO TERMINATE ALGORITHM                OPTCHK
C ITNLIM      <--> MAXIMUM NUMBER OF ALLOWABLE ITERATIONS               OPTCHK
C NDIGIT      <--> NUMBER OF GOOD DIGITS IN OPTIMIZATION FUNCTION FCN   OPTCHK
C EPSM         --> MACHINE EPSILON                                      OPTCHK
C DLT         <--> TRUST REGION RADIUS                                  OPTCHK
C METHOD      <--> ALGORITHM INDICATOR                                  OPTCHK
C IEXP        <--> EXPENSE FLAG                                         OPTCHK
C IAGFLG      <--> =1 IF ANALYTIC GRADIENT SUPPLIED                     OPTCHK
C IAHFLG      <--> =1 IF ANALYTIC HESSIAN SUPPLIED                      OPTCHK
C STEPMX      <--> MAXIMUM STEP SIZE                                    OPTCHK
C MSG         <--> MESSAGE AND ERROR CODE                               OPTCHK
C IPR          --> DEVICE TO WHICH TO SEND OUTPUT                       OPTCHK
C                                                                       OPTCHK
      INTEGER N,ITNLIM,NDIGIT,METHOD,IEXP,IAGFLG,I                      OPTCHK
      INTEGER IAHFLG,MSG,IPR                                            OPTCHK
      DOUBLE PRECISION FSCALE,GRADTL,EPSM,DLT,STEPMX,STPSIZ             OPTCHK
      DOUBLE PRECISION X(N),TYPSIZ(N),SX(N)                             OPTCHK
C                                                                       OPTCHK
C CHECK THAT PARAMETERS ONLY TAKE ON ACCEPTABLE VALUES.                 OPTCHK
C IF NOT, SET THEM TO DEFAULT VALUES.                                   OPTCHK
      IF(METHOD.LT.1 .OR. METHOD.GT.3) METHOD=1                         OPTCHK
      IF(IAGFLG.NE.1) IAGFLG=0                                          OPTCHK
      IF(IAHFLG.NE.1) IAHFLG=0                                          OPTCHK
      IF(IEXP.NE.0) IEXP=1                                              OPTCHK
      IF(MOD(MSG/2,2).EQ.1 .AND. IAGFLG.EQ.0) GO TO 830                 OPTCHK
      IF(MOD(MSG/4,2).EQ.1 .AND. IAHFLG.EQ.0) GO TO 835                 OPTCHK
C                                                                       OPTCHK
C CHECK DIMENSION OF PROBLEM                                            OPTCHK
C                                                                       OPTCHK
      IF(N.LE.0) GO TO 805                                              OPTCHK
      IF(N.EQ.1 .AND. MOD(MSG,2).EQ.0) GO TO 810                        OPTCHK
C                                                                       OPTCHK
C COMPUTE SCALE MATRIX                                                  OPTCHK
C                                                                       OPTCHK
      DO 10 I=1,N                                                       OPTCHK
        IF(TYPSIZ(I).EQ.0.) TYPSIZ(I)=1.0                               OPTCHK
        IF(TYPSIZ(I).LT.0.) TYPSIZ(I)=-TYPSIZ(I)                        OPTCHK
        SX(I)=1.0/TYPSIZ(I)                                             OPTCHK
   10 CONTINUE                                                          OPTCHK
C                                                                       OPTCHK
C CHECK MAXIMUM STEP SIZE                                               OPTCHK
C                                                                       OPTCHK
      IF (STEPMX .GT. 0.0) GO TO 20                                     OPTCHK
      STPSIZ = 0.0                                                      OPTCHK
      DO 15 I = 1, N                                                    OPTCHK
         STPSIZ = STPSIZ + X(I)*X(I)*SX(I)*SX(I)                        OPTCHK
   15 CONTINUE                                                          OPTCHK
      STPSIZ =DSQRT(STPSIZ)                                             OPTCHK
      STEPMX = MAX(1.0D3*STPSIZ, 1.0D3)                                 OPTCHK
   20 CONTINUE                                                          OPTCHK
C CHECK FUNCTION SCALE                                                  OPTCHK
      IF(FSCALE.EQ.0.) FSCALE=1.0                                       OPTCHK
      IF(FSCALE.LT.0.) FSCALE=-FSCALE                                   OPTCHK
C                                                                       OPTCHK
C CHECK GRADIENT TOLERANCE                                              OPTCHK
      IF(GRADTL.LT.0.) GO TO 815                                        OPTCHK
C                                                                       OPTCHK
C CHECK ITERATION LIMIT                                                 OPTCHK
      IF(ITNLIM.LE.0) GO TO 820                                         OPTCHK
C                                                                       OPTCHK
C CHECK NUMBER OF DIGITS OF ACCURACY IN FUNCTION FCN                    OPTCHK
      IF(NDIGIT.EQ.0) GO TO 825                                         OPTCHK
      IF(NDIGIT.LT.0) NDIGIT=-DLOG10(EPSM)                              OPTCHK
C                                                                       OPTCHK
C CHECK TRUST REGION RADIUS                                             OPTCHK
      IF(DLT.LE.0.) DLT=-1.0                                            OPTCHK
      IF (DLT .GT. STEPMX) DLT = STEPMX                                 OPTCHK
      RETURN                                                            OPTCHK
C                                                                       OPTCHK
C ERROR EXITS                                                           OPTCHK
C                                                                       OPTCHK
  805 WRITE(IPR,901) N                                                  OPTCHK
      MSG=-1                                                            OPTCHK
      GO TO 895                                                         OPTCHK
  810 WRITE(IPR,902)                                                    OPTCHK
      MSG=-2                                                            OPTCHK
      GO TO 895                                                         OPTCHK
  815 WRITE(IPR,903) GRADTL                                             OPTCHK
      MSG=-3                                                            OPTCHK
      GO TO 895                                                         OPTCHK
  820 WRITE(IPR,904) ITNLIM                                             OPTCHK
      MSG=-4                                                            OPTCHK
      GO TO 895                                                         OPTCHK
  825 WRITE(IPR,905) NDIGIT                                             OPTCHK
      MSG=-5                                                            OPTCHK
      GO TO 895                                                         OPTCHK
  830 WRITE(IPR,906) MSG,IAGFLG                                         OPTCHK
      MSG=-6                                                            OPTCHK
      GO TO 895                                                         OPTCHK
  835 WRITE(IPR,907) MSG,IAHFLG                                         OPTCHK
      MSG=-7                                                            OPTCHK
  895 RETURN                                                            OPTCHK
  901 FORMAT(32H0OPTCHK    ILLEGAL DIMENSION, N=,I5)                    OPTCHK
  902 FORMAT(55H0OPTCHK    +++ WARNING +++  THIS PACKAGE IS INEFFICIENT,OPTCHK
     +       26H FOR PROBLEMS OF SIZE N=1./                             OPTCHK
     +       48H OPTCHK    CHECK INSTALLATION LIBRARIES FOR MORE,       OPTCHK
     +       22H APPROPRIATE ROUTINES./                                 OPTCHK
     +       41H OPTCHK    IF NONE, SET MSG AND RESUBMIT.)              OPTCHK
  903 FORMAT(38H0OPTCHK    ILLEGAL TOLERANCE.  GRADTL=,E20.13)          OPTCHK
  904 FORMAT(44H0OPTCHK    ILLEGAL ITERATION LIMIT.  ITNLIM=,I5)        OPTCHK
  905 FORMAT(52H0OPTCHK    MINIMIZATION FUNCTION HAS NO GOOD DIGITS.,   OPTCHK
     +        9H  NDIGIT=,I5)                                           OPTCHK
  906 FORMAT(50H0OPTCHK    USER REQUESTS THAT ANALYTIC GRADIENT BE,     OPTCHK
     +       33H ACCEPTED AS PROPERLY CODED (MSG=,I5, 2H),/             OPTCHK
     +       45H OPTCHK    BUT ANALYTIC GRADIENT NOT SUPPLIED,          OPTCHK
     +        9H (IAGFLG=,I5, 2H).)                                     OPTCHK
  907 FORMAT(49H0OPTCHK    USER REQUESTS THAT ANALYTIC HESSIAN BE,      OPTCHK
     +       33H ACCEPTED AS PROPERLY CODED (MSG=,I5, 2H),/             OPTCHK
     +       44H OPTCHK    BUT ANALYTIC HESSIAN NOT SUPPLIED,           OPTCHK
     +        9H (IAHFLG=,I5, 2H).)                                     OPTCHK
      END                                                               OPTCHK
      SUBROUTINE OPTDRV(NR,N,X,FCN,D1FCN,D2FCN,TYPSIZ,FSCALE,           OPTDRV
     +     METHOD,IEXP,MSG,NDIGIT,ITNLIM,IAGFLG,IAHFLG,IPR,             OPTDRV
     +     DLT,GRADTL,STEPMX,STEPTL,XPLS,FPLS,GPLS,ITRMCD,              OPTDRV
     +     A,UDIAG,G,P,SX,WRK0,WRK1,WRK2,WRK3,                          OPTDRV
     +     AJA,ANLS,SHAT,VECT1,VECT2,VECT3,VECT4,VECT5,VECT6,           OPTDRV
     +     NRM,NRN,MM,NN,IQ)                                            OPTDRV
C                                                                       OPTDRV
C PURPOSE                                                               OPTDRV
C -------                                                               OPTDRV
C DRIVER FOR NON-LINEAR OPTIMIZATION PROBLEM                            OPTDRV
C                                                                       OPTDRV
C PARAMETERS                                                            OPTDRV
C ----------                                                            OPTDRV
C NR           --> ROW DIMENSION OF MATRIX                              OPTDRV
C N            --> DIMENSION OF PROBLEM                                 OPTDRV
C X(N)         --> ON ENTRY: ESTIMATE TO A ROOT OF FCN                  OPTDRV
C FCN          --> NAME OF SUBROUTINE TO EVALUATE OPTIMIZATION FUNCTION OPTDRV
C                  MUST BE DECLARED EXTERNAL IN CALLING ROUTINE         OPTDRV
C                            FCN: R(N) --> R(1)                         OPTDRV
C D1FCN        --> (OPTIONAL) NAME OF SUBROUTINE TO EVALUATE GRADIENT   OPTDRV
C                  OF FCN.  MUST BE DECLARED EXTERNAL IN CALLING ROUTINEOPTDRV
C D2FCN        --> (OPTIONAL) NAME OF SUBROUTINE TO EVALUATE HESSIAN OF OPTDRV
C                  OF FCN.  MUST BE DECLARED EXTERNAL IN CALLING ROUTINEOPTDRV
C TYPSIZ(N)    --> TYPICAL SIZE FOR EACH COMPONENT OF X                 OPTDRV
C FSCALE       --> ESTIMATE OF SCALE OF OBJECTIVE FUNCTION              OPTDRV
C METHOD       --> ALGORITHM TO USE TO SOLVE MINIMIZATION PROBLEM       OPTDRV
C                    =1 LINE SEARCH                                     OPTDRV
C                    =2 DOUBLE DOGLEG                                   OPTDRV
C                    =3 MORE-HEBDON                                     OPTDRV
C IEXP         --> =1 IF OPTIMIZATION FUNCTION FCN IS EXPENSIVE TO      OPTDRV
C                  EVALUATE, =0 OTHERWISE.  IF SET THEN HESSIAN WILL    OPTDRV
C                  BE EVALUATED BY SECANT UPDATE INSTEAD OF             OPTDRV
C                  ANALYTICALLY OR BY FINITE DIFFERENCES                OPTDRV
C MSG         <--> ON INPUT:  (.GT.0) MESSAGE TO INHIBIT CERTAIN        OPTDRV
C                    AUTOMATIC CHECKS                                   OPTDRV
C                  ON OUTPUT: (.LT.0) ERROR CODE; =0 NO ERROR           OPTDRV
C NDIGIT       --> NUMBER OF GOOD DIGITS IN OPTIMIZATION FUNCTION FCN   OPTDRV
C ITNLIM       --> MAXIMUM NUMBER OF ALLOWABLE ITERATIONS               OPTDRV
C IAGFLG       --> =1 IF ANALYTIC GRADIENT SUPPLIED                     OPTDRV
C IAHFLG       --> =1 IF ANALYTIC HESSIAN SUPPLIED                      OPTDRV
C IPR          --> DEVICE TO WHICH TO SEND OUTPUT                       OPTDRV
C DLT          --> TRUST REGION RADIUS                                  OPTDRV
C GRADTL       --> TOLERANCE AT WHICH GRADIENT CONSIDERED CLOSE         OPTDRV
C                  ENOUGH TO ZERO TO TERMINATE ALGORITHM                OPTDRV
C STEPMX       --> MAXIMUM ALLOWABLE STEP SIZE                          OPTDRV
C STEPTL       --> RELATIVE STEP SIZE AT WHICH SUCCESSIVE ITERATES      OPTDRV
C                  CONSIDERED CLOSE ENOUGH TO TERMINATE ALGORITHM       OPTDRV
C XPLS(N)     <--> ON EXIT:  XPLS IS LOCAL MINIMUM                      OPTDRV
C FPLS        <--> ON EXIT:  FUNCTION VALUE AT SOLUTION, XPLS           OPTDRV
C GPLS(N)     <--> ON EXIT:  GRADIENT AT SOLUTION XPLS                  OPTDRV
C ITRMCD      <--  TERMINATION CODE                                     OPTDRV
C A(N,N)       --> WORKSPACE FOR HESSIAN (OR ESTIMATE)                  OPTDRV
C                  AND ITS CHOLESKY DECOMPOSITION                       OPTDRV
C UDIAG(N)     --> WORKSPACE [FOR DIAGONAL OF HESSIAN]                  OPTDRV
C G(N)         --> WORKSPACE (FOR GRADIENT AT CURRENT ITERATE)          OPTDRV
C P(N)         --> WORKSPACE FOR STEP                                   OPTDRV
C SX(N)        --> WORKSPACE (FOR DIAGONAL SCALING MATRIX)              OPTDRV
C WRK0(N)      --> WORKSPACE                                            OPTDRV
C WRK1(N)      --> WORKSPACE                                            OPTDRV
C WRK2(N)      --> WORKSPACE                                            OPTDRV
C WRK3(N)      --> WORKSPACE                                            OPTDRV
C                                                                       OPTDRV
C                                                                       OPTDRV
C INTERNAL VARIABLES                                                    OPTDRV
C ------------------                                                    OPTDRV
C ANALTL           TOLERANCE FOR COMPARISON OF ESTIMATED AND            OPTDRV
C                  ANALYTICAL GRADIENTS AND HESSIANS                    OPTDRV
C EPSM             MACHINE EPSILON                                      OPTDRV
C F                FUNCTION VALUE: FCN(X)                               OPTDRV
C ITNCNT           CURRENT ITERATION, K                                 OPTDRV
C RNF              RELATIVE NOISE IN OPTIMIZATION FUNCTION FCN.         OPTDRV
C                       NOISE=10.**(-NDIGIT)                            OPTDRV
C                                                                       OPTDRV
      INTEGER NR,N,METHOD,IEXP,MSG,NDIGIT,ITNLIM,IAGFLG,IAHFLG,IPR      OPTDRV
      INTEGER NRM,NRN,MM,NN,IQ,I,ITRMCD,ITNCNT,IRETCD,ICSCMX            OPTDRV
      DOUBLE PRECISION FSCALE,DLT,GRADTL,STEPMX,STEPTL,F,FPLS           OPTDRV
      DOUBLE PRECISION EPSM,DPMEPS,RNF,ANALTL,DLTSAV                    OPTDRV
      DOUBLE PRECISION AMUSAV,AMU,DLPSAV,DLTP,PHISAV,PHI,PHPSAV,PHIP0   OPTDRV
      DOUBLE PRECISION X(N),XPLS(N),G(N),GPLS(N),P(N)                   OPTDRV
      DOUBLE PRECISION TYPSIZ(N),SX(N),WRK(1)                           OPTDRV
      DOUBLE PRECISION A(NR,1),UDIAG(N)                                 OPTDRV
      DOUBLE PRECISION WRK0(N),WRK1(N),WRK2(N),WRK3(N)                  OPTDRV
      DOUBLE PRECISION AJA(NRM,1),ANLS(NRM,1),SHAT(NRN,1)               OPTDRV
      DOUBLE PRECISION VECT1(MM),VECT2(MM),VECT3(MM)                    OPTDRV
      DOUBLE PRECISION VECT4(MM),VECT5(MM),VECT6(MM)                    OPTDRV
      LOGICAL MXTAKE                                                    OPTDRV
      EXTERNAL FCN,D1FCN,D2FCN                                          OPTDRV
C                                                                       OPTDRV
C INITIALIZATION                                                        OPTDRV
C --------------                                                        OPTDRV
      DO 10 I=1,N                                                       OPTDRV
        P(I)=0.                                                         OPTDRV
   10 CONTINUE                                                          OPTDRV
      ITNCNT=0                                                          OPTDRV
      IRETCD=-1                                                         OPTDRV
      EPSM=DPMEPS()                                                     OPTDRV
      CALL OPTCHK(N,X,TYPSIZ,SX,FSCALE,GRADTL,ITNLIM,NDIGIT,EPSM,       OPTDRV
     +     DLT,METHOD,IEXP,IAGFLG,IAHFLG,STEPMX,MSG,IPR)                OPTDRV
      IF(MSG.LT.0) RETURN                                               OPTDRV
      RNF=MAX(10.0D0**(-NDIGIT),EPSM)                                   OPTDRV
      ANALTL=MAX(1.D-2,SQRT(RNF))                                       OPTDRV
C                                                                       OPTDRV
      IF(MOD(MSG/8,2).EQ.1) GO TO 15                                    OPTDRV
      WRITE(IPR,901)                                                    OPTDRV
      WRITE(IPR,900) (TYPSIZ(I),I=1,N)                                  OPTDRV
      WRITE(IPR,902)                                                    OPTDRV
      WRITE(IPR,900) (SX(I),I=1,N)                                      OPTDRV
      WRITE(IPR,903) FSCALE                                             OPTDRV
      WRITE(IPR,904) NDIGIT,IAGFLG,IAHFLG,IEXP,METHOD,ITNLIM,EPSM       OPTDRV
      WRITE(IPR,905) STEPMX,STEPTL,GRADTL,DLT,RNF,ANALTL                OPTDRV
   15 CONTINUE                                                          OPTDRV
C                                                                       OPTDRV
C EVALUATE FCN(X)                                                       OPTDRV
C                                                                       OPTDRV
      CALL FCN(N,X,F,AJA,ANLS,SHAT,VECT1,VECT2,VECT3,VECT4,             OPTDRV
     +  VECT5,VECT6,NRM,NRN,MM,NN,IQ)                                   OPTDRV
C                                                                       OPTDRV
C EVALUATE ANALYTIC OR FINITE DIFFERENCE GRADIENT AND CHECK ANALYTIC    OPTDRV
C GRADIENT, IF REQUESTED.                                               OPTDRV
C                                                                       OPTDRV
      IF (IAGFLG .EQ. 1) GO TO 20                                       OPTDRV
C     IF (IAGFLG .EQ. 0)                                                OPTDRV
C     THEN                                                              OPTDRV
        CALL FSTOFD (1, 1, N, X, FCN, F, G, SX, RNF, WRK, 1,            OPTDRV
     +  AJA,ANLS,SHAT,VECT1,VECT2,VECT3,VECT4,VECT5,VECT6,              OPTDRV
     +  NRM,NRN,MM,NN,IQ)                                               OPTDRV
        GO TO 25                                                        OPTDRV
C                                                                       OPTDRV
   20 CALL D1FCN (N, X, G,AJA,ANLS,SHAT,VECT1,VECT2,VECT3,              OPTDRV
     +  VECT4,VECT5,VECT6,NRM,NRN,MM,NN,IQ)                             OPTDRV
      IF (MOD(MSG/2,2) .EQ. 1) GO TO 25                                 OPTDRV
C     IF (MOD(MSG/2,2).EQ.0)                                            OPTDRV
C     THEN                                                              OPTDRV
        CALL GRDCHK (N, X, FCN, F, G, TYPSIZ, SX, FSCALE,               OPTDRV
     1    RNF, ANALTL, WRK1, MSG, IPR,                                  OPTDRV
     1    AJA,ANLS,SHAT,VECT1,VECT2,VECT3,VECT4,VECT5,VECT6,            OPTDRV
     1    NRM,NRN,MM,NN,IQ)                                             OPTDRV
        IF (MSG .LT. 0) RETURN                                          OPTDRV
   25 CONTINUE                                                          OPTDRV
C                                                                       OPTDRV
      CALL OPTSTP(N,X,F,G,WRK1,ITNCNT,ICSCMX,                           OPTDRV
     +            ITRMCD,GRADTL,STEPTL,SX,FSCALE,ITNLIM,IRETCD,MXTAKE,  OPTDRV
     +            IPR,MSG)                                              OPTDRV
      IF(ITRMCD.NE.0) GO TO 700                                         OPTDRV
C                                                                       OPTDRV
      IF(IEXP.NE.1) GO TO 80                                            OPTDRV
C                                                                       OPTDRV
C IF OPTIMIZATION FUNCTION EXPENSIVE TO EVALUATE (IEXP=1), THEN         OPTDRV
C HESSIAN WILL BE OBTAINED BY SECANT UPDATES.  GET INITIAL HESSIAN.     OPTDRV
C                                                                       OPTDRV
      CALL HSNINT(NR,N,A,SX,METHOD)                                     OPTDRV
      GO TO 90                                                          OPTDRV
   80 CONTINUE                                                          OPTDRV
C                                                                       OPTDRV
C EVALUATE ANALYTIC OR FINITE DIFFERENCE HESSIAN AND CHECK ANALYTIC     OPTDRV
C HESSIAN IF REQUESTED (ONLY IF USER-SUPPLIED ANALYTIC HESSIAN          OPTDRV
C ROUTINE D2FCN FILLS ONLY LOWER TRIANGULAR PART AND DIAGONAL OF A).    OPTDRV
C                                                                       OPTDRV
      IF (IAHFLG .EQ. 0) THEN                                           OPTDRV
         IF (IAGFLG .EQ. 1) CALL FSTOFD (NR, N, N, X, D1FCN, G, A, SX,  OPTDRV
     1      RNF, WRK1, 3,AJA,ANLS,SHAT,VECT1,VECT2,VECT3,VECT4,VECT5,   OPTDRV
     1      VECT6,NRM,NRN,MM,NN,IQ)                                     OPTDRV
         IF (IAGFLG .NE. 1) CALL SNDOFD (NR, N, X, FCN, F, A, SX, RNF,  OPTDRV
     1      WRK1, WRK2)                                                 OPTDRV
C                                                                       OPTDRV
      ELSE                                                              OPTDRV
         IF (MOD(MSG/4, 2) .EQ. 1) THEN                                 OPTDRV
            CALL D2FCN (NR, N, X, A)                                    OPTDRV
C                                                                       OPTDRV
C        ELSE                                                           OPTDRV
           CALL HESCHK (NR, N, X, FCN, D1FCN, D2FCN, F, G, A, TYPSIZ,   OPTDRV
     1         SX, RNF, ANALTL, IAGFLG, UDIAG, WRK1, WRK2, MSG, IPR)    OPTDRV
C                                                                       OPTDRV
C           HESCHK EVALUATES D2FCN AND CHECKS IT AGAINST THE FINITE     OPTDRV
C           DIFFERENCE HESSIAN WHICH IT CALCULATES BY CALLING FSTOFD    OPTDRV
C           (IF IAGFLG .EQ. 1) OR SNDOFD (OTHERWISE).                   OPTDRV
C                                                                       OPTDRV
            IF (MSG .LT. 0) RETURN                                      OPTDRV
         ENDIF                                                          OPTDRV
      ENDIF                                                             OPTDRV
C                                                                       OPTDRV
   90 IF(MOD(MSG/8,2).EQ.0)                                             OPTDRV
     +     CALL RESLT(NR,N,X,F,G,A,P,ITNCNT,1,IPR)                      OPTDRV
C                                                                       OPTDRV
C                                                                       OPTDRV
C ITERATION                                                             OPTDRV
C ---------                                                             OPTDRV
  100 ITNCNT=ITNCNT+1                                                   OPTDRV
C                                                                       OPTDRV
C FIND PERTURBED LOCAL MODEL HESSIAN AND ITS LL+ DECOMPOSITION          OPTDRV
C (SKIP THIS STEP IF LINE SEARCH OR DOGSTEP TECHNIQUES BEING USED WITH  OPTDRV
C SECANT UPDATES.  CHOLESKY DECOMPOSITION L ALREADY OBTAINED FROM       OPTDRV
C SECFAC.)                                                              OPTDRV
C                                                                       OPTDRV
      IF(IEXP.EQ.1 .AND. METHOD.NE.3) GO TO 105                         OPTDRV
  103   CALL CHLHSN(NR,N,A,EPSM,SX,UDIAG)                               OPTDRV
  105 CONTINUE                                                          OPTDRV
C                                                                       OPTDRV
C SOLVE FOR NEWTON STEP:  AP=-G                                         OPTDRV
C                                                                       OPTDRV
      DO 110 I=1,N                                                      OPTDRV
        WRK1(I)=-G(I)                                                   OPTDRV
  110 CONTINUE                                                          OPTDRV
      CALL LLTSLV(NR,N,A,P,WRK1)                                        OPTDRV
C                                                                       OPTDRV
C DECIDE WHETHER TO ACCEPT NEWTON STEP  XPLS=X + P                      OPTDRV
C OR TO CHOOSE XPLS BY A GLOBAL STRATEGY.                               OPTDRV
C                                                                       OPTDRV
      IF (IAGFLG .NE. 0 .OR. METHOD .EQ. 1) GO TO 111                   OPTDRV
      DLTSAV = DLT                                                      OPTDRV
      IF (METHOD .EQ. 2) GO TO 111                                      OPTDRV 
      AMUSAV = AMU                                                      OPTDRV
      DLPSAV = DLTP                                                     OPTDRV
      PHISAV = PHI                                                      OPTDRV
      PHPSAV = PHIP0                                                    OPTDRV
  111 CONTINUE                                                          OPTDRV
      IF(METHOD.EQ.2)                                                   OPTDRV
     +     CALL DOGDRV(NR,N,X,F,G,A,P,XPLS,FPLS,FCN,SX,STEPMX,          OPTDRV
     +     STEPTL,DLT,IRETCD,MXTAKE,WRK0,WRK1,WRK2,WRK3,IPR,            OPTDRV
     +     AJA,ANLS,SHAT,VECT1,VECT2,VECT3,VECT4,VECT5,VECT6,           OPTDRV
     +     NRM,NRN,MM,NN,IQ)                                            OPTDRV
      IF(METHOD.EQ.3)                                                   OPTDRV
     +     CALL HOOKDR(NR,N,X,F,G,A,UDIAG,P,XPLS,FPLS,FCN,SX,STEPMX,    OPTDRV
     +     STEPTL,DLT,IRETCD,MXTAKE,AMU,DLTP,PHI,PHIP0,WRK0,            OPTDRV
     +     WRK1,WRK2,EPSM,ITNCNT,IPR)                                   OPTDRV
C                                                                       OPTDRV
C IF COULD NOT FIND SATISFACTORY STEP AND FORWARD DIFFERENCE            OPTDRV
C GRADIENT WAS USED, RETRY USING CENTRAL DIFFERENCE GRADIENT.           OPTDRV
C                                                                       OPTDRV
      IF (IRETCD .NE. 1 .OR. IAGFLG .NE. 0) GO TO 112                   OPTDRV
C     IF (IRETCD .EQ. 1 .AND. IAGFLG .EQ. 0)                            OPTDRV
C     THEN                                                              OPTDRV
C                                                                       OPTDRV
C        SET IAGFLG FOR CENTRAL DIFFERENCES                             OPTDRV
C                                                                       OPTDRV
         IAGFLG = -1                                                    OPTDRV
         WRITE(IPR,906) ITNCNT                                          OPTDRV
C                                                                       OPTDRV
         CALL FSTOCD (N, X, FCN, SX, RNF, G)                            OPTDRV
         IF (METHOD .EQ. 1) GO TO 105                                   OPTDRV
         DLT = DLTSAV                                                   OPTDRV
         IF (METHOD .EQ. 2) GO TO 105                                   OPTDRV
         AMU = AMUSAV                                                   OPTDRV
         DLTP = DLPSAV                                                  OPTDRV
         PHI = PHISAV                                                   OPTDRV
         PHIP0 = PHPSAV                                                 OPTDRV
         GO TO 103                                                      OPTDRV
C                                                                       OPTDRV
C CALCULATE STEP FOR OUTPUT                                             OPTDRV
C                                                                       OPTDRV
  112 CONTINUE                                                          OPTDRV
      DO 114 I = 1, N                                                   OPTDRV
         P(I) = XPLS(I) - X(I)                                          OPTDRV
  114 CONTINUE                                                          OPTDRV
C                                                                       OPTDRV
C CALCULATE GRADIENT AT XPLS                                            OPTDRV
C                                                                       OPTDRV
      IF (IAGFLG .EQ. (-1)) GO TO 116                                   OPTDRV
      IF (IAGFLG .EQ. 0) GO TO 118                                      OPTDRV
C                                                                       OPTDRV
C ANALYTIC GRADIENT                                                     OPTDRV
C                                                                       OPTDRV
      CALL D1FCN (N, XPLS, GPLS,                                        OPTDRV
     +  AJA,ANLS,SHAT,VECT1,VECT2,VECT3,VECT4,VECT5,VECT6,              OPTDRV
     +  NRM,NRN,MM,NN,IQ)                                               OPTDRV
      GO TO 120                                                         OPTDRV
C                                                                       OPTDRV
C CENTRAL DIFFERENCE GRADIENT                                           OPTDRV
C                                                                       OPTDRV
  116 CALL FSTOCD (N, XPLS, FCN, SX, RNF, GPLS)                         OPTDRV
      GO TO 120                                                         OPTDRV
C                                                                       OPTDRV
C FORWARD DIFFERENCE GRADIENT                                           OPTDRV
C                                                                       OPTDRV
  118 CALL FSTOFD (1, 1, N, XPLS, FCN, FPLS, GPLS, SX, RNF, WRK, 1,     OPTDRV
     +  AJA,ANLS,SHAT,VECT1,VECT2,VECT3,VECT4,VECT5,VECT6,              OPTDRV
     +  NRM,NRN,MM,NN,IQ)                                               OPTDRV
  120 CONTINUE                                                          OPTDRV
C                                                                       OPTDRV
C CHECK WHETHER STOPPING CRITERIA SATISFIED                             OPTDRV
C                                                                       OPTDRV
      CALL OPTSTP(N,XPLS,FPLS,GPLS,X,ITNCNT,ICSCMX,                     OPTDRV
     +            ITRMCD,GRADTL,STEPTL,SX,FSCALE,ITNLIM,IRETCD,MXTAKE,  OPTDRV
     +            IPR,MSG)                                              OPTDRV
      IF(ITRMCD.NE.0) GO TO 690                                         OPTDRV
C                                                                       OPTDRV
C EVALUATE HESSIAN AT XPLS                                              OPTDRV
C                                                                       OPTDRV
      IF(IEXP.NE.0) GO TO 150                                           OPTDRV
  130 IF(IAHFLG.EQ.1) GO TO 140                                         OPTDRV
      IF(IAGFLG.EQ.1)                                                   OPTDRV
     +     CALL FSTOFD(NR,N,N,XPLS,D1FCN,GPLS,A,SX,RNF,WRK1,3,          OPTDRV
     +     AJA,ANLS,SHAT,VECT1,VECT2,VECT3,VECT4,VECT5,VECT6,           OPTDRV
     +     NRM,NRN,MM,NN,IQ)                                            OPTDRV
      IF(IAGFLG.NE.1) CALL SNDOFD(NR,N,XPLS,FCN,FPLS,A,SX,              OPTDRV
     +                            RNF,WRK1,WRK2)                        OPTDRV
      GO TO 150                                                         OPTDRV
  140 CALL D2FCN(NR,N,XPLS,A)                                           OPTDRV
  150 CONTINUE                                                          OPTDRV
      IF(MOD(MSG/16,2).EQ.1)                                            OPTDRV
     +     CALL RESLT(NR,N,XPLS,FPLS,GPLS,A,P,ITNCNT,1,IPR)             OPTDRV
C                                                                       OPTDRV
C X <-- XPLS  AND  G <-- GPLS  AND  F <-- FPLS                          OPTDRV
C                                                                       OPTDRV
      F=FPLS                                                            OPTDRV
      DO 160 I=1,N                                                      OPTDRV
        X(I)=XPLS(I)                                                    OPTDRV
        G(I)=GPLS(I)                                                    OPTDRV
  160 CONTINUE                                                          OPTDRV
      GO TO 100                                                         OPTDRV
C                                                                       OPTDRV
C TERMINATION                                                           OPTDRV
C -----------                                                           OPTDRV
C RESET XPLS,FPLS,GPLS,  IF PREVIOUS ITERATE SOLUTION                   OPTDRV
C                                                                       OPTDRV
  690 IF(ITRMCD.NE.3) GO TO 710                                         OPTDRV
  700 CONTINUE                                                          OPTDRV
      FPLS=F                                                            OPTDRV
      DO 705 I=1,N                                                      OPTDRV
        XPLS(I)=X(I)                                                    OPTDRV
        GPLS(I)=G(I)                                                    OPTDRV
  705 CONTINUE                                                          OPTDRV
C                                                                       OPTDRV
C PRINT RESULTS                                                         OPTDRV
C                                                                       OPTDRV
  710 CONTINUE                                                          OPTDRV
      IF(MOD(MSG/8,2).EQ.0)                                             OPTDRV
     +     CALL RESLT(NR,N,XPLS,FPLS,GPLS,A,P,ITNCNT,0,IPR)             OPTDRV
      MSG=0                                                             OPTDRV
      RETURN                                                            OPTDRV
C                                                                       OPTDRV
  900 FORMAT(14H OPTDRV       ,5(E20.13,3X))                            OPTDRV
  901 FORMAT(20H0OPTDRV    TYPICAL X)                                   OPTDRV
  902 FORMAT(40H OPTDRV    DIAGONAL SCALING MATRIX FOR X)               OPTDRV
  903 FORMAT(22H OPTDRV    TYPICAL F =,E20.13)                          OPTDRV
  904 FORMAT(40H0OPTDRV    NUMBER OF GOOD DIGITS IN FCN=,I5/            OPTDRV
     +       27H OPTDRV    GRADIENT FLAG  =,I5,18H   (=1 IF ANALYTIC,   OPTDRV
     +       19H GRADIENT SUPPLIED)/                                    OPTDRV
     +       27H OPTDRV    HESSIAN FLAG   =,I5,18H   (=1 IF ANALYTIC,   OPTDRV
     +       18H HESSIAN SUPPLIED)/                                     OPTDRV
     +       27H OPTDRV    EXPENSE FLAG   =,I5, 9H   (=1 IF,            OPTDRV
     +       45H MINIMIZATION FUNCTION EXPENSIVE TO EVALUATE)/          OPTDRV
     +       27H OPTDRV    METHOD TO USE  =,I5,19H   (=1,2,3 FOR LINE,  OPTDRV
     +       49H SEARCH, DOUBLE DOGLEG, MORE-HEBDON RESPECTIVELY)/      OPTDRV
     +       27H OPTDRV    ITERATION LIMIT=,I5/                         OPTDRV
     +       27H OPTDRV    MACHINE EPSILON=,E20.13)                     OPTDRV
  905 FORMAT(30H0OPTDRV    MAXIMUN STEP SIZE =,E20.13/                  OPTDRV
     +       30H OPTDRV    STEP TOLERANCE    =,E20.13/                  OPTDRV
     +       30H OPTDRV    GRADIENT TOLERANCE=,E20.13/                  OPTDRV 
     +       30H OPTDRV    TRUST REG RADIUS  =,E20.13/                  OPTDRV
     +       30H OPTDRV    REL NOISE IN FCN  =,E20.13/                  OPTDRV
     +       30H OPTDRV    ANAL-FD TOLERANCE =,E20.13)                  OPTDRV
  906 FORMAT(52H OPTDRV    SHIFT FROM FORWARD TO CENTRAL DIFFERENCES,   OPTDRV
     1   14H IN ITERATION , I5)                                         OPTDRV
      END                                                               OPTDRV
      SUBROUTINE OPTIF9(NR,N,X,FCN,D1FCN,D2FCN,TYPSIZ,FSCALE,METHOD,    OPTIF9
     +     IEXP,MSG,NDIGIT,ITNLIM,IAGFLG,IAHFLG,IPR,DLT,GRADTL,         OPTIF9
     +     STEPMX,STEPTL,XPLS,FPLS,GPLS,ITRMCD,A,WRK,AJA,ANLS,SHAT,     OPTIF9
     +     VECT1,VECT2,VECT3,VECT4,VECT5,VECT6,NRM,NRN,MM,NN,IQ)        OPTIF9
C                                                                       OPTIF9
C PURPOSE                                                               OPTIF9
C -------                                                               OPTIF9
C PROVIDE COMPLETE INTERFACE TO MINIMIZATION PACKAGE.                   OPTIF9
C USER HAS FULL CONTROL OVER OPTIONS.                                   OPTIF9
C                                                                       OPTIF9
C PARAMETERS                                                            OPTIF9
C ----------                                                            OPTIF9
C NR           --> ROW DIMENSION OF MATRIX                              OPTIF9
C N            --> DIMENSION OF PROBLEM                                 OPTIF9
C X(N)         --> ON ENTRY: ESTIMATE TO A ROOT OF FCN                  OPTIF9
C FCN          --> NAME OF SUBROUTINE TO EVALUATE OPTIMIZATION FUNCTION OPTIF9
C                  MUST BE DECLARED EXTERNAL IN CALLING ROUTINE         OPTIF9
C                            FCN: R(N) --> R(1)                         OPTIF9
C D1FCN        --> (OPTIONAL) NAME OF SUBROUTINE TO EVALUATE GRADIENT   OPTIF9
C                  OF FCN.  MUST BE DECLARED EXTERNAL IN CALLING ROUTINEOPTIF9
C D2FCN        --> (OPTIONAL) NAME OF SUBROUTINE TO EVALUATE HESSIAN OF OPTIF9
C                  OF FCN.  MUST BE DECLARED EXTERNAL IN CALLING ROUTINEOPTIF9
C TYPSIZ(N)    --> TYPICAL SIZE FOR EACH COMPONENT OF X                 OPTIF9
C FSCALE       --> ESTIMATE OF SCALE OF OBJECTIVE FUNCTION              OPTIF9
C METHOD       --> ALGORITHM TO USE TO SOLVE MINIMIZATION PROBLEM       OPTIF9
C                    =1 LINE SEARCH                                     OPTIF9
C                    =2 DOUBLE DOGLEG                                   OPTIF9
C                    =3 MORE-HEBDON                                     OPTIF9
C IEXP         --> =1 IF OPTIMIZATION FUNCTION FCN IS EXPENSIVE TO      OPTIF9
C                  EVALUATE, =0 OTHERWISE.  IF SET THEN HESSIAN WILL    OPTIF9
C                  BE EVALUATED BY SECANT UPDATE INSTEAD OF             OPTIF9
C                  ANALYTICALLY OR BY FINITE DIFFERENCES                OPTIF9
C MSG         <--> ON INPUT:  (.GT.0) MESSAGE TO INHIBIT CERTAIN        OPTIF9
C                    AUTOMATIC CHECKS                                   OPTIF9
C                  ON OUTPUT: (.LT.0) ERROR CODE; =0 NO ERROR           OPTIF9
C NDIGIT       --> NUMBER OF GOOD DIGITS IN OPTIMIZATION FUNCTION FCN   OPTIF9
C ITNLIM       --> MAXIMUM NUMBER OF ALLOWABLE ITERATIONS               OPTIF9
C IAGFLG       --> =1 IF ANALYTIC GRADIENT SUPPLIED                     OPTIF9
C IAHFLG       --> =1 IF ANALYTIC HESSIAN SUPPLIED                      OPTIF9
C IPR          --> DEVICE TO WHICH TO SEND OUTPUT                       OPTIF9
C DLT          --> TRUST REGION RADIUS                                  OPTIF9
C GRADTL       --> TOLERANCE AT WHICH GRADIENT CONSIDERED CLOSE         OPTIF9
C                  ENOUGH TO ZERO TO TERMINATE ALGORITHM                OPTIF9
C STEPMX       --> MAXIMUM ALLOWABLE STEP SIZE                          OPTIF9
C STEPTL       --> RELATIVE STEP SIZE AT WHICH SUCCESSIVE ITERATES      OPTIF9
C                  CONSIDERED CLOSE ENOUGH TO TERMINATE ALGORITHM       OPTIF9
C XPLS(N)     <--> ON EXIT:  XPLS IS LOCAL MINIMUM                      OPTIF9
C FPLS        <--> ON EXIT:  FUNCTION VALUE AT SOLUTION, XPLS           OPTIF9
C GPLS(N)     <--> ON EXIT:  GRADIENT AT SOLUTION XPLS                  OPTIF9
C ITRMCD      <--  TERMINATION CODE                                     OPTIF9
C A(N,N)       --> WORKSPACE FOR HESSIAN (OR ESTIMATE)                  OPTIF9
C                  AND ITS CHOLESKY DECOMPOSITION                       OPTIF9
C WRK(N,8)     --> WORKSPACE                                            OPTIF9
C                                                                       OPTIF9
      INTEGER NR,N,METHOD,IEXP,MSG,NDIGIT,ITNLIM,IAGFLG,IAHFLG,IPR      OPTIF9
      INTEGER NRM,NRN,MM,NN,IQ,ITRMCD                                   OPTIF9
      DOUBLE PRECISION FSCALE,DLT,GRADTL,STEPMX,STEPTL,FPLS             OPTIF9
      DOUBLE PRECISION X(N),XPLS(N),GPLS(N),TYPSIZ(N)                   OPTIF9
      DOUBLE PRECISION A(NR,1),WRK(NR,1)                                OPTIF9
      DOUBLE PRECISION AJA(NRM,1),ANLS(NRM,1),SHAT(NRN,1)               OPTIF9
      DOUBLE PRECISION VECT1(MM),VECT2(MM),VECT3(MM)                    OPTIF9
      DOUBLE PRECISION VECT4(MM),VECT5(MM),VECT6(MM)                    OPTIF9
      EXTERNAL FCN,D1FCN,D2FCN                                          OPTIF9
C                                                                       OPTIF9
C EQUIVALENCE WRK(N,1) = UDIAG(N)                                       OPTIF9
C             WRK(N,2) = G(N)                                           OPTIF9
C             WRK(N,3) = P(N)                                           OPTIF9
C             WRK(N,4) = SX(N)                                          OPTIF9
C             WRK(N,5) = WRK0(N)                                        OPTIF9
C             WRK(N,6) = WRK1(N)                                        OPTIF9
C             WRK(N,7) = WRK2(N)                                        OPTIF9
C             WRK(N,8) = WRK3(N)                                        OPTIF9
C                                                                       OPTIF9
      CALL OPTDRV(NR,N,X,FCN,D1FCN,D2FCN,TYPSIZ,FSCALE,                 OPTIF9
     +     METHOD,IEXP,MSG,NDIGIT,ITNLIM,IAGFLG,IAHFLG,IPR,             OPTIF9
     +     DLT,GRADTL,STEPMX,STEPTL,XPLS,FPLS,GPLS,ITRMCD,              OPTIF9
     +     A,WRK(1,1),WRK(1,2),WRK(1,3),WRK(1,4),WRK(1,5),              OPTIF9
     +     WRK(1,6),WRK(1,7),WRK(1,8),                                  OPTIF9
     +     AJA,ANLS,SHAT,VECT1,VECT2,VECT3,VECT4,VECT5,VECT6,           OPTIF9
     +     NRM,NRN,MM,NN,IQ)                                            OPTIF9
      RETURN                                                            OPTIF9
      END                                                               OPTIF9
      SUBROUTINE OPTSTP(N,XPLS,FPLS,GPLS,X,ITNCNT,ICSCMX,               OPTSTP
     +      ITRMCD,GRADTL,STEPTL,SX,FSCALE,ITNLIM,IRETCD,               OPTIF9
     +      MXTAKE,IPR,MSG)                                             OPTIF9
C                                                                       OPTSTP
C UNCONSTRAINED MINIMIZATION STOPPING CRITERIA                          OPTSTP
C --------------------------------------------                          OPTSTP
C FIND WHETHER THE ALGORITHM SHOULD TERMINATE, DUE TO ANY               OPTSTP
C OF THE FOLLOWING:                                                     OPTSTP
C 1) PROBLEM SOLVED WITHIN USER TOLERANCE                               OPTSTP
C 2) CONVERGENCE WITHIN USER TOLERANCE                                  OPTSTP
C 3) ITERATION LIMIT REACHED                                            OPTSTP
C 4) DIVERGENCE OR TOO RESTRICTIVE MAXIMUM STEP (STEPMX) SUSPECTED      OPTSTP
C                                                                       OPTSTP
C                                                                       OPTSTP
C PARAMETERS                                                            OPTSTP
C ----------                                                            OPTSTP
C N            --> DIMENSION OF PROBLEM                                 OPTSTP
C XPLS(N)      --> NEW ITERATE X[K]                                     OPTSTP
C FPLS         --> FUNCTION VALUE AT NEW ITERATE F(XPLS)                OPTSTP
C GPLS(N)      --> GRADIENT AT NEW ITERATE, G(XPLS), OR APPROXIMATE     OPTSTP
C X(N)         --> OLD ITERATE X[K-1]                                   OPTSTP
C ITNCNT       --> CURRENT ITERATION K                                  OPTSTP
C ICSCMX      <--> NUMBER CONSECUTIVE STEPS .GE. STEPMX                 OPTSTP
C                  [RETAIN VALUE BETWEEN SUCCESSIVE CALLS]              OPTSTP
C ITRMCD      <--  TERMINATION CODE                                     OPTSTP
C GRADTL       --> TOLERANCE AT WHICH RELATIVE GRADIENT CONSIDERED CLOSEOPTSTP
C                  ENOUGH TO ZERO TO TERMINATE ALGORITHM                OPTSTP
C STEPTL       --> RELATIVE STEP SIZE AT WHICH SUCCESSIVE ITERATES      OPTSTP
C                  CONSIDERED CLOSE ENOUGH TO TERMINATE ALGORITHM       OPTSTP
C SX(N)        --> DIAGONAL SCALING MATRIX FOR X                        OPTSTP 
C FSCALE       --> ESTIMATE OF SCALE OF OBJECTIVE FUNCTION              OPTSTP
C ITNLIM       --> MAXIMUM NUMBER OF ALLOWABLE ITERATIONS               OPTSTP
C IRETCD       --> RETURN CODE                                          OPTSTP
C MXTAKE       --> BOOLEAN FLAG INDICATING STEP OF MAXIMUM LENGTH USED  OPTSTP
C IPR          --> DEVICE TO WHICH TO SEND OUTPUT                       OPTSTP
C MSG          --> IF MSG INCLUDES A TERM 8, SUPPRESS OUTPUT            OPTSTP
C                                                                       OPTSTP
C                                                                       OPTSTP
      INTEGER N,MSG,ITNLIM,IPR,I                                        OPTSTP
      INTEGER JTRMCD,ITNCNT,IRETCD,ITRMCD,ICSCMX                        OPTSTP
      DOUBLE PRECISION FSCALE,GRADTL,STEPTL,FPLS,D,RGX                  OPTSTP
      DOUBLE PRECISION RELGRD,RELSTP,RSX                                OPTSTP
      DOUBLE PRECISION SX(N)                                            OPTSTP
      DOUBLE PRECISION XPLS(N),GPLS(N),X(N)                             OPTSTP
      LOGICAL MXTAKE                                                    OPTSTP
C                                                                       OPTSTP
      ITRMCD=0                                                          OPTSTP
C                                                                       OPTSTP
C LAST GLOBAL STEP FAILED TO LOCATE A POINT LOWER THAN X                OPTSTP
      IF(IRETCD.NE.1) GO TO 50                                          OPTSTP
C     IF(IRETCD.EQ.1)                                                   OPTSTP
C     THEN                                                              OPTSTP
        JTRMCD=3                                                        OPTSTP
        GO TO 600                                                       OPTSTP
C     ENDIF                                                             OPTSTP
   50 CONTINUE                                                          OPTSTP
C                                                                       OPTSTP
C FIND DIRECTION IN WHICH RELATIVE GRADIENT MAXIMUM.                    OPTSTP
C CHECK WHETHER WITHIN TOLERANCE                                        OPTSTP
C                                                                       OPTSTP
      D=MAX(ABS(FPLS),FSCALE)                                           OPTSTP
      RGX=0.0                                                           OPTSTP
      DO 100 I=1,N                                                      OPTSTP
        RELGRD=ABS(GPLS(I))*MAX(ABS(XPLS(I)),1./SX(I))/D                OPTSTP
        RGX=MAX(RGX,RELGRD)                                             OPTSTP
  100 CONTINUE                                                          OPTSTP
      JTRMCD=1                                                          OPTSTP
      IF(RGX.LE.GRADTL) GO TO 600                                       OPTSTP
C                                                                       OPTSTP
      IF(ITNCNT.EQ.0) RETURN                                            OPTSTP
C                                                                       OPTSTP
C FIND DIRECTION IN WHICH RELATIVE STEPSIZE MAXIMUM                     OPTSTP
C CHECK WHETHER WITHIN TOLERANCE.                                       OPTSTP
C                                                                       OPTSTP
      RSX=0.0                                                           OPTSTP
      DO 120 I=1,N                                                      OPTSTP
        RELSTP=ABS(XPLS(I)-X(I))/MAX(ABS(XPLS(I)),1./SX(I))             OPTSTP
        RSX=MAX(RSX,RELSTP)                                             OPTSTP
  120 CONTINUE                                                          OPTSTP
      JTRMCD=2                                                          OPTSTP
      IF(RSX.LE.STEPTL) GO TO 600                                       OPTSTP
C                                                                       OPTSTP
C CHECK ITERATION LIMIT                                                 OPTSTP
C                                                                       OPTSTP
      JTRMCD=4                                                          OPTSTP
      IF(ITNCNT.GE.ITNLIM) GO TO 600                                    OPTSTP
C                                                                       OPTSTP
C CHECK NUMBER OF CONSECUTIVE STEPS \ STEPMX                            OPTSTP
C                                                                       OPTSTP
      IF(MXTAKE) GO TO 140                                              OPTSTP
C     IF(.NOT.MXTAKE)                                                   OPTSTP
C     THEN                                                              OPTSTP
        ICSCMX=0                                                        OPTSTP
        RETURN                                                          OPTSTP
C     ELSE                                                              OPTSTP
  140   CONTINUE                                                        OPTSTP
        IF (MOD(MSG/8,2) .EQ. 0) WRITE(IPR,900)                         OPTSTP
        ICSCMX=ICSCMX+1                                                 OPTSTP
        IF(ICSCMX.LT.5) RETURN                                          OPTSTP
        JTRMCD=5                                                        OPTSTP
C     ENDIF                                                             OPTSTP
C                                                                       OPTSTP
C                                                                       OPTSTP
C PRINT TERMINATION CODE                                                OPTSTP
C                                                                       OPTSTP
  600 ITRMCD=JTRMCD                                                     OPTSTP
      IF (MOD(MSG/8,2) .EQ. 0) GO TO(601,602,603,604,605), ITRMCD       OPTSTP
      GO TO 700                                                         OPTSTP
  601 WRITE(IPR,901)                                                    OPTSTP
      GO TO 700                                                         OPTSTP
  602 WRITE(IPR,902)                                                    OPTSTP
      GO TO 700                                                         OPTSTP
  603 WRITE(IPR,903)                                                    OPTSTP
      GO TO 700                                                         OPTSTP
  604 WRITE(IPR,904)                                                    OPTSTP
      GO TO 700                                                         OPTSTP
  605 WRITE(IPR,905)                                                    OPTSTP
C                                                                       OPTSTP
  700 RETURN                                                            OPTSTP
C                                                                       OPTSTP
  900 FORMAT(48H0OPTSTP    STEP OF MAXIMUM LENGTH (STEPMX) TAKEN)       OPTSTP
  901 FORMAT(43H0OPTSTP    RELATIVE GRADIENT CLOSE TO ZERO./            OPTSTP
     +       48H OPTSTP    CURRENT ITERATE IS PROBABLY SOLUTION.)       OPTSTP
  902 FORMAT(48H0OPTSTP    SUCCESSIVE ITERATES WITHIN TOLERANCE./       OPTSTP
     +       48H OPTSTP    CURRENT ITERATE IS PROBABLY SOLUTION.)       OPTSTP
  903 FORMAT(52H0OPTSTP    LAST GLOBAL STEP FAILED TO LOCATE A POINT,   OPTSTP
     +       14H LOWER THAN X./                                         OPTSTP
     +       51H OPTSTP    EITHER X IS AN APPROXIMATE LOCAL MINIMUM,    OPTSTP
     +       17H OF THE FUNCTION,/                                      OPTSTP
     +       50H OPTSTP    THE FUNCTION IS TOO NON-LINEAR FOR THIS,     OPTSTP
     +       11H ALGORITHM,/                                            OPTSTP
     +       34H OPTSTP    OR STEPTL IS TOO LARGE.)                     OPTSTP
  904 FORMAT(36H0OPTSTP    ITERATION LIMIT EXCEEDED./                   OPTSTP
     +       28H OPTSTP    ALGORITHM FAILED.)                           OPTSTP
  905 FORMAT(39H0OPTSTP    MAXIMUM STEP SIZE EXCEEDED 5,                OPTSTP
     +       19H CONSECUTIVE TIMES./                                    OPTSTP
     +       50H OPTSTP    EITHER THE FUNCTION IS UNBOUNDED BELOW,/     OPTSTP
     +       47H OPTSTP    BECOMES ASYMPTOTIC TO A FINITE VALUE,        OPTSTP
     +       30H FROM ABOVE IN SOME DIRECTION,/                         OPTSTP
     +       33H OPTSTP    OR STEPMX IS TOO SMALL)                      OPTSTP
      END                                                               OPTSTP
      SUBROUTINE RESLT(NR,N,X,F,G,A,P,ITNCNT,IFLG,IPR)                  RESULT
C                                                                       RESULT
C PURPOSE                                                               RESULT
C -------                                                               RESULT
C PRINT INFORMATION                                                     RESULT
C                                                                       RESULT
C PARAMETERS                                                            RESULT
C ----------                                                            RESULT
C NR           --> ROW DIMENSION OF MATRIX                              RESULT
C N            --> DIMENSION OF PROBLEM                                 RESULT
C X(N)         --> ITERATE X[K]                                         RESULT
C F            --> FUNCTION VALUE AT X[K]                               RESULT
C G(N)         --> GRADIENT AT X[K]                                     RESULT
C A(N,N)       --> HESSIAN AT X[K]                                      RESULT
C P(N)         --> STEP TAKEN                                           RESULT
C ITNCNT       --> ITERATION NUMBER K                                   RESULT
C IFLG         --> FLAG CONTROLLING INFO TO PRINT                       RESULT
C IPR          --> DEVICE TO WHICH TO SEND OUTPUT                       RESULT
C                                                                       RESULT
      INTEGER NR,N,ITNCNT,IFLG,IPR,I,J                                  RESULT
      DOUBLE PRECISION F                                                RESULT
      DOUBLE PRECISION X(N),G(N),P(N),A(NR,1)                           RESULT
C PRINT ITERATION NUMBER                                                RESULT
      WRITE(IPR,903) ITNCNT                                             RESULT
      IF(IFLG.EQ.0) GO TO 120                                           RESULT
C                                                                       RESULT
C PRINT STEP                                                            RESULT
      WRITE(IPR,907)                                                    RESULT
      WRITE(IPR,905) (P(I),I=1,N)                                       RESULT
C                                                                       RESULT
C PRINT CURRENT ITERATE                                                 RESULT
  120 CONTINUE                                                          RESULT
      WRITE(IPR,904)                                                    RESULT
      WRITE(IPR,905) (X(I),I=1,N)                                       RESULT
C                                                                       RESULT
C PRINT FUNCTION VALUE                                                  RESULT
      WRITE(IPR,906)                                                    RESULT
      WRITE(IPR,905) F                                                  RESULT
C                                                                       RESULT
C PRINT GRADIENT                                                        RESULT
      WRITE(IPR,908)                                                    RESULT
      WRITE(IPR,905) (G(I),I=1,N)                                       RESULT
C                                                                       RESULT
C PRINT HESSIAN FROM ITERATION K                                        RESULT
      IF(IFLG.EQ.0) GO TO 140                                           RESULT
      WRITE(IPR,901)                                                    RESULT
      DO 130 I=1,N                                                      RESULT
        WRITE(IPR,900) I                                                RESULT
        WRITE(IPR,902) (A(I,J),J=1,I)                                   RESULT
  130 CONTINUE                                                          RESULT
C                                                                       RESULT
  140 RETURN                                                            RESULT
  900 FORMAT(15H RESULT     ROW,I5)                                     RESULT
  901 FORMAT(29H RESULT       HESSIAN AT X(K))                          RESULT
  902 FORMAT(14H RESULT       ,5(2X,E20.13))                            RESULT
  903 FORMAT(22H RESULT     ITERATE K=,I5)                              RESULT
  904 FORMAT(18H RESULT       X(K))                                     RESULT
  905 FORMAT(22H RESULT               ,5(2X,E20.13) )                   RESULT
  906 FORMAT(30H RESULT       FUNCTION AT X(K))                         RESULT
  907 FORMAT(18H RESULT       STEP)                                     RESULT
  908 FORMAT(30H RESULT       GRADIENT AT X(K))                         RESULT
      END                                                               RESULT
      SUBROUTINE SNDOFD(NR,N,XPLS,FCN,FPLS,A,SX,RNOISE,STEPSZ,ANBR)     SNDOFD
C PURPOSE                                                               SNDOFD
C -------                                                               SNDOFD
C FIND SECOND ORDER FORWARD FINITE DIFFERENCE APPROXIMATION "A"         SNDOFD
C TO THE SECOND DERIVATIVE (HESSIAN) OF THE FUNCTION DEFINED BY THE SUBPSNDOFD
C "FCN" EVALUATED AT THE NEW ITERATE "XPLS"                             SNDOFD
C                                                                       SNDOFD
C FOR OPTIMIZATION USE THIS ROUTINE TO ESTIMATE                         SNDOFD
C 1) THE SECOND DERIVATIVE (HESSIAN) OF THE OPTIMIZATION FUNCTION       SNDOFD
C    IF NO ANALYTICAL USER FUNCTION HAS BEEN SUPPLIED FOR EITHER        SNDOFD
C    THE GRADIENT OR THE HESSIAN AND IF THE OPTIMIZATION FUNCTION       SNDOFD
C    "FCN" IS INEXPENSIVE TO EVALUATE.                                  SNDOFD
C                                                                       SNDOFD
C PARAMETERS                                                            SNDOFD
C ----------                                                            SNDOFD
C NR           --> ROW DIMENSION OF MATRIX                              SNDOFD
C N            --> DIMENSION OF PROBLEM                                 SNDOFD
C XPLS(N)      --> NEW ITERATE:   X[K]                                  SNDOFD
C FCN          --> NAME OF SUBROUTINE TO EVALUATE FUNCTION              SNDOFD
C FPLS         --> FUNCTION VALUE AT NEW ITERATE, F(XPLS)               SNDOFD
C A(N,N)      <--  FINITE DIFFERENCE APPROXIMATION TO HESSIAN           SNDOFD
C                  ONLY LOWER TRIANGULAR MATRIX AND DIAGONAL            SNDOFD
C                  ARE RETURNED                                         SNDOFD
C SX(N)        --> DIAGONAL SCALING MATRIX FOR X                        SNDOFD
C RNOISE       --> RELATIVE NOISE IN FNAME [F(X)]                       SNDOFD
C STEPSZ(N)    --> WORKSPACE (STEPSIZE IN I-TH COMPONENT DIRECTION)     SNDOFD
C ANBR(N)      --> WORKSPACE (NEIGHBOR IN I-TH DIRECTION)               SNDOFD
C                                                                       SNDOFD
C                                                                       SNDOFD
      INTEGER NR,N,I,J,IP1                                              SNDOFD
      DOUBLE PRECISION FPLS,RNOISE,OV3,XTMPI,XTMPJ,FHAT                 SNDOFD
      DOUBLE PRECISION XPLS(N)                                          SNDOFD
      DOUBLE PRECISION SX(N)                                            SNDOFD
      DOUBLE PRECISION STEPSZ(N),ANBR(N)                                SNDOFD
      DOUBLE PRECISION A(NR,1)                                          SNDOFD
      EXTERNAL FCN                                                      SNDOFD
C                                                                       SNDOFD
C FIND I-TH STEPSIZE AND EVALUATE NEIGHBOR IN DIRECTION                 SNDOFD
C OF I-TH UNIT VECTOR.                                                  SNDOFD
C                                                                       SNDOFD
      OV3 = 1.0/3.0                                                     SNDOFD
      DO 10 I=1,N                                                       SNDOFD
        STEPSZ(I)=RNOISE**OV3 * MAX(ABS(XPLS(I)),1./SX(I))              SNDOFD
        XTMPI=XPLS(I)                                                   SNDOFD
        XPLS(I)=XTMPI+STEPSZ(I)                                         SNDOFD
        CALL FCN(N,XPLS,ANBR(I))                                        SNDOFD
        XPLS(I)=XTMPI                                                   SNDOFD
   10 CONTINUE                                                          SNDOFD
C                                                                       SNDOFD
C CALCULATE COLUMN I OF A                                               SNDOFD
C                                                                       SNDOFD
      DO 30 I=1,N                                                       SNDOFD
        XTMPI=XPLS(I)                                                   SNDOFD
        XPLS(I)=XTMPI+2.0*STEPSZ(I)                                     SNDOFD
        CALL FCN(N,XPLS,FHAT)                                           SNDOFD
        A(I,I)=((FPLS-ANBR(I))+(FHAT-ANBR(I)))/(STEPSZ(I)*STEPSZ(I))    SNDOFD
C                                                                       SNDOFD
C CALCULATE SUB-DIAGONAL ELEMENTS OF COLUMN                             SNDOFD
        IF(I.EQ.N) GO TO 25                                             SNDOFD
        XPLS(I)=XTMPI+STEPSZ(I)                                         SNDOFD
        IP1=I+1                                                         SNDOFD
        DO 20 J=IP1,N                                                   SNDOFD
          XTMPJ=XPLS(J)                                                 SNDOFD
          XPLS(J)=XTMPJ+STEPSZ(J)                                       SNDOFD
          CALL FCN(N,XPLS,FHAT)                                         SNDOFD
          A(J,I)=((FPLS-ANBR(I))+(FHAT-ANBR(J)))/(STEPSZ(I)*STEPSZ(J))  SNDOFD
          XPLS(J)=XTMPJ                                                 SNDOFD
   20   CONTINUE                                                        SNDOFD
   25   XPLS(I)=XTMPI                                                   SNDOFD
   30 CONTINUE                                                          SNDOFD
      RETURN                                                            SNDOFD
      END                                                               SNDOFD
      SUBROUTINE TREGUP(NR,N,X,F,G,A,FCN,SC,SX,NWTAKE,STEPMX,STEPTL,    TREGUP
     + DLT,IRETCD,XPLSP,FPLSP,XPLS,FPLS,MXTAKE,IPR,METHOD,UDIAG,        TREGUP
     + AJA,ANLS,SHAT,VECT1,VECT2,VECT3,VECT4,VECT5,VECT6,NRM,NRN,MM,    TREGUP
     + NN,IQ)                                                           TREGUP
C                                                                       TREGUP
C PURPOSE                                                               TREGUP
C -------                                                               TREGUP
C DECIDE WHETHER TO ACCEPT XPLS=X+SC AS THE NEXT ITERATE AND UPDATE THE TREGUP
C TRUST REGION DLT.                                                     TREGUP
C                                                                       TREGUP
C PARAMETERS                                                            TREGUP
C ----------                                                            TREGUP
C NR           --> ROW DIMENSION OF MATRIX                              TREGUP
C N            --> DIMENSION OF PROBLEM                                 TREGUP
C X(N)         --> OLD ITERATE X[K-1]                                   TREGUP
C F            --> FUNCTION VALUE AT OLD ITERATE, F(X)                  TREGUP
C G(N)         --> GRADIENT AT OLD ITERATE, G(X), OR APPROXIMATE        TREGUP
C A(N,N)       --> CHOLESKY DECOMPOSITION OF HESSIAN IN                 TREGUP
C                  LOWER TRIANGULAR PART AND DIAGONAL.                  TREGUP
C                  HESSIAN OR APPROX IN UPPER TRIANGULAR PART           TREGUP
C FCN          --> NAME OF SUBROUTINE TO EVALUATE FUNCTION              TREGUP
C SC(N)        --> CURRENT STEP                                         TREGUP
C SX(N)        --> DIAGONAL SCALING MATRIX FOR X                        TREGUP
C NWTAKE       --> BOOLEAN, =.TRUE. IF NEWTON STEP TAKEN                TREGUP
C STEPMX       --> MAXIMUM ALLOWABLE STEP SIZE                          TREGUP
C STEPTL       --> RELATIVE STEP SIZE AT WHICH SUCCESSIVE ITERATES      TREGUP
C                  CONSIDERED CLOSE ENOUGH TO TERMINATE ALGORITHM       TREGUP
C DLT         <--> TRUST REGION RADIUS                                  TREGUP
C IRETCD      <--> RETURN CODE                                          TREGUP
C                    =0 XPLS ACCEPTED AS NEXT ITERATE;                  TREGUP
C                       DLT TRUST REGION FOR NEXT ITERATION.            TREGUP
C                    =1 XPLS UNSATISFACTORY BUT ACCEPTED AS NEXT ITERATETREGUP
C                       BECAUSE XPLS-X .LT. SMALLEST ALLOWABLE          TREGUP
C                       STEP LENGTH.                                    TREGUP
C                    =2 F(XPLS) TOO LARGE.  CONTINUE CURRENT ITERATION  TREGUP
C                       WITH NEW REDUCED DLT.                           TREGUP
C                    =3 F(XPLS) SUFFICIENTLY SMALL, BUT QUADRATIC MODEL TREGUP
C                       PREDICTS F(XPLS) SUFFICIENTLY WELL TO CONTINUE  TREGUP
C                       CURRENT ITERATION WITH NEW DOUBLED DLT.         TREGUP
C XPLSP(N)    <--> WORKSPACE [VALUE NEEDS TO BE RETAINED BETWEEN        TREGUP
C                  SUCCESSIVE CALLS OF K-TH GLOBAL STEP]                TREGUP
C FPLSP       <--> [RETAIN VALUE BETWEEN SUCCESSIVE CALLS]              TREGUP
C XPLS(N)     <--  NEW ITERATE X[K]                                     TREGUP
C FPLS        <--  FUNCTION VALUE AT NEW ITERATE, F(XPLS)               TREGUP
C MXTAKE      <--  BOOLEAN FLAG INDICATING STEP OF MAXIMUM LENGTH USED  TREGUP
C IPR          --> DEVICE TO WHICH TO SEND OUTPUT                       TREGUP
C METHOD       --> ALGORITHM TO USE TO SOLVE MINIMIZATION PROBLEM       TREGUP
C                    =1 LINE SEARCH                                     TREGUP
C                    =2 DOUBLE DOGLEG                                   TREGUP
C                    =3 MORE-HEBDON                                     TREGUP
C UDIAG(N)     --> DIAGONAL OF HESSIAN IN A(.,.)                        TREGUP
C                                                                       TREGUP
      INTEGER NR,N,IRETCD,IPR,METHOD,NRM,NRN,MM,NN,IQ,I,J,IP1           TREGUP
      DOUBLE PRECISION STEPMX,STEPTL,DLT,FPLSP,FPLS,SLP,RLN             TREGUP
      DOUBLE PRECISION DLTMP,DLTFP,TEMP,DLTF,F                          TREGUP
      DOUBLE PRECISION X(N),XPLS(N),G(N)                                TREGUP
      DOUBLE PRECISION SX(N),SC(N),XPLSP(N)                             TREGUP
      DOUBLE PRECISION A(NR,1)                                          TREGUP
      DOUBLE PRECISION  UDIAG(N)                                        TREGUP
      DOUBLE PRECISION AJA(NRM,1),ANLS(NRM,1),SHAT(NRN,1),VECT1(MM)     TREGUP
      DOUBLE PRECISION VECT2(MM),VECT3(MM),VECT4(MM),VECT5(MM)          TREGUP  
      DOUBLE PRECISION VECT6(MM)                                        TREGUP
      DOUBLE PRECISION DDOT                                             TREGUP
      LOGICAL NWTAKE,MXTAKE                                             TREGUP
      EXTERNAL FCN                                                      TREGUP
C                                                                       TREGUP
      MXTAKE=.FALSE.                                                    TREGUP
      DO 100 I=1,N                                                      TREGUP
        XPLS(I)=X(I)+SC(I)                                              TREGUP
  100 CONTINUE                                                          TREGUP
      CALL FCN(N,XPLS,FPLS,AJA,ANLS,SHAT,VECT1,VECT2,VECT3,             TREGUP
     +   VECT4,VECT5,VECT6,NRM,NRN,MM,NN,IQ)                            TREGUP
      DLTF=FPLS-F                                                       TREGUP
      SLP=DDOT(N,G,1,SC,1)                                              TREGUP
C                                                                       TREGUP
C NEXT STATEMENT ADDED FOR CASE OF COMPILERS WHICH DO NOT OPTIMIZE      TREGUP
C EVALUATION OF NEXT "IF" STATEMENT (IN WHICH CASE FPLSP COULD BE       TREGUP
C UNDEFINED).                                                           TREGUP
      IF(IRETCD.EQ.4) FPLSP=0.0                                         TREGUP
C$    WRITE(IPR,961) IRETCD,FPLS,FPLSP,DLTF,SLP                         TREGUP
      IF(IRETCD.NE.3 .OR. (FPLS.LT.FPLSP .AND. DLTF.LE. 1.E-4*SLP))     TREGUP
     +                                                     GO TO 130    TREGUP
C     IF(IRETCD.EQ.3 .AND. (FPLS.GE.FPLSP .OR. DLTF.GT. 1.E-4*SLP))     TREGUP
C     THEN                                                              TREGUP
C                                                                       TREGUP
C       RESET XPLS TO XPLSP AND TERMINATE GLOBAL STEP                   TREGUP
C                                                                       TREGUP
        IRETCD=0                                                        TREGUP
        DO 110 I=1,N                                                    TREGUP
          XPLS(I)=XPLSP(I)                                              TREGUP
  110   CONTINUE                                                        TREGUP
        FPLS=FPLSP                                                      TREGUP
        DLT=.5*DLT                                                      TREGUP
C$      WRITE(IPR,951)                                                  TREGUP
        GO TO 230                                                       TREGUP
C     ELSE                                                              TREGUP
C                                                                       TREGUP
C       FPLS TOO LARGE                                                  TREGUP
C                                                                       TREGUP
  130   IF(DLTF.LE. 1.E-4*SLP) GO TO 170                                TREGUP
C       IF(DLTF.GT. 1.E-4*SLP)                                          TREGUP
C       THEN                                                            TREGUP
C$        WRITE(IPR,952)                                                TREGUP
          RLN=0.                                                        TREGUP
          DO 140 I=1,N                                                  TREGUP
            RLN=MAX(RLN,ABS(SC(I))/MAX(ABS(XPLS(I)),1./SX(I)))          TREGUP
  140     CONTINUE                                                      TREGUP
C$        WRITE(IPR,962) RLN                                            TREGUP
          IF(RLN.GE.STEPTL) GO TO 150                                   TREGUP
C         IF(RLN.LT.STEPTL)                                             TREGUP
C         THEN                                                          TREGUP
C                                                                       TREGUP
C           CANNOT FIND SATISFACTORY XPLS SUFFICIENTLY DISTINCT FROM X  TREGUP
C                                                                       TREGUP
            IRETCD=1                                                    TREGUP
C$          WRITE(IPR,954)                                              TREGUP
            GO TO 230                                                   TREGUP
C         ELSE                                                          TREGUP
C                                                                       TREGUP
C           REDUCE TRUST REGION AND CONTINUE GLOBAL STEP                TREGUP
C                                                                       TREGUP
  150       IRETCD=2                                                    TREGUP
            DLTMP=-SLP*DLT/(2.*(DLTF-SLP))                              TREGUP
C$          WRITE(IPR,963) DLTMP                                        TREGUP
            IF(DLTMP.GE. .1*DLT) GO TO 155                              TREGUP
C           IF(DLTMP.LT. .1*DLT)                                        TREGUP
C           THEN                                                        TREGUP
              DLT=.1*DLT                                                TREGUP
              GO TO 160                                                 TREGUP
C           ELSE                                                        TREGUP
  155         DLT=DLTMP                                                 TREGUP
C           ENDIF                                                       TREGUP
  160       CONTINUE                                                    TREGUP
C$          WRITE(IPR,955)                                              TREGUP
            GO TO 230                                                   TREGUP
C         ENDIF                                                         TREGUP
C       ELSE                                                            TREGUP
C                                                                       TREGUP
C         FPLS SUFFICIENTLY SMALL                                       TREGUP
C                                                                       TREGUP
  170     CONTINUE                                                      TREGUP
C$        WRITE(IPR,958)                                                TREGUP
          DLTFP=0.                                                      TREGUP
          IF (METHOD .EQ. 3) GO TO 180                                  TREGUP
C                                                                       TREGUP
C         IF (METHOD .EQ. 2)                                            TREGUP
C         THEN                                                          TREGUP
C                                                                       TREGUP
          DO 177 I = 1, N                                               TREGUP
             TEMP = 0.0                                                 TREGUP
             DO 173 J = I, N                                            TREGUP
                TEMP = TEMP + (A(J, I)*SC(J))                           TREGUP
  173        CONTINUE                                                   TREGUP
             DLTFP = DLTFP + TEMP*TEMP                                  TREGUP
  177     CONTINUE                                                      TREGUP
          GO TO 190                                                     TREGUP
C                                                                       TREGUP
C         ELSE                                                          TREGUP
C                                                                       TREGUP
  180     DO 187 I = 1, N                                               TREGUP
             DLTFP = DLTFP + UDIAG(I)*SC(I)*SC(I)                       TREGUP
             IF (I .EQ. N) GO TO 187                                    TREGUP
             TEMP = 0                                                   TREGUP
             IP1 = I + 1                                                TREGUP
             DO 183 J = IP1, N                                          TREGUP
                TEMP = TEMP + A(I, J)*SC(I)*SC(J)                       TREGUP
  183        CONTINUE                                                   TREGUP
             DLTFP = DLTFP + 2.0*TEMP                                   TREGUP
  187     CONTINUE                                                      TREGUP
C                                                                       TREGUP
C         END IF                                                        TREGUP
C                                                                       TREGUP
  190     DLTFP = SLP + DLTFP/2.0                                       TREGUP
C$        WRITE(IPR,964) DLTFP,NWTAKE                                   TREGUP
          IF(IRETCD.EQ.2 .OR. (ABS(DLTFP-DLTF).GT. .1*ABS(DLTF))        TREGUP
     +         .OR. NWTAKE .OR. (DLT.GT. .99*STEPMX)) GO TO 210         TREGUP
C         IF(IRETCD.NE.2 .AND. (ABS(DLTFP-DLTF) .LE. .1*ABS(DLTF))      TREGUP
C    +         .AND. (.NOT.NWTAKE) .AND. (DLT.LE. .99*STEPMX))          TREGUP
C         THEN                                                          TREGUP
C                                                                       TREGUP
C           DOUBLE TRUST REGION AND CONTINUE GLOBAL STEP                TREGUP
C                                                                       TREGUP
            IRETCD=3                                                    TREGUP
            DO 200 I=1,N                                                TREGUP
              XPLSP(I)=XPLS(I)                                          TREGUP
  200       CONTINUE                                                    TREGUP
            FPLSP=FPLS                                                  TREGUP
            DLT=MIN(2.0D0*DLT,STEPMX)                                   TREGUP
C$          WRITE(IPR,959)                                              TREGUP
            GO TO 230                                                   TREGUP
C         ELSE                                                          TREGUP
C                                                                       TREGUP
C           ACCEPT XPLS AS NEXT ITERATE.  CHOOSE NEW TRUST REGION.      TREGUP
C                                                                       TREGUP
  210       CONTINUE                                                    TREGUP
C$          WRITE(IPR,960)                                              TREGUP
            IRETCD=0                                                    TREGUP
            IF(DLT.GT. .99*STEPMX) MXTAKE=.TRUE.                        TREGUP
            IF(DLTF.LT. .1*DLTFP) GO TO 220                             TREGUP
C           IF(DLTF.GE. .1*DLTFP)                                       TREGUP
C           THEN                                                        TREGUP
C                                                                       TREGUP
C             DECREASE TRUST REGION FOR NEXT ITERATION                  TREGUP
C                                                                       TREGUP
              DLT=.5*DLT                                                TREGUP
              GO TO 230                                                 TREGUP
C           ELSE                                                        TREGUP
C                                                                       TREGUP
C             CHECK WHETHER TO INCREASE TRUST REGION FOR NEXT ITERATION TREGUP
C                                                                       TREGUP
  220         IF(DLTF.LE. .75*DLTFP) DLT=MIN(2.*DLT,STEPMX)             TREGUP
C           ENDIF                                                       TREGUP
C         ENDIF                                                         TREGUP
C       ENDIF                                                           TREGUP
C     ENDIF                                                             TREGUP
  230 CONTINUE                                                          TREGUP
C$    WRITE(IPR,953)                                                    TREGUP
C$    WRITE(IPR,956) IRETCD,MXTAKE,DLT,FPLS                             TREGUP
C$    WRITE(IPR,957)                                                    TREGUP
C$    WRITE(IPR,965) (XPLS(I),I=1,N)                                    TREGUP
      RETURN                                                            TREGUP
C                                                                       TREGUP
  951 FORMAT(55H TREGUP    RESET XPLS TO XPLSP. TERMINATION GLOBAL STEP)TREGUP
  952 FORMAT(26H TREGUP    FPLS TOO LARGE.)                             TREGUP
  953 FORMAT(38H0TREGUP    VALUES AFTER CALL TO TREGUP)                 TREGUP
  954 FORMAT(54H TREGUP    CANNOT FIND SATISFACTORY XPLS DISTINCT FROM, TREGUP
     +       27H X.  TERMINATE GLOBAL STEP.)                            TREGUP
  955 FORMAT(53H TREGUP    REDUCE TRUST REGION. CONTINUE GLOBAL STEP.)  TREGUP
  956 FORMAT(21H TREGUP       IRETCD=,I3/                               TREGUP
     +       21H TREGUP       MXTAKE=,L1/                               TREGUP
     +       21H TREGUP       DLT   =,E20.13/                           TREGUP
     +       21H TREGUP       FPLS  =,E20.13)                           TREGUP
  957 FORMAT(32H TREGUP       NEW ITERATE (XPLS))                       TREGUP
  958 FORMAT(35H TREGUP    FPLS SUFFICIENTLY SMALL.)                    TREGUP
  959 FORMAT(54H TREGUP    DOUBLE TRUST REGION.  CONTINUE GLOBAL STEP.) TREGUP
  960 FORMAT(50H TREGUP    ACCEPT XPLS AS NEW ITERATE.  CHOOSE NEW,     TREGUP
     +       38H TRUST REGION.  TERMINATE GLOBAL STEP.)                 TREGUP
  961 FORMAT(18H TREGUP    IRETCD=,I5/                                  TREGUP
     +       18H TREGUP    FPLS  =,E20.13/                              TREGUP
     +       18H TREGUP    FPLSP =,E20.13/                              TREGUP
     +       18H TREGUP    DLTF  =,E20.13/                              TREGUP
     +       18H TREGUP    SLP   =,E20.13)                              TREGUP
  962 FORMAT(18H TREGUP    RLN   =,E20.13)                              TREGUP
  963 FORMAT(18H TREGUP    DLTMP =,E20.13)                              TREGUP
  964 FORMAT(18H TREGUP    DLTFP =,E20.13/                              TREGUP
     +       18H TREGUP    NWTAKE=,L1)                                  TREGUP
  965 FORMAT(14H TREGUP       ,5(E20.13,3X))                            TREGUP
      END                                                               TREGUP

      DOUBLE PRECISION FUNCTION DPMEPS()
C     **********
C
C     SUBROUTINE DPMEPS
C
C     THIS SUBROUTINE COMPUTES THE MACHINE PRECISION PARAMETER
C     DPMEPS AS THE SMALLEST FLOATING POINT NUMBER SUCH THAT
C     1 + DPMEPS DIFFERS FROM 1.
C
C     THIS SUBROUTINE IS BASED ON THE SUBROUTINE MACHAR DESCRIBED IN
C
C     W. J. CODY, MACHAR: A SUBROUTINE TO DYNAMICALLY DETERMINE
C     MACHINE PARAMETERS, ACM TRANS. MATH. SOFTWARE 14 (1988), 303-311.
C
C     THE FUNCTION STATEMENT IS
C
C       DOUBLE PRECISION DPMEPS()
C
C     MINPACK-2 PROJECT. FEBRUARY 1991.
C     ARGONNE NATIONAL LABORATORY AND UNIVERSITY OF MINNESOTA.
C     BRETT M. AVERICK.
C
C     **********
      INTEGER I, IBETA, IRND, IT, ITEMP, NEGEP
      DOUBLE PRECISION A, B, BETA, BETAIN, BETAH, TEMP, TEMPA, TEMP1
      DOUBLE PRECISION ZERO, ONE, TWO
      DATA ZERO, ONE, TWO/0.0D0, 1.0D0, 2.0D0/

C     DETERMINE IBETA, BETA ALA MALCOLM.

      A = ONE
      B = ONE
   10 CONTINUE
      A = A + A
      TEMP = A + ONE
      TEMP1 = TEMP - A
      IF (TEMP1-ONE .EQ. ZERO) GO TO 10
   20 CONTINUE
      B = B + B
      TEMP = A + B
      ITEMP = INT(TEMP-A)
      IF (ITEMP .EQ. 0) GO TO 20
      IBETA = ITEMP
      BETA = DBLE(IBETA)

C     DETERMINE IT, IRND.

      IT = 0
      B = ONE
   30 CONTINUE
      IT = IT + 1
      B = B*BETA
      TEMP = B + ONE
      TEMP1 = TEMP - B
      IF (TEMP1-ONE .EQ. ZERO) GO TO 30
      IRND = 0
      BETAH = BETA/TWO
      TEMP = A + BETAH
      IF (TEMP-A .NE. ZERO) IRND = 1
      TEMPA = A + BETA
      TEMP = TEMPA + BETAH
      IF ((IRND .EQ. 0) .AND. (TEMP-TEMPA .NE. ZERO)) IRND = 2

C     DETERMINE DPMEPS.

      NEGEP = IT + 3
      BETAIN = ONE/BETA
      A = ONE
      DO 40 I = 1, NEGEP
         A = A*BETAIN
   40 CONTINUE
   50 CONTINUE
      TEMP = ONE + A
      IF (TEMP-ONE .NE. ZERO) GO TO 60
      A = A*BETA
      GO TO 50
   60 CONTINUE
      DPMEPS = A
      IF ((IBETA .EQ. 2) .OR. (IRND .EQ. 0)) GO TO 70
      A = (A*(ONE+A))/TWO
      TEMP = ONE + A
      IF (TEMP-ONE .NE. ZERO) DPMEPS = A

   70 CONTINUE

      END

